@@ -31,19 +31,22 @@
 
 #include "aidata.h"
 
-static void ai_diplomacy_new(const struct player *plr1,
-                             const struct player *plr2);
-static void ai_diplomacy_defaults(const struct player *plr1,
+static void dai_diplomacy_new(struct ai_type *ait,
+                              const struct player *plr1,
+                              const struct player *plr2);
+static void dai_diplomacy_defaults(struct ai_type *ait,
+                                   const struct player *plr1,
+                                   const struct player *plr2);
+static void dai_diplomacy_destroy(struct ai_type *ait,
+                                  const struct player *plr1,
                                   const struct player *plr2);
-static void ai_diplomacy_destroy(const struct player *plr1,
-                                 const struct player *plr2);
 
 /****************************************************************************
   Initialize ai data structure
 ****************************************************************************/
-void ai_data_init(struct player *pplayer)
+void dai_data_init(struct ai_type *ait, struct player *pplayer)
 {
-  struct ai_plr *ai = def_ai_player_data(pplayer);
+  struct ai_plr *ai = def_ai_player_data(pplayer, ait);
 
   ai->phase_initialized = FALSE;
 
@@ -63,12 +66,12 @@ void ai_data_init(struct player *pplayer)
 
   players_iterate(aplayer) {
     /* create ai diplomacy states for all other players */
-    ai_diplomacy_new(pplayer, aplayer);
-    ai_diplomacy_defaults(pplayer, aplayer);
+    dai_diplomacy_new(ait, pplayer, aplayer);
+    dai_diplomacy_defaults(ait, pplayer, aplayer);
     /* create ai diplomacy state of this player */
     if (aplayer != pplayer) {
-      ai_diplomacy_new(aplayer, pplayer);
-      ai_diplomacy_defaults(aplayer, pplayer);
+      dai_diplomacy_new(ait, aplayer, pplayer);
+      dai_diplomacy_defaults(ait, aplayer, pplayer);
     }
   } players_iterate_end;
 
@@ -82,26 +85,26 @@ void ai_data_init(struct player *pplayer)
   ai->settler = NULL;
 
   /* Initialise autosettler. */
-  ai_auto_settler_init(ai);
+  dai_auto_settler_init(ai);
 }
 
 /****************************************************************************
   Deinitialize ai data structure
 ****************************************************************************/
-void ai_data_close(struct player *pplayer)
+void dai_data_close(struct ai_type *ait, struct player *pplayer)
 {
-  struct ai_plr *ai = def_ai_player_data(pplayer);
+  struct ai_plr *ai = def_ai_player_data(pplayer, ait);
 
   /* Free autosettler. */
-  ai_auto_settler_free(ai);
+  dai_auto_settler_free(ai);
 
   if (ai->diplomacy.player_intel_slots != NULL) {
     players_iterate(aplayer) {
       /* destroy the ai diplomacy states of this player with others ... */
-      ai_diplomacy_destroy(pplayer, aplayer);
+      dai_diplomacy_destroy(ait, pplayer, aplayer);
       /* and of others with this player. */
       if (aplayer != pplayer) {
-        ai_diplomacy_destroy(aplayer, pplayer);
+        dai_diplomacy_destroy(ait, aplayer, pplayer);
       }
     } players_iterate_end;
     free(ai->diplomacy.player_intel_slots);
@@ -110,11 +113,11 @@ void ai_data_close(struct player *pplayer)
 
 /**************************************************************************
   Return whether data phase is currently open. Data phase is open
-  between ai_data_phase_begin() and ai_data_phase_finished() calls.
+  between dai_data_phase_begin() and dai_data_phase_finished() calls.
 **************************************************************************/
-bool is_ai_data_phase_open(struct player *pplayer)
+bool is_ai_data_phase_open(struct ai_type *ait, struct player *pplayer)
 {
-  struct ai_plr *ai = def_ai_player_data(pplayer);
+  struct ai_plr *ai = def_ai_player_data(pplayer, ait);
 
   return ai->phase_initialized;
 }
@@ -122,9 +125,10 @@ bool is_ai_data_phase_open(struct player *pplayer)
 /****************************************************************************
   Make and cache lots of calculations needed for other functions.
 ****************************************************************************/
-void dai_data_phase_begin(struct player *pplayer, bool is_new_phase)
+void dai_data_phase_begin(struct ai_type *ait, struct player *pplayer,
+                          bool is_new_phase)
 {
-  struct ai_plr *ai = def_ai_player_data(pplayer);
+  struct ai_plr *ai = def_ai_player_data(pplayer, ait);
   bool close;
 
   /* Note that this refreshes advisor data if needed. ai_plr_data_get()
@@ -150,14 +154,14 @@ void dai_data_phase_begin(struct player *pplayer, bool is_new_phase)
 
   /*** Diplomacy ***/
   if (pplayer->ai_controlled && !is_barbarian(pplayer) && is_new_phase) {
-    ai_diplomacy_begin_new_phase(pplayer);
+    dai_diplomacy_begin_new_phase(ait, pplayer);
   }
 
   /* Set per-player variables. We must set all players, since players
    * can be created during a turn, and we don't want those to have
    * invalid values. */
   players_iterate(aplayer) {
-    struct ai_dip_intel *adip = ai_diplomacy_get(pplayer, aplayer);
+    struct ai_dip_intel *adip = dai_diplomacy_get(ait, pplayer, aplayer);
 
     adip->is_allied_with_enemy = NULL;
     adip->at_war_with_ally = NULL;
@@ -241,18 +245,23 @@ void dai_data_phase_begin(struct player *pplayer, bool is_new_phase)
 
   BV_CLR_ALL(ai->stats.diplomat_reservations);
   unit_list_iterate(pplayer->units, punit) {
-    if (unit_has_type_flag(punit, F_DIPLOMAT)
-        && def_ai_unit_data(punit)->task == AIUNIT_ATTACK) {
-      /* Heading somewhere on a mission, reserve target. */
-      struct city *pcity = tile_city(punit->goto_tile);
+    if (unit_has_type_flag(punit, UTYF_DIPLOMAT)
+        && def_ai_unit_data(punit, ait)->task == AIUNIT_ATTACK) {
 
-      if (pcity) {
-        BV_SET(ai->stats.diplomat_reservations, pcity->id);
+      fc_assert_msg(punit->goto_tile != NULL, "No target city for spy action");
+
+      if (punit->goto_tile != NULL) {
+        struct city *pcity = tile_city(punit->goto_tile);
+
+        if (pcity != NULL) {
+          /* Heading somewhere on a mission, reserve target. */
+          BV_SET(ai->stats.diplomat_reservations, pcity->id);
+        }
       }
     }
   } unit_list_iterate_end;
 
-  aiferry_init_stats(pplayer);
+  aiferry_init_stats(ait, pplayer);
 
   /*** Interception engine ***/
 
@@ -265,7 +274,7 @@ void dai_data_phase_begin(struct player *pplayer, bool is_new_phase)
       continue;
     }
     unit_list_iterate(aplayer->units, punit) {
-      struct unit_ai *unit_data = def_ai_unit_data(punit);
+      struct unit_ai *unit_data = def_ai_unit_data(punit, ait);
 
       if (!unit_data->cur_pos) {
         /* Start tracking */
@@ -287,9 +296,9 @@ void dai_data_phase_begin(struct player *pplayer, bool is_new_phase)
 /****************************************************************************
   Clean up ai data after phase finished.
 ****************************************************************************/
-void dai_data_phase_finished(struct player *pplayer)
+void dai_data_phase_finished(struct ai_type *ait, struct player *pplayer)
 {
-  struct ai_plr *ai = def_ai_player_data(pplayer);
+  struct ai_plr *ai = def_ai_player_data(pplayer, ait);
 
   if (!ai->phase_initialized) {
     return;
@@ -307,16 +316,17 @@ void dai_data_phase_finished(struct player *pplayer)
   and the boolean will be set accordingly to tell caller that phase needs
   closing.
 ****************************************************************************/
-struct ai_plr *ai_plr_data_get(struct player *pplayer, bool *close)
+struct ai_plr *dai_plr_data_get(struct ai_type *ait, struct player *pplayer,
+                                bool *close)
 {
-  struct ai_plr *ai = def_ai_player_data(pplayer);
+  struct ai_plr *ai = def_ai_player_data(pplayer, ait);
 
   fc_assert_ret_val(ai != NULL, NULL);
 
   /* This assert really is required. See longer comment
      in adv_data_get() for equivalent code. */
 #if defined(DEBUG) || defined(IS_DEVEL_VERSION)
-  fc_assert(close != NULL || ai->phase_initialized || game.info.phase_mode != PMT_CONCURRENT);
+  fc_assert(close != NULL || ai->phase_initialized);
 #endif
 
   if (close != NULL) {
@@ -329,22 +339,22 @@ struct ai_plr *ai_plr_data_get(struct player *pplayer, bool *close)
 
     /* See adv_data_get() */
     if (ai->phase_initialized) {
-      dai_data_phase_finished(pplayer);
-      dai_data_phase_begin(pplayer, FALSE);
+      dai_data_phase_finished(ait, pplayer);
+      dai_data_phase_begin(ait, pplayer, FALSE);
     } else {
       /* wrong order */
       log_debug("%s ai data phase closed when dai_plr_data_get() called",
                 player_name(pplayer));
-      dai_data_phase_begin(pplayer, FALSE);
+      dai_data_phase_begin(ait, pplayer, FALSE);
       if (close != NULL) {
         *close = TRUE;
       } else {
-        dai_data_phase_finished(pplayer);
+        dai_data_phase_finished(ait, pplayer);
       }
     }
   } else {
     if (!ai->phase_initialized && close != NULL) {
-      dai_data_phase_begin(pplayer, FALSE);
+      dai_data_phase_begin(ait, pplayer, FALSE);
       *close = TRUE;
     }
   }
@@ -356,9 +366,10 @@ struct ai_plr *ai_plr_data_get(struct player *pplayer, bool *close)
   Is there a channel going from ocean c1 to ocean c2?
   Returns FALSE if either is not an ocean.
 **************************************************************************/
-bool ai_channel(struct player *pplayer, Continent_id c1, Continent_id c2)
+bool dai_channel(struct ai_type *ait, struct player *pplayer,
+                 Continent_id c1, Continent_id c2)
 {
-  struct ai_plr *ai = ai_plr_data_get(pplayer, NULL);
+  struct ai_plr *ai = dai_plr_data_get(ait, pplayer, NULL);
   struct adv_data *adv = adv_data_get(pplayer, NULL);
 
   if (c1 >= 0 || c2 >= 0) {
@@ -371,8 +382,9 @@ bool ai_channel(struct player *pplayer, Continent_id c1, Continent_id c2)
 /****************************************************************************
   Allocate new ai diplomacy slot
 ****************************************************************************/
-static void ai_diplomacy_new(const struct player *plr1,
-                             const struct player *plr2)
+static void dai_diplomacy_new(struct ai_type *ait,
+                              const struct player *plr1,
+                              const struct player *plr2)
 {
   struct ai_dip_intel *player_intel;
 
@@ -380,7 +392,7 @@ static void ai_diplomacy_new(const struct player *plr1,
   fc_assert_ret(plr2 != NULL);
 
   const struct ai_dip_intel **player_intel_slot
-    = def_ai_player_data(plr1)->diplomacy.player_intel_slots
+    = def_ai_player_data(plr1, ait)->diplomacy.player_intel_slots
       + player_index(plr2);
 
   fc_assert_ret(*player_intel_slot == NULL);
@@ -392,10 +404,11 @@ static void ai_diplomacy_new(const struct player *plr1,
 /****************************************************************************
   Set diplomacy data between two players to its default values.
 ****************************************************************************/
-static void ai_diplomacy_defaults(const struct player *plr1,
-                                  const struct player *plr2)
+static void dai_diplomacy_defaults(struct ai_type *ait,
+                                   const struct player *plr1,
+                                   const struct player *plr2)
 {
-  struct ai_dip_intel *player_intel = ai_diplomacy_get(plr1, plr2);
+  struct ai_dip_intel *player_intel = dai_diplomacy_get(ait, plr1, plr2);
 
   fc_assert_ret(player_intel != NULL);
 
@@ -414,14 +427,15 @@ static void ai_diplomacy_defaults(const struct player *plr1,
 /***************************************************************
   Returns diplomatic state type between two players
 ***************************************************************/
-struct ai_dip_intel *ai_diplomacy_get(const struct player *plr1,
-                                      const struct player *plr2)
+struct ai_dip_intel *dai_diplomacy_get(struct ai_type *ait,
+                                       const struct player *plr1,
+                                       const struct player *plr2)
 {
   fc_assert_ret_val(plr1 != NULL, NULL);
   fc_assert_ret_val(plr2 != NULL, NULL);
 
   const struct ai_dip_intel **player_intel_slot
-    = def_ai_player_data(plr1)->diplomacy.player_intel_slots
+    = def_ai_player_data(plr1, ait)->diplomacy.player_intel_slots
       + player_index(plr2);
 
   fc_assert_ret_val(player_intel_slot != NULL, NULL);
@@ -432,18 +446,19 @@ struct ai_dip_intel *ai_diplomacy_get(const struct player *plr1,
 /****************************************************************************
   Free resources allocated for diplomacy information between two players.
 ****************************************************************************/
-static void ai_diplomacy_destroy(const struct player *plr1,
-                                 const struct player *plr2)
+static void dai_diplomacy_destroy(struct ai_type *ait,
+                                  const struct player *plr1,
+                                  const struct player *plr2)
 {
   fc_assert_ret(plr1 != NULL);
   fc_assert_ret(plr2 != NULL);
 
   const struct ai_dip_intel **player_intel_slot
-    = def_ai_player_data(plr1)->diplomacy.player_intel_slots
+    = def_ai_player_data(plr1, ait)->diplomacy.player_intel_slots
       + player_index(plr2);
 
   if (*player_intel_slot != NULL) {
-    free(ai_diplomacy_get(plr1, plr2));
+    free(dai_diplomacy_get(ait, plr1, plr2));
   }
 
   *player_intel_slot = NULL;
