@@ -58,7 +58,7 @@
  * This one is used only by ferryboats in ai.passenger field 
  */
 #define FERRY_AVAILABLE     (-1)  /* Boat is looking for a passenger */
-#define FERRY_ABANDON_BOSS  0     /* Passenger is assigned for boat, but boat
+#define FERRY_ABANDON_BOSS  (-2)  /* Passenger is assigned for boat, but boat
                                    * might take another passenger. Probably
                                    * passenger already left the boat*/
 
@@ -86,22 +86,22 @@
 /**************************************************************************
   Call to initialize the ferryboat statistics
 **************************************************************************/
-void aiferry_init_stats(struct player *pplayer)
+void aiferry_init_stats(struct ai_type *ait, struct player *pplayer)
 {
-  /* def_ai_player_data() instead of ai_plr_data_get() is deliberate.
-     We are only initializing player data structures and ai_plre_data_get()
+  /* def_ai_player_data() instead of dai_plr_data_get() is deliberate.
+     We are only initializing player data structures and dai_plr_data_get()
      would try to use it uninitialized. We are only setting values to
      data structure, not reading them, so we have no need for extra
-     arrangements ai_plr_data_get() would do compared to def_ai_player_data()
+     arrangements dai_plr_data_get() would do compared to def_ai_player_data()
   */
-  struct ai_plr *ai = def_ai_player_data(pplayer);
+  struct ai_plr *ai = def_ai_player_data(pplayer, ait);
 
   ai->stats.passengers = 0;
   ai->stats.boats = 0;
   ai->stats.available_boats = 0;
  
   unit_list_iterate(pplayer->units, punit) {
-    struct unit_ai *unit_data = def_ai_unit_data(punit);
+    struct unit_ai *unit_data = def_ai_unit_data(punit, ait);
 
     if (dai_is_ferry(punit)) {
       ai->stats.boats++;
@@ -109,7 +109,6 @@ void aiferry_init_stats(struct player *pplayer)
         ai->stats.available_boats++;
       }
     }
-
     if (unit_data->ferryboat == FERRY_WANTED) {
       UNIT_LOG(LOG_DEBUG, punit, "wants a boat.");
       ai->stats.passengers++;
@@ -121,9 +120,9 @@ void aiferry_init_stats(struct player *pplayer)
   Print the list of boats of pplayer.
 **************************************************************************/
 #ifdef LOGLEVEL_FERRY_STATS
-static void aiferry_print_stats(struct player *pplayer)
+static void aiferry_print_stats(struct ai_type *ait, struct player *pplayer)
 {
-  struct ai_plr *ai = ai_plr_data_get(pplayer, NULL);
+  struct ai_plr *ai = dai_plr_data_get(ait, pplayer, NULL);
   int n = 1;
 
   log_base(LOGLEVEL_FERRY_STATS, "Boat stats for %s[%d]",
@@ -132,10 +131,9 @@ static void aiferry_print_stats(struct player *pplayer)
            ai->stats.available_boats, ai->stats.boats);
   unit_list_iterate(pplayer->units, punit) {
     if (dai_is_ferry(punit)) {
-      /* Can transport some land units. */
       log_base(LOGLEVEL_FERRY_STATS, "#%d. %s[%d], psngr=%d", n,
                unit_rule_name(punit), punit->id,
-               def_ai_unit_data(punit)->passenger);
+               def_ai_unit_data(punit, ait)->passenger);
       n++;
     }
   } unit_list_iterate_end;
@@ -143,17 +141,17 @@ static void aiferry_print_stats(struct player *pplayer)
 #endif /* LOGLEVEL_FERRY_STATS */
 
 /**************************************************************************
-  Should unit be considered a ferry?
+  Should unit type be considered a ferry?
 **************************************************************************/
-bool dai_is_ferry(struct unit *pferry)
+static bool dai_is_ferry_type(struct unit_type *pferry)
 {
-  if (get_transporter_capacity(pferry) > 0
-      && uclass_move_type(unit_class(pferry)) != UMT_LAND) {
+  if (pferry->transport_capacity > 0
+      && dai_uclass_move_type(utype_class(pferry)) != UMT_LAND) {
     unit_class_iterate(pclass) {
-      enum unit_move_type mt = uclass_move_type(pclass);
+      enum unit_move_type mt = dai_uclass_move_type(pclass);
 
       if (mt == UMT_LAND
-          && can_unit_type_transport(unit_type(pferry), pclass)) {
+          && can_unit_type_transport(pferry, pclass)) {
         /* Can transport some land moving unit. */
         return TRUE;
       }
@@ -163,14 +161,22 @@ bool dai_is_ferry(struct unit *pferry)
   return FALSE;
 }
 
+/**************************************************************************
+  Should unit be considered a ferry?
+**************************************************************************/
+bool dai_is_ferry(struct unit *pferry)
+{
+  return dai_is_ferry_type(unit_type(pferry));
+}
+
 /**************************************************************************
   Initialize new ferry when player gets it
 **************************************************************************/
-void dai_ferry_init_ferry(struct unit *ferry)
+void dai_ferry_init_ferry(struct ai_type *ait, struct unit *ferry)
 {
   if (dai_is_ferry(ferry)) {
-    struct unit_ai *unit_data = def_ai_unit_data(ferry);
-    struct ai_plr *ai = ai_plr_data_get(unit_owner(ferry), NULL);
+    struct unit_ai *unit_data = def_ai_unit_data(ferry, ait);
+    struct ai_plr *ai = dai_plr_data_get(ait, unit_owner(ferry), NULL);
 
     unit_data->passenger = FERRY_AVAILABLE;
     ai->stats.boats++;
@@ -179,14 +185,63 @@ void dai_ferry_init_ferry(struct unit *ferry)
 }
 
 /**************************************************************************
-  Player lost unit. Clear it from ferry system
+  Update ferry system when unit is transformed.
 **************************************************************************/
-void dai_unit_lost(struct unit *punit)
+void dai_ferry_transformed(struct ai_type *ait, struct unit *ferry,
+                           struct unit_type *old)
+{
+  bool old_f = dai_is_ferry_type(old);
+  bool new_f = dai_is_ferry(ferry);
+
+  if (old_f && !new_f) {
+    struct ai_plr *ai = dai_plr_data_get(ait, unit_owner(ferry), NULL);
+    struct unit_ai *unit_data = def_ai_unit_data(ferry, ait);
+
+    ai->stats.boats--;
+
+    if (unit_data->passenger == FERRY_AVAILABLE) {
+      ai->stats.available_boats--;
+    } else if (unit_data->passenger > 0) {
+      struct unit *passenger = game_unit_by_number(unit_data->passenger);
+
+      if (passenger != NULL) {
+        aiferry_clear_boat(ait, passenger);
+      }
+    }
+  } else if (!old_f && new_f) {
+    struct ai_plr *ai = dai_plr_data_get(ait, unit_owner(ferry), NULL);
+
+    ai->stats.boats++;
+    ai->stats.available_boats++;
+  }
+}
+
+/**************************************************************************
+  Close ferry when player loses it
+**************************************************************************/
+void dai_ferry_lost(struct ai_type *ait, struct unit *punit)
 {
   /* Ignore virtual units. */
-  if (punit->id != 0 && is_ai_data_phase_open(unit_owner(punit))) {
-    if (!dai_is_ferry(punit)) {
-      aiferry_clear_boat(punit);
+  if (punit->id != 0 && is_ai_data_phase_open(ait, unit_owner(punit))) {
+    bool close;
+    struct unit_ai *unit_data = def_ai_unit_data(punit, ait);
+    struct player *pplayer = unit_owner(punit);
+    struct ai_plr *ai = dai_plr_data_get(ait, pplayer, &close);
+
+    if (dai_is_ferry(punit)) {
+      ai->stats.boats--;
+      if (unit_data->passenger == FERRY_AVAILABLE) {
+        ai->stats.available_boats--;
+      }
+    } else {
+      /* Not a ferry */
+      if (unit_data->ferryboat > 0) {
+        aiferry_clear_boat(ait, punit);
+      }
+    }
+
+    if (close) {
+      dai_data_phase_finished(ait, pplayer);
     }
   }
 }
@@ -194,15 +249,15 @@ void dai_unit_lost(struct unit *punit)
 /**************************************************************************
   Use on a unit which no longer needs a boat. 
 **************************************************************************/
-void aiferry_clear_boat(struct unit *punit)
+void aiferry_clear_boat(struct ai_type *ait, struct unit *punit)
 {
-  struct unit_ai *unit_data = def_ai_unit_data(punit);
+  struct unit_ai *unit_data = def_ai_unit_data(punit, ait);
 
   if (unit_data->ferryboat == FERRY_WANTED) {
     struct player *pplayer = unit_owner(punit);
 
-    if (is_ai_data_phase_open(pplayer)) {
-      struct ai_plr *ai = ai_plr_data_get(pplayer, NULL);
+    if (is_ai_data_phase_open(ait, pplayer)) {
+      struct ai_plr *ai = dai_plr_data_get(ait, pplayer, NULL);
 
       ai->stats.passengers--;
     }
@@ -210,14 +265,14 @@ void aiferry_clear_boat(struct unit *punit)
     struct unit *ferry = game_unit_by_number(unit_data->ferryboat);
 
     if (ferry) {
-      struct unit_ai *ferry_data = def_ai_unit_data(ferry);
+      struct unit_ai *ferry_data = def_ai_unit_data(ferry, ait);
 
       if (ferry_data->passenger == punit->id) {
         /* punit doesn't want us anymore */
         struct player *pplayer = unit_owner(ferry);
 
-        if (is_ai_data_phase_open(pplayer)) {
-          ai_plr_data_get(pplayer, NULL)->stats.available_boats++;
+        if (is_ai_data_phase_open(ait, pplayer)) {
+          dai_plr_data_get(ait, pplayer, NULL)->stats.available_boats++;
         }
         ferry_data->passenger = FERRY_AVAILABLE;
       }
@@ -231,13 +286,18 @@ void aiferry_clear_boat(struct unit *punit)
   Request a boat for the unit.  Should only be used if the unit is on the
   coast, otherwise ferries will not see it.
 **************************************************************************/
-static void aiferry_request_boat(struct unit *punit)
+static void aiferry_request_boat(struct ai_type *ait, struct unit *punit)
 {
-  struct ai_plr *ai = ai_plr_data_get(unit_owner(punit), NULL);
-  struct unit_ai *unit_data = def_ai_unit_data(punit);
+  struct ai_plr *ai = dai_plr_data_get(ait, unit_owner(punit), NULL);
+  struct unit_ai *unit_data = def_ai_unit_data(punit, ait);
 
-  /* First clear the previous assignments (just in case). */
-  aiferry_clear_boat(punit);
+  /* First clear the previous assignments (just in case there are). 
+   * Substract virtual units or already counted */
+  if ((punit->id == 0) || 
+      ((ai->stats.passengers > 0) && 
+       (unit_data->ferryboat == FERRY_WANTED))) {
+    aiferry_clear_boat(ait, punit);
+  }
 
   /* Now add ourselves to the list of potential passengers */
   ai->stats.passengers++;
@@ -252,36 +312,37 @@ static void aiferry_request_boat(struct unit *punit)
 /**************************************************************************
   Assign the passenger to the boat and vice versa.
 **************************************************************************/
-static void aiferry_psngr_meet_boat(struct unit *punit, struct unit *pferry)
+static void aiferry_psngr_meet_boat(struct ai_type *ait,
+                                    struct unit *punit, struct unit *pferry)
 {
-  struct unit_ai *ferry_data = def_ai_unit_data(pferry);
+  struct unit_ai *ferry_data = def_ai_unit_data(pferry, ait);
   struct player *ferry_owner = unit_owner(pferry);
 
   fc_assert_ret(unit_owner(punit) == ferry_owner);
 
   /* First delete the unit from the list of passengers and 
    * release its previous ferry */
-  aiferry_clear_boat(punit);
+  aiferry_clear_boat(ait, punit);
 
   /* If ferry was available, update the stats */
   if (ferry_data->passenger == FERRY_AVAILABLE) {
-    ai_plr_data_get(ferry_owner, NULL)->stats.available_boats--;
+    dai_plr_data_get(ait, ferry_owner, NULL)->stats.available_boats--;
   }
 
   /* Exchange the phone numbers */
-  def_ai_unit_data(punit)->ferryboat = pferry->id;
+  def_ai_unit_data(punit, ait)->ferryboat = pferry->id;
   ferry_data->passenger = punit->id;
 }
 
 /**************************************************************************
   Mark the ferry as available and update the statistics.
 **************************************************************************/
-static void aiferry_make_available(struct unit *pferry)
+static void aiferry_make_available(struct ai_type *ait, struct unit *pferry)
 {
-  struct unit_ai *ferry_data = def_ai_unit_data(pferry);
+  struct unit_ai *ferry_data = def_ai_unit_data(pferry, ait);
 
   if (ferry_data->passenger != FERRY_AVAILABLE) {
-    ai_plr_data_get(unit_owner(pferry), NULL)->stats.available_boats++;
+    dai_plr_data_get(ait, unit_owner(pferry), NULL)->stats.available_boats++;
     ferry_data->passenger = FERRY_AVAILABLE;
   }
 }
@@ -290,20 +351,18 @@ static void aiferry_make_available(struct unit *pferry)
   Returns the number of available boats.  A simple accessor made to perform 
   debug checks.
 **************************************************************************/
-static int aiferry_avail_boats(struct player *pplayer)
+int aiferry_avail_boats(struct ai_type *ait, struct player *pplayer)
 {
-  struct ai_plr *ai = ai_plr_data_get(pplayer, NULL);
+  struct ai_plr *ai = dai_plr_data_get(ait, pplayer, NULL);
 
   /* To developer: Switch this checking on when testing some new 
    * ferry code. */
-  /* There is one "legitimate" reason for failing this check: a ferry got
-   * killed but wasn't taken off the register */
 #ifdef LOGLEVEL_FERRY_STATS
   int boats = 0;
 
   unit_list_iterate(pplayer->units, punit) {
     if (dai_is_ferry(punit)
-        && def_ai_unit_data(punit)->passenger == FERRY_AVAILABLE) {
+        && def_ai_unit_data(punit, ait)->passenger == FERRY_AVAILABLE) {
       boats++;
     }
   } unit_list_iterate_end;
@@ -312,14 +371,13 @@ static int aiferry_avail_boats(struct player *pplayer)
     log_base(LOGLEVEL_FERRY_STATS,
              "Player[%d] in turn %d: boats miscounted.",
              player_number(pplayer), game.info.turn);
-    aiferry_print_stats(pplayer);
+    aiferry_print_stats(ait, pplayer);
   }
 #endif /* LOGLEVEL_FERRY_STATS */
 
   return ai->stats.available_boats;
 }
 
-
 /* ================== functions to find a boat ========================= */
 
 /**************************************************************************
@@ -350,7 +408,7 @@ static int combined_land_sea_move(const struct tile *src_tile,
     move_cost = PF_IMPOSSIBLE_MC;
   } else {
     /* Land-to-Land */
-    move_cost = map_move_cost_ai(param->owner, src_tile, tgt_tile);
+    move_cost = map_move_cost(param->owner, param->uclass, src_tile, tgt_tile);
   }
 
   return move_cost;
@@ -377,7 +435,8 @@ static int sea_move(const struct tile *ptile, enum known_type known,
   Runs a few checks to determine if "boat" is a free boat that can carry
   "cap" units of the same type as "punit" over sea.
 ****************************************************************************/
-bool is_boat_free(struct unit *boat, struct unit *punit, int cap)
+bool is_boat_free(struct ai_type *ait, struct unit *boat,
+                  struct unit *punit, int cap)
 {
   /* - Only transporters capable of transporting this unit are eligible.
    * - Units with orders are skipped (the AI doesn't control units with
@@ -390,7 +449,7 @@ bool is_boat_free(struct unit *boat, struct unit *punit, int cap)
    * - Units that require fuel are lose hitpoints are not eligible.
    */
   struct unit_class *ferry_class = unit_class(boat);
-  struct unit_ai *boat_data = def_ai_unit_data(boat);
+  struct unit_ai *boat_data = def_ai_unit_data(boat, ait);
 
   return (can_unit_transport(boat, punit)
           && !unit_has_orders(boat)
@@ -407,7 +466,7 @@ bool is_boat_free(struct unit *boat, struct unit *punit, int cap)
 /****************************************************************************
   Check if unit is boss in ferry
 ****************************************************************************/
-bool is_boss_of_boat(struct unit *punit)
+bool is_boss_of_boat(struct ai_type *ait, struct unit *punit)
 {
   if (!unit_transported(punit)) {
     /* Not even in boat */
@@ -415,7 +474,7 @@ bool is_boss_of_boat(struct unit *punit)
   }
 
   if (unit_transported(punit)
-      && def_ai_unit_data(unit_transport_get(punit))->passenger
+      && def_ai_unit_data(unit_transport_get(punit), ait)->passenger
          == punit->id) {
     return TRUE;
   }
@@ -431,7 +490,8 @@ bool is_boss_of_boat(struct unit *punit)
   WARNING: Due to the nature of this function and PF (see the comment of 
   combined_land_sea_move), the path won't lead onto the boat itself.
 ****************************************************************************/
-int aiferry_find_boat(struct unit *punit, int cap, struct pf_path **path)
+int aiferry_find_boat(struct ai_type *ait, struct unit *punit,
+                      int cap, struct pf_path **path)
 {
   int best_turns = FC_INFINITY;
   int best_id = 0;
@@ -439,7 +499,7 @@ int aiferry_find_boat(struct unit *punit, int cap, struct pf_path **path)
   struct pf_map *search_map;
 
   /* currently assigned ferry */
-  int ferryboat = def_ai_unit_data(punit)->ferryboat;
+  int ferryboat = def_ai_unit_data(punit, ait)->ferryboat;
 
   /* We may end calling pf_destroy_path for *path if it's not NULL.
    * Most likely you are passing garbage or path you don't want
@@ -454,7 +514,7 @@ int aiferry_find_boat(struct unit *punit, int cap, struct pf_path **path)
                     || FERRY_WANTED == ferryboat, 0);
   UNIT_LOG(LOGLEVEL_FINDFERRY, punit, "asked aiferry_find_boat for a boat");
 
-  if (aiferry_avail_boats(unit_owner(punit)) <= 0 
+  if (aiferry_avail_boats(ait, unit_owner(punit)) <= 0 
       && ferryboat <= 0) {
     /* No boats to be found (the second check is to ensure that we are not 
      * the ones keeping the last boat busy) */
@@ -482,7 +542,7 @@ int aiferry_find_boat(struct unit *punit, int cap, struct pf_path **path)
     
     square_iterate(pos.tile, radius, ptile) {
       unit_list_iterate(ptile->units, aunit) {
-        if (is_boat_free(aunit, punit, cap)) {
+        if (is_boat_free(ait, aunit, punit, cap)) {
           /* Turns for the unit to get to rendezvous pnt */
           int u_turns = pos.turn;
           /* Turns for the boat to get to the rendezvous pnt */
@@ -517,13 +577,14 @@ int aiferry_find_boat(struct unit *punit, int cap, struct pf_path **path)
 /****************************************************************************
   Find a boat within one move from us (i.e. a one we can board).
 ****************************************************************************/
-static int aiferry_find_boat_nearby(struct unit *punit, int cap)
+static int aiferry_find_boat_nearby(struct ai_type *ait, struct unit *punit,
+                                    int cap)
 {
   UNIT_LOG(LOGLEVEL_FINDFERRY, punit, "asked find_ferry_nearby for a boat");
 
   square_iterate(unit_tile(punit), 1, ptile) {
     unit_list_iterate(ptile->units, aunit) {
-      if (is_boat_free(aunit, punit, cap)) {
+      if (is_boat_free(ait, aunit, punit, cap)) {
         return aunit->id;
       }
     } unit_list_iterate_end;
@@ -540,13 +601,13 @@ static int aiferry_find_boat_nearby(struct unit *punit, int cap)
   This is suitable for when the commander of a ferry has left;
   it gives a chance for another passenger to take control.
 **************************************************************************/
-static void ai_activate_passengers(struct unit *ferry)
+static void dai_activate_passengers(struct ai_type *ait, struct unit *ferry)
 {
   unit_list_iterate_safe(unit_tile(ferry)->units, aunit) {
     if (unit_transport_get(aunit) == ferry) {
       unit_activity_handling(aunit, ACTIVITY_IDLE);
-      def_ai_unit_data(aunit)->done = FALSE;
-      ai_manage_unit(unit_owner(aunit), aunit);
+      def_ai_unit_data(aunit, ait)->done = FALSE;
+      dai_manage_unit(ait, unit_owner(aunit), aunit);
     }
   } unit_list_iterate_safe_end;
 }
@@ -558,10 +619,11 @@ static void ai_activate_passengers(struct unit *ferry)
   the ferry to a beach head, disembark, then continue on land.
   Return FALSE iff we died.
 **************************************************************************/
-bool ai_amphibious_goto_constrained(struct unit *ferry,
-				    struct unit *passenger,
-				    struct tile *ptile,
-				    struct pft_amphibious *parameter)
+bool dai_amphibious_goto_constrained(struct ai_type *ait,
+                                     struct unit *ferry,
+                                     struct unit *passenger,
+                                     struct tile *ptile,
+                                     struct pft_amphibious *parameter)
 {
   bool alive = TRUE;
   struct player *pplayer = unit_owner(passenger);
@@ -588,7 +650,7 @@ bool ai_amphibious_goto_constrained(struct unit *ferry,
   path = pf_map_path(pfm, ptile);
 
   if (path) {
-    ai_log_path(passenger, path, &parameter->combined);
+    dai_log_path(passenger, path, &parameter->combined);
     /* Sea leg */
     alive = adv_follow_path(ferry, path, ptile);
     if (alive && unit_tile(passenger) != ptile) {
@@ -623,7 +685,7 @@ bool ai_amphibious_goto_constrained(struct unit *ferry,
 	   * of the ferry.
 	   */
 	  UNIT_LOG(LOG_DEBUG, ferry, "Activating passengers");
-	  ai_activate_passengers(ferry);
+          dai_activate_passengers(ait, ferry);
           /* It is theoretically possible passenger died here due to
            * autoattack against another passing unit at its location. */
         }
@@ -646,26 +708,28 @@ bool ai_amphibious_goto_constrained(struct unit *ferry,
   Move a passenger on a ferry to a specified destination.
   Return FALSE iff we died.
 **************************************************************************/
-bool aiferry_goto_amphibious(struct unit *ferry,
+bool aiferry_goto_amphibious(struct ai_type *ait, struct unit *ferry,
 			     struct unit *passenger, struct tile *ptile)
 {
   struct pft_amphibious parameter;
   struct adv_risk_cost land_risk_cost;
   struct adv_risk_cost sea_risk_cost;
 
-  ai_fill_unit_param(&parameter.land, &land_risk_cost, passenger, ptile);
+  dai_fill_unit_param(ait, &parameter.land, &land_risk_cost, passenger, ptile);
   if (parameter.land.get_TB != no_fights) {
     /* Use the ferry to go around danger areas: */
     parameter.land.get_TB = no_intermediate_fights;
   }
-  ai_fill_unit_param(&parameter.sea, &sea_risk_cost, ferry, ptile);
+  dai_fill_unit_param(ait, &parameter.sea, &sea_risk_cost, ferry, ptile);
   pft_fill_amphibious_parameter(&parameter);
 
   /* Move as far along the path to the destination as we can;
    * that is, ignore the presence of enemy units when computing the
    * path */
   parameter.combined.get_zoc = NULL;
-  return ai_amphibious_goto_constrained(ferry, passenger, ptile, &parameter);
+
+  return dai_amphibious_goto_constrained(ait, ferry, passenger, ptile,
+                                         &parameter);
 }
 
 /****************************************************************************
@@ -678,8 +742,8 @@ bool aiferry_goto_amphibious(struct unit *ferry,
   implemented, we won't have to be at the coast to ask for a boat to come 
   to us.
 ****************************************************************************/
-bool aiferry_gobyboat(struct player *pplayer, struct unit *punit,
-		      struct tile *dest_tile, bool with_bodyguard)
+bool aiferry_gobyboat(struct ai_type *ait, struct player *pplayer,
+                      struct unit *punit, struct tile *dest_tile, bool with_bodyguard)
 {
   if (!unit_transported(punit)) {
     /* We are not on a boat and we cannot walk */
@@ -690,14 +754,14 @@ bool aiferry_gobyboat(struct player *pplayer, struct unit *punit,
     UNIT_LOG(LOGLEVEL_GOBYBOAT, punit, "will have to go to (%d,%d) by boat",
              TILE_XY(dest_tile));
 
-    if (!is_ocean_near_tile(unit_tile(punit))) {
+    if (!is_terrain_class_near_tile(unit_tile(punit), TC_OCEAN)) {
       struct pf_path *path_to_ferry = NULL;
 
-      boatid = aiferry_find_boat(punit, cap, &path_to_ferry);
+      boatid = aiferry_find_boat(ait, punit, cap, &path_to_ferry);
       if (boatid <= 0) {
         UNIT_LOG(LOGLEVEL_GOBYBOAT, punit,
                  "in ai_gothere cannot find any boats.");
-        def_ai_unit_data(punit)->done = TRUE; /* Nothing to do */
+        def_ai_unit_data(punit, ait)->done = TRUE; /* Nothing to do */
         return FALSE;
       }
 
@@ -714,16 +778,16 @@ bool aiferry_gobyboat(struct player *pplayer, struct unit *punit,
       pf_path_destroy(path_to_ferry);
     }
 
-    if (!is_ocean_near_tile(unit_tile(punit))) {
+    if (!is_terrain_class_near_tile(unit_tile(punit), TC_OCEAN)) {
       /* Still haven't reached the coast */
       return FALSE;
     }
 
     /* We are on the coast, look around for a boat */
-    boatid = aiferry_find_boat_nearby(punit, cap);
+    boatid = aiferry_find_boat_nearby(ait, punit, cap);
     if (boatid <= 0) {
       UNIT_LOG(LOGLEVEL_GOBYBOAT, punit, "requesting a boat.");
-      aiferry_request_boat(punit);
+      aiferry_request_boat(ait, punit);
       return FALSE;
     }
 
@@ -733,10 +797,10 @@ bool aiferry_gobyboat(struct player *pplayer, struct unit *punit,
 	     ferryboat->id, TILE_XY(unit_tile(ferryboat)));
     /* Setting ferry now in hope it won't run away even 
      * if we can't board it right now */
-    aiferry_psngr_meet_boat(punit, ferryboat);
+    aiferry_psngr_meet_boat(ait, punit, ferryboat);
 
     if (is_tiles_adjacent(unit_tile(punit), unit_tile(ferryboat))) {
-      (void) ai_unit_move(punit, unit_tile(ferryboat));
+      (void) dai_unit_move(ait, punit, unit_tile(ferryboat));
     }
 
     if (!can_unit_load(punit, ferryboat)) {
@@ -757,33 +821,33 @@ bool aiferry_gobyboat(struct player *pplayer, struct unit *punit,
     struct unit *ferryboat = unit_transport_get(punit);
 
     /* Check if we are the passenger-in-charge */
-    if (is_boat_free(ferryboat, punit, 0)) {
-      struct unit *bodyguard = aiguard_guard_of(punit);
+    if (is_boat_free(ait, ferryboat, punit, 0)) {
+      struct unit *bodyguard = aiguard_guard_of(ait, punit);
 
       UNIT_LOG(LOGLEVEL_GOBYBOAT, punit, 
 	       "got boat[%d](moves left: %d), going (%d,%d)",
                ferryboat->id, ferryboat->moves_left, TILE_XY(dest_tile));
-      aiferry_psngr_meet_boat(punit, ferryboat);
+      aiferry_psngr_meet_boat(ait, punit, ferryboat);
 
       punit->goto_tile = dest_tile;
       /* Grab bodyguard */
       if (bodyguard
           && !same_pos(unit_tile(punit), unit_tile(bodyguard))) {
-        if (!goto_is_sane(bodyguard, unit_tile(punit), TRUE)
-            || !ai_unit_goto(bodyguard, unit_tile(punit))) {
+        if (!goto_is_sane(ait, bodyguard, unit_tile(punit), TRUE)
+            || !dai_unit_goto(ait, bodyguard, unit_tile(punit))) {
           /* Bodyguard can't get there or died en route */
-          aiguard_request_guard(punit);
+          aiguard_request_guard(ait, punit);
           bodyguard = NULL;
         } else if (bodyguard->moves_left <= 0) {
           /* Wait for me, I'm cooooming!! */
           UNIT_LOG(LOGLEVEL_GOBYBOAT, punit, "waiting for bodyguard");
-          def_ai_unit_data(punit)->done = TRUE;
+          def_ai_unit_data(punit, ait)->done = TRUE;
           return FALSE;
         } else {
           /* Crap bodyguard. Got stuck somewhere. Ditch it! */
           UNIT_LOG(LOGLEVEL_GOBYBOAT, punit, "ditching useless bodyguard");
-          aiguard_request_guard(punit);
-          ai_unit_new_task(bodyguard, AIUNIT_NONE, NULL);
+          aiguard_request_guard(ait, punit);
+          dai_unit_new_task(ait, bodyguard, AIUNIT_NONE, NULL);
           bodyguard = NULL;
         }
       }
@@ -791,7 +855,7 @@ bool aiferry_gobyboat(struct player *pplayer, struct unit *punit,
         fc_assert(same_pos(unit_tile(punit), unit_tile(bodyguard)));
         handle_unit_load(pplayer, bodyguard->id, ferryboat->id);
       }
-      if (!aiferry_goto_amphibious(ferryboat, punit, dest_tile)) {
+      if (!aiferry_goto_amphibious(ait, ferryboat, punit, dest_tile)) {
         /* died */
         return FALSE;
       }
@@ -800,14 +864,14 @@ bool aiferry_gobyboat(struct player *pplayer, struct unit *punit,
         unit_activity_handling(punit, ACTIVITY_IDLE);
       } else {
         /* We are in still transit */
-        def_ai_unit_data(punit)->done = TRUE;
+        def_ai_unit_data(punit, ait)->done = TRUE;
         return FALSE;
       }
     } else {
       /* Waiting for the boss to load and move us */
       UNIT_LOG(LOGLEVEL_GOBYBOAT, punit, "Cannot command boat [%d],"
                " its boss is [%d]", 
-               ferryboat->id, def_ai_unit_data(ferryboat)->passenger);
+               ferryboat->id, def_ai_unit_data(ferryboat, ait)->passenger);
       return FALSE;
     }
   }
@@ -825,12 +889,12 @@ bool aiferry_gobyboat(struct player *pplayer, struct unit *punit,
 
   TODO: lift the path off the map
 ****************************************************************************/
-static bool aiferry_findcargo(struct unit *pferry)
+static bool aiferry_findcargo(struct ai_type *ait, struct unit *pferry)
 {
   /* Path-finding stuff */
   struct pf_map *pfm;
   struct pf_parameter parameter;
-  int passengers = ai_plr_data_get(unit_owner(pferry), NULL)->stats.passengers;
+  int passengers = dai_plr_data_get(ait, unit_owner(pferry), NULL)->stats.passengers;
 
   if (passengers <= 0) {
     /* No passangers anywhere */
@@ -847,7 +911,7 @@ static bool aiferry_findcargo(struct unit *pferry)
   pfm = pf_map_new(&parameter);
   pf_map_tiles_iterate(pfm, ptile, TRUE) {
     unit_list_iterate(ptile->units, aunit) {
-      struct unit_ai *unit_data = def_ai_unit_data(aunit);
+      struct unit_ai *unit_data = def_ai_unit_data(aunit, ait);
 
       if (unit_owner(pferry) == unit_owner(aunit) 
           && (unit_data->ferryboat == FERRY_WANTED
@@ -859,7 +923,7 @@ static bool aiferry_findcargo(struct unit *pferry)
                  TILE_XY(unit_tile(aunit)));
 	pferry->goto_tile = unit_tile(aunit);
         /* Exchange phone numbers */
-        aiferry_psngr_meet_boat(aunit, pferry);
+        aiferry_psngr_meet_boat(ait, aunit, pferry);
         pf_map_destroy(pfm);
         return TRUE;
       }
@@ -886,7 +950,8 @@ static bool aiferry_findcargo(struct unit *pferry)
   will save some code lines but will be faster in the rare cases when there
   passengers that can not be reached ("false positive").
 ****************************************************************************/
-static bool aiferry_find_interested_city(struct unit *pferry)
+static bool aiferry_find_interested_city(struct ai_type *ait,
+                                         struct unit *pferry)
 {
   /* Path-finding stuff */
   struct pf_map *pfm;
@@ -915,7 +980,7 @@ static bool aiferry_find_interested_city(struct unit *pferry)
     pcity = tile_city(pos.tile);
     
     if (pcity && city_owner(pcity) == unit_owner(pferry)
-        && (def_ai_city_data(pcity)->choice.need_boat 
+        && (def_ai_city_data(pcity, ait)->choice.need_boat 
             || (VUT_UTYPE == pcity->production.kind
 		&& utype_has_role(pcity->production.value.utype,
 				  L_FERRYBOAT)))) {
@@ -978,7 +1043,8 @@ static bool aiferry_find_interested_city(struct unit *pferry)
   If there is no one aboard, look for potential cargo.  If none found, 
   explore and then go to the nearest port.
 ****************************************************************************/
-void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
+void dai_manage_ferryboat(struct ai_type *ait, struct player *pplayer,
+                          struct unit *punit)
 {
   struct city *pcity;
   int sanity = punit->id;
@@ -992,7 +1058,7 @@ void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
       && (pcity = tile_city(unit_tile(punit)))) {
     UNIT_LOG(LOGLEVEL_FERRY, punit, "waiting in %s to recover hitpoints", 
              city_name(pcity));
-    def_ai_unit_data(punit)->done = TRUE;
+    def_ai_unit_data(punit, ait)->done = TRUE;
     return;
   }
 
@@ -1002,14 +1068,12 @@ void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
     return;
   }
 
-  unit_data = def_ai_unit_data(punit);
+  unit_data = def_ai_unit_data(punit, ait);
 
   bossid = unit_data->passenger; /* Old boss */
 
   do {
     /* Do we have the passenger-in-charge on board? */
-    struct tile *ptile = unit_tile(punit);
-
     if (unit_data->passenger > 0) {
       struct unit *psngr = game_unit_by_number(unit_data->passenger);
 
@@ -1029,16 +1093,15 @@ void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
       struct unit *candidate = NULL;
 
       /* Try to select passanger-in-charge from among our passengers */
-      unit_list_iterate(ptile->units, aunit) {
-        if (unit_owner(aunit) != pplayer 
-            || unit_transport_get(aunit) != punit) {
+      unit_list_iterate(punit->transporting, aunit) {
+        if (unit_owner(aunit) != pplayer) {
           /* We used to check if ferryboat was set to us or to
            * FERRY_WANTED too, but this was a bit strict. Especially
            * when we don't save these values in a savegame. */
           continue;
         }
 
-        if (def_ai_unit_data(aunit)->task != AIUNIT_ESCORT) {
+        if (def_ai_unit_data(aunit, ait)->task != AIUNIT_ESCORT) {
           candidate = aunit;
           break;
         } else {
@@ -1053,7 +1116,7 @@ void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
                  unit_rule_name(candidate),
                  candidate->id);
         bossid = candidate->id;
-        aiferry_psngr_meet_boat(candidate, punit);
+        aiferry_psngr_meet_boat(ait, candidate, punit);
       }
     }
 
@@ -1062,7 +1125,8 @@ void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
 
       fc_assert_ret(NULL != boss);
 
-      if (unit_has_type_flag(boss, F_SETTLERS) || unit_has_type_flag(boss, F_CITIES)) {
+      if (unit_has_type_flag(boss, UTYF_SETTLERS)
+          || unit_has_type_flag(boss, UTYF_CITIES)) {
         /* Temporary hack: settlers all go in the end, forcing them 
          * earlier might mean uninitialised cache, so just wait for them */
         return;
@@ -1071,7 +1135,7 @@ void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
       UNIT_LOG(LOGLEVEL_FERRY, punit, "passing control to %s[%d]",
 		unit_rule_name(boss),
 		boss->id);
-      ai_manage_unit(pplayer, boss);
+      dai_manage_unit(ait, pplayer, boss);
     
       if (!game_unit_by_number(sanity) || punit->moves_left <= 0) {
         return;
@@ -1080,7 +1144,7 @@ void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
 	if (same_pos(unit_tile(punit), unit_tile(boss))) {
 	  /* The boss decided to stay put on the ferry. We aren't moving. */
           UNIT_LOG(LOG_DEBUG, boss, "drove ferry - done for now");
-          def_ai_unit_data(boss)->done = TRUE;
+          def_ai_unit_data(boss, ait)->done = TRUE;
           return;
         } else if (unit_data->passenger == bossid
                    && get_transporter_occupancy(punit) != 0) {
@@ -1111,45 +1175,45 @@ void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
   if (IS_ATTACKER(punit) && punit->moves_left > 0) {
      /* AI used to build frigates to attack and then use them as ferries 
       * -- Syela */
-     ai_unit_new_task(punit, AIUNIT_ATTACK, NULL);
-     UNIT_LOG(LOGLEVEL_FERRY, punit, "passing ferry over to attack code");
-     ai_manage_military(pplayer, punit);
-     return;
+    dai_unit_new_task(ait, punit, AIUNIT_NONE, NULL);
+    UNIT_LOG(LOGLEVEL_FERRY, punit, "passing ferry over to attack code");
+    dai_manage_military(ait, pplayer, punit);
+    return;
   }
 
   UNIT_LOG(LOGLEVEL_FERRY, punit, "Ferryboat is not carrying anyone "
 	   "(moves left: %d).", punit->moves_left);
-  aiferry_make_available(punit);
+  aiferry_make_available(ait, punit);
   unit_activity_handling(punit, ACTIVITY_IDLE);
-  ai_unit_new_task(punit, AIUNIT_NONE, NULL);
+  dai_unit_new_task(ait, punit, AIUNIT_NONE, NULL);
   CHECK_UNIT(punit);
 
   /* Try to find passengers */
-  if (aiferry_findcargo(punit)) {
+  if (aiferry_findcargo(ait, punit)) {
     UNIT_LOG(LOGLEVEL_FERRY, punit, "picking up cargo (moves left: %d)",
 	     punit->moves_left);
-    if (ai_unit_goto(punit, punit->goto_tile)) {
+    if (dai_unit_goto(ait, punit, punit->goto_tile)) {
       if (is_tiles_adjacent(unit_tile(punit), punit->goto_tile)
           || same_pos(unit_tile(punit), punit->goto_tile)) {
         struct unit *cargo = game_unit_by_number(unit_data->passenger);
 
         /* See if passenger can jump on board! */
         fc_assert_ret(cargo != punit);
-        ai_manage_unit(pplayer, cargo);
+        dai_manage_unit(ait, pplayer, cargo);
       }
     }
     return;
   }
 
   /* Try to find a city that needs a ferry */
-  if (aiferry_find_interested_city(punit)) {
+  if (aiferry_find_interested_city(ait, punit)) {
     if (same_pos(unit_tile(punit), punit->goto_tile)) {
       UNIT_LOG(LOGLEVEL_FERRY, punit, "staying in city that needs us");
       unit_data->done = TRUE;
       return;
     } else {
       UNIT_LOG(LOGLEVEL_FERRY, punit, "going to city that needs us");
-      if (ai_unit_goto(punit, punit->goto_tile)
+      if (dai_unit_goto(ait, punit, punit->goto_tile)
           && same_pos(unit_tile(punit), punit->goto_tile)) {
         unit_data->done = TRUE; /* save some CPU */
       }
@@ -1176,8 +1240,8 @@ void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
       punit->goto_tile = pcity->tile;
       UNIT_LOG(LOGLEVEL_FERRY, punit, "No work, going home");
       unit_data->done = TRUE;
-      ai_unit_new_task(punit, AIUNIT_NONE, NULL);
-      (void) ai_unit_goto(punit, pcity->tile);
+      dai_unit_new_task(ait, punit, AIUNIT_NONE, NULL);
+      (void) dai_unit_goto(ait, punit, pcity->tile);
     }
   }
  
