@@ -63,9 +63,10 @@ int get_citydlg_canvas_height(void)
 void generate_citydlg_dimensions(void)
 {
   int min_x = 0, max_x = 0, min_y = 0, max_y = 0;
+  int max_rad = rs_max_city_radius_sq();
 
   /* use maximum possible squared city radius. */
-  city_map_iterate_without_index(CITY_MAP_MAX_RADIUS_SQ, city_x, city_y) {
+  city_map_iterate_without_index(max_rad, city_x, city_y) {
     int canvas_x, canvas_y;
 
     map_to_gui_vector(tileset, &canvas_x, &canvas_y, CITY_ABS2REL(city_x),
@@ -138,8 +139,8 @@ bool canvas_to_city_pos(int *city_x, int *city_y, int city_radius_sq,
 
   /* Add on the offset of the top-left corner to get the final
    * coordinates (like in canvas_to_map_pos). */
-  *city_x += CITY_MAP_MAX_RADIUS;
-  *city_y += CITY_MAP_MAX_RADIUS;
+  *city_x = CITY_REL2ABS(*city_x);
+  *city_y = CITY_REL2ABS(*city_y);
 
   log_debug("canvas_to_city_pos(pos=(%d,%d))=(%d,%d)@radius=%d",
             orig_canvas_x, orig_canvas_y, *city_x, *city_y, city_radius_sq);
@@ -462,7 +463,7 @@ void get_city_dialog_output_text(const struct city *pcity,
   if (otype == O_TRADE) {
     int i;
 
-    for (i = 0; i < NUM_TRADE_ROUTES; i++) {
+    for (i = 0; i < MAX_TRADE_ROUTES; i++) {
       if (pcity->trade[i] != 0) {
         /* There have been bugs causing the trade city to not be sent
          * properly to the client.  If this happens we trust the
@@ -507,7 +508,8 @@ void get_city_dialog_output_text(const struct city *pcity,
 	bonus += peffect->value;
 	new_total = bonus * base / 100;
 	cat_snprintf(buf, bufsz,
-		     _("%+4d : Bonus from %s (%+d%%)\n"),
+                     (peffect->value > 0) ? _("%+4d : Bonus from %s (%+d%%)\n")
+                                          : _("%+4d : Loss from %s (%+d%%)\n"),
 		     (new_total - total), buf2,
 		     peffect->value);
 	total = new_total;
@@ -621,7 +623,8 @@ void get_city_dialog_illness_text(const struct city *pcity,
     get_effect_req_text(peffect, buf2, sizeof(buf2));
 
     cat_snprintf(buf, bufsz,
-                 _("%+5.1f : Bonus from %s\n"),
+                 (peffect->value > 0) ? _("%+5.1f : Bonus from %s\n")
+                                      : _("%+5.1f : Risk from %s\n"),
                  -(0.1 * ill_base * peffect->value / 100), buf2);
   } effect_list_iterate_end;
   effect_list_destroy(plist);
