@@ -35,6 +35,7 @@
 #include "log.h"
 #include "mem.h"
 #include "rand.h"
+#include "registry.h"
 #include "support.h"
 #include "timing.h"
 
@@ -213,6 +214,7 @@ static void at_exit(void)
   client_kill_server(TRUE);
   fc_shutdown_network();
   update_queue_free();
+  fc_destroy_ow_mutex();
 }
 
 /**************************************************************************
@@ -325,8 +327,16 @@ int client_main(int argc, char *argv[])
   }
 
   init_nls();
+#ifdef ENABLE_NLS
+  (void) bindtextdomain("freeciv-nations", LOCALEDIR);
+#endif
+
+  registry_module_init();
   audio_init();
   init_character_encodings(gui_character_encoding, gui_use_transliteration);
+#ifdef ENABLE_NLS
+  bind_textdomain_codeset("freeciv-nations", get_internal_encoding());
+#endif
 
   i = 1;
 
@@ -545,6 +555,8 @@ int client_main(int argc, char *argv[])
   fc_init_network();
   update_queue_init();
 
+  fc_init_ow_mutex();
+
   /* register exit handler */ 
   atexit(at_exit);
 
@@ -643,6 +655,8 @@ void client_exit(void)
   conn_list_destroy(game.all_connections);
   conn_list_destroy(game.est_connections);
 
+  registry_module_close();
+  free_libfreeciv();
   free_nls();
 
   backtrace_deinit();
@@ -979,10 +993,10 @@ static int seconds_shown_to_turndone;
 **************************************************************************/
 void set_seconds_to_turndone(double seconds)
 {
-  if (game.info.timeout > 0) {
+  if (client_current_turn_timeout() > 0) {
     seconds_to_turndone = seconds;
-    turndone_timer = renew_timer_start(turndone_timer, TIMER_USER,
-				       TIMER_ACTIVE);
+    turndone_timer = timer_renew(turndone_timer, TIMER_USER, TIMER_ACTIVE);
+    timer_start(turndone_timer);
 
     /* Maybe we should do an update_timeout_label here, but it doesn't
      * seem to be necessary. */
@@ -992,11 +1006,11 @@ void set_seconds_to_turndone(double seconds)
 
 /**************************************************************************
   Return the number of seconds until turn-done.  Don't call this unless
-  game.info.timeout != 0.
+  client_current_turn_timeout() != 0.
 **************************************************************************/
 int get_seconds_to_turndone(void)
 {
-  if (game.info.timeout > 0) {
+  if (client_current_turn_timeout() > 0) {
     return seconds_shown_to_turndone;
   } else {
     /* This shouldn't happen. */
@@ -1036,10 +1050,10 @@ double real_timer_callback(void)
     time_until_next_call = MIN(time_until_next_call, blink_time);
   }
 
-  /* It is possible to have game.info.timeout > 0 but !turndone_timer, in the
-   * first moments after the timeout is set. */
-  if (game.info.timeout > 0 && turndone_timer) {
-    double seconds = seconds_to_turndone - read_timer_seconds(turndone_timer);
+  /* It is possible to have client_current_turn_timeout() > 0 but !turndone_timer,
+   * in the first moments after the timeout is set. */
+  if (client_current_turn_timeout() > 0 && turndone_timer) {
+    double seconds = seconds_to_turndone - timer_read_seconds(turndone_timer);
     int iseconds = ceil(seconds) + 0.1; /* Turn should end right on 0. */
 
     if (iseconds < seconds_shown_to_turndone) {
@@ -1290,3 +1304,23 @@ static struct rgbcolor *mapimg_client_plrcolor_get(int i)
 
   return NULL;
 }
+
+/****************************************************************************
+  Return timeout value for the current turn.
+****************************************************************************/
+int client_current_turn_timeout(void)
+{
+  if (game.info.turn == 0) {
+    struct option *opt = optset_option_by_name(server_optset, "first_timeout");
+
+    if (opt != NULL) {
+      int val = option_int_get(opt);
+
+      if (val != -1) {
+        return val;
+      }
+    }
+  }
+
+  return game.info.timeout;
+}
