@@ -25,6 +25,7 @@
 #include "support.h"
 
 /* common */
+#include "game.h"
 #include "map.h"
 #include "movement.h"
 #include "packets.h"
@@ -123,6 +124,12 @@ static void tool_init(enum editor_tool_type ett, const char *name,
   tool->size = 1;
   tool->count = 1;
   tool->applied_player_no = 0;
+
+  if (ett == ETT_TERRAIN_SPECIAL) {
+    tool->value = S_IRRIGATION;
+  } else {
+    tool->value = 0;
+  }
 }
 
 /****************************************************************************
@@ -147,6 +154,10 @@ void editor_init(void)
             | ETF_HAS_SIZE | ETF_HAS_VALUE_ERASE,
             _("Modify tile specials.\nShortcut: s\n"
               "Select special type: shift+s or right-click here."));
+  tool_init(ETT_ROAD, _("Road"), ETF_HAS_VALUE
+            | ETF_HAS_SIZE | ETF_HAS_VALUE_ERASE,
+            _("Modify roads on tile.\nShortcut: p\n"
+              "Select road type: shift+p or right-click here."));
   tool_init(ETT_MILITARY_BASE, _("Military Base"), ETF_HAS_VALUE
             | ETF_HAS_SIZE | ETF_HAS_VALUE_ERASE,
             _("Create a military base.\nShortcut: m\n"
@@ -163,7 +174,7 @@ void editor_init(void)
             _("Place a start position which allows any nation to "
               "start at the tile. To allow only certain nations to "
               "start there, middle click on the start position on "
-              "the map and use the property editor.\nShortcut: p"));
+              "the map and use the property editor.\nShortcut: b"));
 
   tool_init(ETT_COPYPASTE, _("Copy/Paste"), ETF_HAS_SIZE,
             _("Copy and paste tiles.\n"
@@ -284,7 +295,7 @@ bool editor_is_active(void)
 }
 
 /****************************************************************************
-  Returns TRUE if the given tool should be made availble to the user via
+  Returns TRUE if the given tool should be made available to the user via
   the editor GUI. For example, this will return FALSE for ETT_MILITARY_BASE
   if there are no bases defined in the ruleset.
 
@@ -300,13 +311,12 @@ bool editor_tool_is_usable(enum editor_tool_type ett)
   switch (ett) {
   case ETT_MILITARY_BASE:
     return base_count() > 0;
-    break;
+  case ETT_ROAD:
+    return road_count() > 0;
   case ETT_TERRAIN_RESOURCE:
     return resource_count() > 0;
-    break;
   case ETT_UNIT:
     return utype_count() > 0;
-    break;
   default:
     break;
   }
@@ -347,7 +357,7 @@ int editor_tool_get_value(enum editor_tool_type ett)
       || !editor_tool_has_value(ett)) {
     return 0;
   }
-  
+
   return editor->tools[ett].value;
 }
 
@@ -432,6 +442,7 @@ static void editor_grab_tool(const struct tile *ptile)
 {
   int ett = -1, value = 0;
   struct base_type *first_base = NULL;
+  struct road_type *first_road = NULL;
 
   if (!editor) {
     return;
@@ -448,6 +459,13 @@ static void editor_grab_tool(const struct tile *ptile)
     }
   } base_type_iterate_end;
 
+  road_type_iterate(proad) {
+    if (tile_has_road(ptile, proad)) {
+      first_road = proad;
+      break;
+    }
+  } road_type_iterate_end;
+
   if (client_has_player()
       && tile_get_known(ptile, client_player()) == TILE_UNKNOWN) {
     ett = ETT_VISION;
@@ -487,6 +505,10 @@ static void editor_grab_tool(const struct tile *ptile)
     ett = ETT_MILITARY_BASE;
     value = base_number(first_base);
 
+  } else if (first_road != NULL) {
+    ett = ETT_ROAD;
+    value = road_number(first_road);
+
   } else if (tile_really_has_any_specials(ptile)) {
     int specials_array[S_LAST];
     int count = 0, i, special = -1;
@@ -881,7 +903,8 @@ void editor_apply_tool(const struct tile *ptile,
     break;
 
   case ETT_TERRAIN_RESOURCE:
-    dsend_packet_edit_tile_resource(my_conn, tile, erase ? -1 : value,
+    dsend_packet_edit_tile_resource(my_conn, tile,
+                                    erase ? resource_count() : value,
                                     size);
     break;
 
@@ -889,6 +912,10 @@ void editor_apply_tool(const struct tile *ptile,
     dsend_packet_edit_tile_special(my_conn, tile, value, erase, size);
     break;
 
+  case ETT_ROAD:
+    dsend_packet_edit_tile_road(my_conn, tile, value, erase, size);
+    break;
+
   case ETT_MILITARY_BASE:
     dsend_packet_edit_tile_base(my_conn, tile, value, erase, size);
     break;
@@ -1094,6 +1121,7 @@ const char *editor_tool_get_value_name(enum editor_tool_type emt, int value)
   struct resource *presource;
   struct unit_type *putype;
   struct base_type *pbase;
+  struct road_type *proad;
 
   if (!editor) {
     return "";
@@ -1114,10 +1142,14 @@ const char *editor_tool_get_value_name(enum editor_tool_type emt, int value)
     }
     return special_name_translation(value);
     break;
+  case ETT_ROAD:
+    proad = road_by_number(value);
+
+    return proad != NULL ? road_name_translation(proad) : "";
   case ETT_MILITARY_BASE:
     pbase = base_by_number(value);
+
     return pbase != NULL ? base_name_translation(pbase) : "";
-    break;
   case ETT_UNIT:
     putype = utype_by_number(value);
     return putype ? utype_name_translation(putype) : "";
@@ -1225,9 +1257,10 @@ struct sprite *editor_tool_get_sprite(enum editor_tool_type ett)
   case ETT_TERRAIN_SPECIAL:
     return sprites->terrain_special;
     break;
+  case ETT_ROAD:
+    return sprites->road;
   case ETT_MILITARY_BASE:
     return sprites->military_base;
-    break;
   case ETT_UNIT:
     return sprites->unit;
     break;
@@ -1467,6 +1500,12 @@ void edit_buffer_copy(struct edit_buffer *ebuf, const struct tile *ptile)
         copied = TRUE;
       }
       break;
+    case EBT_ROAD:
+      if (BV_ISSET_ANY(ptile->roads)) {
+        vtile->roads = tile_roads(ptile);
+        copied = TRUE;
+      }
+      break;
     case EBT_UNIT:
       unit_list_iterate(ptile->units, punit) {
         if (!punit) {
@@ -1528,12 +1567,17 @@ static void fill_tile_edit_packet(struct packet_edit_tile *packet,
   packet->tile = tile_index(ptile);
   packet->specials = tile_specials(ptile);
   packet->bases = tile_bases(ptile);
+  packet->roads = tile_roads(ptile);
 
   presource = tile_resource(ptile);
-  packet->resource = presource ? resource_number(presource) : -1;
+  packet->resource = presource
+                     ? resource_number(presource)
+                     : resource_count();
 
   pterrain = tile_terrain(ptile);
-  packet->terrain = pterrain ? terrain_number(pterrain) : -1;
+  packet->terrain = pterrain
+                    ? terrain_number(pterrain)
+                    : terrain_count();
 
   if (ptile->label == NULL) {
     packet->label[0] = '\0';
@@ -1588,6 +1632,10 @@ static void paste_tile(struct edit_buffer *ebuf,
       tile_packet.bases = tile_bases(vtile);
       send_edit_tile = TRUE;
       break;
+    case EBT_ROAD:
+      tile_packet.roads = tile_roads(vtile);
+      send_edit_tile = TRUE;
+      break;
     case EBT_UNIT:
       unit_list_iterate(vtile->units, vunit) {
         value = utype_number(unit_type(vunit));
