@@ -1131,7 +1131,6 @@ static void color_selected(GtkDialog *dialog, gint res, gpointer data)
 **************************************************************************/
 static void select_color_callback(GtkToolButton *button, gpointer data)
 {
-  char buf[64];
   GtkWidget *dialog, *selection;
   /* "fg_color" or "bg_color". */
   const gchar *color_target = g_object_get_data(G_OBJECT(button),
@@ -1139,7 +1138,7 @@ static void select_color_callback(GtkToolButton *button, gpointer data)
   GdkColor *current_color = g_object_get_data(G_OBJECT(data), color_target);
 
   /* TRANS: "text" or "background". */
-  fc_snprintf(buf, sizeof(buf), _("Select the %s color"),
+  gchar *buf = g_strdup_printf(_("Select the %s color"),
               (const char *) g_object_get_data(G_OBJECT(button),
                                                "color_info"));
   dialog = gtk_dialog_new_with_buttons(buf, NULL, GTK_DIALOG_MODAL,
@@ -1160,6 +1159,7 @@ static void select_color_callback(GtkToolButton *button, gpointer data)
   }
 
   gtk_widget_show_all(dialog);
+  g_free(buf);
 }
 
 /**************************************************************************
@@ -1456,3 +1456,30 @@ void chatline_init(void)
   gtk_box_pack_end(GTK_BOX(hbox), bbox, FALSE, FALSE, 0);
   toolkit.button_box = bbox;
 }
+
+/**************************************************************************
+  Main thread side callback to print version message
+**************************************************************************/
+static gboolean version_message_main_thread(gpointer user_data)
+{
+  char *vertext = (char *)user_data;
+
+  output_window_append(ftc_client, vertext);
+
+  FC_FREE(vertext);
+
+  return FALSE;
+}
+
+/**************************************************************************
+  Got version message from metaserver thread.
+**************************************************************************/
+void version_message(char *vertext)
+{
+  int len = strlen(vertext) + 1;
+  char *persistent = fc_malloc(len);
+
+  strncpy(persistent, vertext, len);
+
+  gdk_threads_add_idle(version_message_main_thread, persistent);
+}
