@@ -542,6 +542,7 @@ static struct editbar *editbar_create(void)
   editbar_add_tool_button(eb, ETT_TERRAIN);
   editbar_add_tool_button(eb, ETT_TERRAIN_RESOURCE);
   editbar_add_tool_button(eb, ETT_TERRAIN_SPECIAL);
+  editbar_add_tool_button(eb, ETT_ROAD);
   editbar_add_tool_button(eb, ETT_MILITARY_BASE);
   editbar_add_tool_button(eb, ETT_UNIT);
   editbar_add_tool_button(eb, ETT_CITY);
@@ -684,6 +685,44 @@ static void editbar_refresh(struct editbar *eb)
   gtk_widget_show_all(eb->widget);
 }
 
+/****************************************************************************
+  Create a pixbuf containing a representative image for the given road
+  type, to be used as an icon in the GUI.
+
+  May return NULL on error.
+
+  NB: You must call g_object_unref on the non-NULL return value when you
+  no longer need it.
+****************************************************************************/
+static GdkPixbuf *create_road_pixbuf(const struct road_type *proad)
+{
+  struct drawn_sprite sprs[80];
+  int count, w, h, canvas_x, canvas_y;
+  GdkPixbuf *pixbuf;
+  struct canvas canvas = FC_STATIC_CANVAS_INIT;
+  cairo_t *cr;
+
+  w = tileset_tile_width(tileset);
+  h = tileset_tile_height(tileset);
+
+  canvas.surface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, w, h);
+  canvas_x = 0;
+  canvas_y = 0;
+
+  cr = cairo_create(canvas.surface);
+  cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
+  cairo_paint(cr);
+  cairo_destroy(cr);
+
+  count = fill_basic_road_sprite_array(tileset, sprs, proad);
+  put_drawn_sprites(&canvas, canvas_x, canvas_y, count, sprs, FALSE);
+
+  pixbuf = surface_get_pixbuf(canvas.surface, w, h);
+  cairo_surface_destroy(canvas.surface);
+
+  return pixbuf;
+}
+
 /****************************************************************************
   Create a pixbuf containing a representative image for the given base
   type, to be used as an icon in the GUI.
@@ -698,14 +737,13 @@ static GdkPixbuf *create_military_base_pixbuf(const struct base_type *pbase)
   struct drawn_sprite sprs[80];
   int count, w, h, canvas_x, canvas_y;
   GdkPixbuf *pixbuf;
-  struct canvas canvas;
+  struct canvas canvas = FC_STATIC_CANVAS_INIT;
   cairo_t *cr;
 
   w = tileset_tile_width(tileset);
   h = tileset_tile_height(tileset);
 
   canvas.surface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, w, h);
-  canvas.drawable = NULL;
   canvas_x = 0;
   canvas_y = 0;
 
@@ -737,14 +775,13 @@ static GdkPixbuf *create_terrain_pixbuf(struct terrain *pterrain)
   struct drawn_sprite sprs[80];
   int count, w, h, canvas_x, canvas_y, i;
   GdkPixbuf *pixbuf;
-  struct canvas canvas;
+  struct canvas canvas = FC_STATIC_CANVAS_INIT;
   cairo_t *cr;
 
   w = tileset_tile_width(tileset);
   h = tileset_tile_height(tileset);
 
   canvas.surface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, w, h);
-  canvas.drawable = NULL;
   canvas_x = 0;
   canvas_y = 0;
 
@@ -851,6 +888,28 @@ static void editbar_reload_tileset(struct editbar *eb)
     gtk_list_store_set(store, &iter, TVS_COL_IMAGE, pixbuf, -1);
   } tile_special_type_iterate_end;
 
+  /* Reload roads. */
+
+  tvs = eb->tool_selectors[ETT_ROAD];
+  store = tvs->store;
+  gtk_list_store_clear(store);
+
+  road_type_iterate(proad) {
+    int id;
+
+    id = road_number(proad);
+
+    gtk_list_store_append(store, &iter);
+    gtk_list_store_set(store, &iter,
+                       TVS_COL_ID, id,
+                       TVS_COL_NAME, road_name_translation(proad),
+                       -1);
+    pixbuf = create_road_pixbuf(proad);
+    if (pixbuf != NULL) {
+      gtk_list_store_set(store, &iter, TVS_COL_IMAGE, pixbuf, -1);
+      g_object_unref(pixbuf);
+    }
+  } road_type_iterate_end;
 
   /* Reload military bases. */
 
@@ -1420,6 +1479,7 @@ static GdkPixbuf *get_tool_value_pixbuf(enum editor_tool_type ett,
   struct resource *presource;
   struct unit_type *putype;
   struct base_type *pbase;
+  struct road_type *proad;
   const struct editor_sprites *sprites;
 
   sprites = get_editor_sprites(tileset);
@@ -1443,6 +1503,10 @@ static GdkPixbuf *get_tool_value_pixbuf(enum editor_tool_type ett,
   case ETT_TERRAIN_SPECIAL:
     sprite = get_basic_special_sprite(tileset, value);
     break;
+  case ETT_ROAD:
+    proad = road_by_number(value);
+    pixbuf = create_road_pixbuf(proad);
+    break;
   case ETT_MILITARY_BASE:
     pbase = base_by_number(value);
     pixbuf = create_military_base_pixbuf(pbase);
@@ -1652,6 +1716,9 @@ static gboolean handle_edit_key_press_with_shift(GdkEventKey *ev)
   case GDK_KEY_S:
     editgui_run_tool_selection(ETT_TERRAIN_SPECIAL);
     break;
+  case GDK_KEY_P:
+    editgui_run_tool_selection(ETT_ROAD);
+    break;
   case GDK_KEY_M:
     editgui_run_tool_selection(ETT_MILITARY_BASE);
     break;
@@ -1695,6 +1762,9 @@ gboolean handle_edit_key_press(GdkEventKey *ev)
   case GDK_KEY_s:
     new_ett = ETT_TERRAIN_SPECIAL;
     break;
+  case GDK_KEY_p:
+    new_ett = ETT_ROAD;
+    break;
   case GDK_KEY_m:
     new_ett = ETT_MILITARY_BASE;
     break;
@@ -1707,7 +1777,7 @@ gboolean handle_edit_key_press(GdkEventKey *ev)
   case GDK_KEY_v:
     new_ett = ETT_VISION;
     break;
-  case GDK_KEY_p:
+  case GDK_KEY_b:
     new_ett = ETT_STARTPOS;
     break;
   case GDK_KEY_plus:
