@@ -80,13 +80,15 @@ void update_turn_done_button(bool do_restore)
   }
 
   if ((do_restore && flip) || !do_restore) {
-    GdkColor *fore = &gtk_widget_get_style(turn_done_button)->bg[GTK_STATE_NORMAL];
-    GdkColor *back = &gtk_widget_get_style(turn_done_button)->light[GTK_STATE_NORMAL];
+    GdkRGBA fore;
+    GdkRGBA back;
+    GtkStyleContext *context = gtk_widget_get_style_context(turn_done_button);
 
-    gtk_widget_get_style(turn_done_button)->bg[GTK_STATE_NORMAL] = *back;
-    gtk_widget_get_style(turn_done_button)->light[GTK_STATE_NORMAL] = *fore;
+    gtk_style_context_get_color(context, GTK_STATE_FLAG_NORMAL, &fore);
+    gtk_style_context_get_background_color(context, GTK_STATE_FLAG_NORMAL, &back);
 
-    gtk_expose_now(turn_done_button);
+    gtk_widget_override_color(turn_done_button, GTK_STATE_FLAG_NORMAL, &back);
+    gtk_widget_override_background_color(turn_done_button, GTK_STATE_FLAG_NORMAL, &fore);
 
     flip = !flip;
   }
@@ -99,7 +101,7 @@ void update_timeout_label(void)
 {
   gtk_label_set_text(GTK_LABEL(timeout_label), get_timeout_label_text());
 
-  if (game.info.timeout > 0) {
+  if (client_current_turn_timeout() > 0) {
     gtk_widget_set_tooltip_text(timeout_label, _("Time to forced turn change"));
   } else {
     gtk_widget_set_tooltip_text(timeout_label, _("Turn timeout disabled"));
