@@ -29,6 +29,7 @@
 
 /* client/gui-gtk-3.0 */
 #include "colors.h"
+#include "dialogs.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
 #include "pages.h"
@@ -528,7 +529,7 @@ static void option_dialog_option_add(struct option_dialog *pdialog,
 
   hbox = gtk_grid_new();
   label = gtk_label_new(option_description(poption));
-  g_object_set(label, "margin", 5, NULL);
+  g_object_set(label, "margin", 2, NULL);
   gtk_container_add(GTK_CONTAINER(hbox), label);
   gtk_container_add(GTK_CONTAINER(ebox), hbox);
 
@@ -1008,6 +1009,8 @@ static void option_gui_update_extra(struct option *poption)
   if (option_optset(poption) == server_optset) {
     if (strcmp(option_name(poption), "aifill") == 0) {
       ai_fill_changed_by_server(option_int_get(poption));
+    } else if (strcmp(option_name(poption), "nationset") == 0) {
+      nationset_sync_to_server(option_str_get(poption));
     }
   }
 }
