@@ -888,3 +888,11 @@ bool popdown_conn_list_dialog(void)
   
   return FALSE;
 }
+
+/**************************************************************************
+  Got version message from metaserver thread.
+**************************************************************************/
+void version_message(char *vertext)
+{
+  output_window_append(ftc_client, vertext);
+}
