@@ -181,11 +181,8 @@ struct City_Icon {
   SDL_Surface *pFemale_Unhappy;
   SDL_Surface *pMale_Angry;
   SDL_Surface *pFemale_Angry;
-	
-  SDL_Surface *pSpec_Lux; /* Elvis */
-  SDL_Surface *pSpec_Tax; /* TaxMan */
-  SDL_Surface *pSpec_Sci; /* Scientist */
 
+  SDL_Surface *specialists[SP_MAX];
 };
 
 extern struct Theme *pTheme;
