@@ -23,8 +23,7 @@
 #include <fc_config.h>
 #endif
 
-/* #define SDL_CVS */
-
+/* SDL */
 #include "SDL.h"
 
 /* utility */
@@ -81,7 +80,7 @@ static struct canvas *terrain_canvas;
 
 /* ================================================================ */
 
-void update_map_canvas_scrollbars_size()
+void update_map_canvas_scrollbars_size(void)
 {
   /* No scrollbars */
 }
@@ -315,8 +314,8 @@ void set_indicator_icons(struct sprite *bulb, struct sprite *sol,
                                     government_name_for_player(client.conn.playing));
   } else {
     fc_snprintf(cBuf, sizeof(cBuf), "%s (%s)\n%s", _("Revolution"), "Ctrl+Shift+R",
-                                    _("None"));
-  }        
+                                    Q_("?gov:None"));
+  }
   copy_chars_to_string16(pBuf->info_label, cBuf);
       
   widget_redraw(pBuf);
@@ -332,7 +331,7 @@ void set_indicator_icons(struct sprite *bulb, struct sprite *sol,
   if (NULL == client.conn.playing) {
     /* TRANS: Research report action */
     fc_snprintf(cBuf, sizeof(cBuf), "%s (%s)\n%s (%d/%d)", _("Research"), "F6",
-                                    _("None"), 0, 0);
+                Q_("?tech:None"), 0, 0);
   } else if (A_UNSET != player_research_get(client.conn.playing)->researching) {
     /* TRANS: Research report action */
     fc_snprintf(cBuf, sizeof(cBuf), "%s (%s)\n%s (%d/%d)", _("Research"), "F6",
@@ -563,8 +562,8 @@ void redraw_unit_info_label(struct unit_list *punitlist)
 
           if (BORDERS_DISABLED != game.info.borders && !pCity) {
 	    const char *diplo_nation_plural_adjectives[DS_LAST] =
-                        {Q_("?nation:Neutral"), Q_("?nation:Hostile"),
-     			"" /* unused, DS_CEASEFIRE*/,
+                       {"" /* unused, DS_ARMISTICE */, Q_("?nation:Hostile"),
+                        "" /* unused, DS_CEASEFIRE */,
      			Q_("?nation:Peaceful"), Q_("?nation:Friendly"), 
      			Q_("?nation:Mysterious")};
             if (tile_owner(pTile) == client.conn.playing) {
@@ -574,12 +573,18 @@ void redraw_unit_info_label(struct unit_list *punitlist)
                 struct player_diplstate *ds
                   = player_diplstate_get(client.conn.playing,
                                          tile_owner(pTile));
-                if (DS_CEASEFIRE == ds->type){
+                if (DS_CEASEFIRE == ds->type) {
                   int turns = ds->turns_left;
-		  cat_snprintf(buffer, sizeof(buffer),
-		  	PL_("\n%s territory (%d turn ceasefire)",
-				"\n%s territory (%d turn ceasefire)", turns),
-		 		nation_adjective_for_player(tile_owner(pTile)), turns);
+                  cat_snprintf(buffer, sizeof(buffer),
+                        PL_("\n%s territory (%d turn ceasefire)",
+                            "\n%s territory (%d turn ceasefire)", turns),
+                        nation_adjective_for_player(tile_owner(pTile)), turns);
+                } else if (DS_ARMISTICE == ds->type) {
+                  int turns = ds->turns_left;
+                  cat_snprintf(buffer, sizeof(buffer),
+                        PL_("\n%s territory (%d turn armistice)",
+                            "\n%s territory (%d turn armistice)", turns),
+                        nation_adjective_for_player(tile_owner(pTile)), turns);
                 } else {
                   cat_snprintf(buffer, sizeof(buffer), _("\nTerritory of the %s %s"),
                     diplo_nation_plural_adjectives[ds->type],
@@ -597,8 +602,8 @@ void redraw_unit_info_label(struct unit_list *punitlist)
             struct player *pOwner = city_owner(pCity);
 /*            bool barrack = FALSE, airport = FALSE, port = FALSE;*/
 	    const char *diplo_city_adjectives[DS_LAST] =
-    			{Q_("?city:Neutral"), Q_("?city:Hostile"),
-     			  "" /*unused, DS_CEASEFIRE */, Q_("?city:Peaceful"),
+			{Q_("?city:Neutral"), Q_("?city:Hostile"),
+			  Q_("?city:Neutral"), Q_("?city:Peaceful"),
 			  Q_("?city:Friendly"), Q_("?city:Mysterious")};
 			  
 	    cat_snprintf(buffer, sizeof(buffer),
@@ -773,12 +778,12 @@ void redraw_unit_info_label(struct unit_list *punitlist)
                       utype_name_translation(pUType),
                       pUType->attack_strength,
                       pUType->defense_strength,
-                      move_points_text(pUType->move_rate, NULL, NULL, FALSE),
+                      move_points_text(pUType->move_rate, FALSE),
                       (vetname != NULL ? "\n" : ""),
                       (vetname != NULL ? vetname : ""),
                       unit_activity_text(aunit),
                       aunit->hp, pUType->hp,
-                      pHome_City ? city_name(pHome_City) : _("None"));
+                      pHome_City ? city_name(pHome_City) : Q_("?homecity:None"));
 
 	  pBuf_Surf = create_surf(tileset_full_tile_width(tileset),
                                   tileset_full_tile_height(tileset), SDL_SWSURFACE);
@@ -895,7 +900,8 @@ void redraw_unit_info_label(struct unit_list *punitlist)
 	hide_scrollbar(pInfo_Window->private_data.adv_dlg->pScroll);
       }
 
-      if (NULL != client.conn.playing) {
+      if (!client_is_observer()
+          && (!client.conn.playing->ai_controlled || ai_manual_turn_done)) {
         char buf[256];
         fc_snprintf(buf, sizeof(buf), "%s\n%s\n%s",
                     _("End of Turn"), _("Press"), _("Shift+Return"));
