@@ -1360,6 +1360,7 @@ void popup_worklist_editor(struct city *pCity, struct global_worklist *pGWL)
       fc_snprintf(cBuf, sizeof(cBuf), "%s", improvement_name_translation(pImprove));
       copy_chars_to_string16(pStr, cBuf);
       pStr->style |= TTF_STYLE_BOLD;
+
       pText_Name = create_text_surf_smaller_that_w(pStr, pIcon->w - 4);
   
       if (is_wonder(pImprove)) {
@@ -1446,7 +1447,7 @@ void popup_worklist_editor(struct city *pCity, struct global_worklist *pGWL)
   
       copy_chars_to_string16(pStr, cBuf);
       pStr->style &= ~TTF_STYLE_BOLD;
-  
+
       pText = create_text_surf_from_str16(pStr);
 
       /*-----------------*/
@@ -1522,7 +1523,7 @@ void popup_worklist_editor(struct city *pCity, struct global_worklist *pGWL)
 		    _("(%d/%d/%s)\n%d/%d %s\nnever"),
 		    pUnit->attack_strength,
                     pUnit->defense_strength,
-                    move_points_text(pUnit->move_rate, NULL, NULL, FALSE),
+                    move_points_text(pUnit->move_rate, TRUE),
 		    pCity->shield_stock, utype_build_shield_cost(un),
 	  	    PL_("shield","shields", utype_build_shield_cost(un)));
         } else {
@@ -1530,7 +1531,7 @@ void popup_worklist_editor(struct city *pCity, struct global_worklist *pGWL)
 		    _("(%d/%d/%s)\n%d/%d %s\n%d %s"),
 		    pUnit->attack_strength,
                     pUnit->defense_strength,
-                    move_points_text(pUnit->move_rate, NULL, NULL, FALSE),
+                    move_points_text(pUnit->move_rate, TRUE),
 		    pCity->shield_stock, utype_build_shield_cost(un), 
 	  	    PL_("shield","shields", utype_build_shield_cost(un)),
 		    turns, PL_("turn", "turns", turns));
@@ -1540,7 +1541,7 @@ void popup_worklist_editor(struct city *pCity, struct global_worklist *pGWL)
 		    _("(%d/%d/%s)\n%d %s"),
 		    pUnit->attack_strength,
                     pUnit->defense_strength,
-                    move_points_text(pUnit->move_rate, NULL, NULL, FALSE),
+                    move_points_text(pUnit->move_rate, TRUE),
 		    utype_build_shield_cost(un),
 		    PL_("shield","shields", utype_build_shield_cost(un)));
       }
