@@ -18,13 +18,16 @@
 /* utility */
 #include "mem.h"
 
+/* gui main header */
+#include "gui_stub.h"
+
 #include "colors.h"
 
 /****************************************************************************
   Allocate a color (adjusting it for our colormap if necessary on paletted
   systems) and return a pointer to it.
 ****************************************************************************/
-struct color *color_alloc(int r, int g, int b)
+struct color *gui_color_alloc(int r, int g, int b)
 {
   struct color *color = fc_malloc(sizeof(*color));
 
@@ -39,7 +42,7 @@ struct color *color_alloc(int r, int g, int b)
 /****************************************************************************
   Free a previously allocated color.  See color_alloc.
 ****************************************************************************/
-void color_free(struct color *color)
+void gui_color_free(struct color *color)
 {
   /* PORTME */
   free(color);
