@@ -17,6 +17,10 @@
 #endif
 
 #ifdef AUDIO_SDL
+/* Though it would happily compile without this include,
+ * it is needed for sound to work as long as SDL-1.2 mixer is
+ * being used. It defines "main" macro to rename our main() so that
+ * it can install SDL's own. */
 #include "SDL.h"
 #endif
 
@@ -54,6 +58,7 @@
 #include "version.h"
 
 /* client */
+#include "chatline_common.h"
 #include "client_main.h"
 #include "climisc.h"
 #include "clinet.h"
@@ -255,21 +260,22 @@ static void print_usage(const char *argv0)
 }
 
 /**************************************************************************
-...
+  Parse and enact any client-specific options.
 **************************************************************************/
 static void parse_options(int argc, char **argv)
 {
-  int i;
+  int i = 1;
 
-  i = 1;
-  while (i < argc)
-  {
-    if (is_option("--help", argv[i]))
-    {
+  while (i < argc) {
+    if (is_option("--help", argv[i])) {
       print_usage(argv[0]);
       exit(EXIT_SUCCESS);
+    } else {
+      fc_fprintf(stderr, _("Unrecognized option: \"%s\"\n"), argv[i]);
+      exit(EXIT_FAILURE);
     }
-    i += 1;
+
+    i++;
   }
 }
 
@@ -315,7 +321,8 @@ int main(int argc, char **argv)
 void ui_main(int argc, char *argv[])
 {
   int i;
-  struct sprite *icon; 
+  struct sprite *icon;
+  const char *rev_ver;
 
   parse_options(argc, argv);
 
@@ -515,6 +522,14 @@ void ui_main(int argc, char *argv[])
 
   XtSetSensitive(toplevel, FALSE);
 
+  rev_ver = fc_git_revision();
+  if (rev_ver != NULL) {
+    char buffer[512];
+
+    fc_snprintf(buffer, sizeof(buffer), _("Commit: %s"), rev_ver);
+    output_window_append(ftc_client, buffer);
+  }
+
   XtAppMainLoop(app_context);
 }
 
@@ -728,10 +743,10 @@ void setup_widgets(void)
 
 
 
-  outputwindow_text= I_SW(XtVaCreateManagedWidget("outputwindowtext", 
-						  asciiTextWidgetClass, 
-						  bottom_form,
-						  NULL));
+  outputwindow_text = I_SW(XtVaCreateManagedWidget("outputwindowtext",
+                                                   asciiTextWidgetClass,
+                                                   bottom_form,
+                                                   NULL));
 
 
   inputline_text= XtVaCreateManagedWidget("inputlinetext", 
