@@ -75,9 +75,6 @@
 #define TEXT_ORDER_CITY_BUILD   0
 #define TEXT_ORDER_CITY_ADD_TO  1
 
-#define TEXT_ORDER_ROAD_ROAD      0
-#define TEXT_ORDER_ROAD_RAILROAD  1
-
 #define TEXT_ORDER_IRRIGATE_IRRIGATE   0
 #define TEXT_ORDER_IRRIGATE_FARMLAND   1
 #define TEXT_ORDER_IRRIGATE_CHANGE_TO  2
@@ -152,7 +149,7 @@ static struct MenuEntry view_menu_entries[]={
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
     { { N_("Terrain"), 0              },      "", MENU_VIEW_SHOW_TERRAIN, 0 },
     { { N_("Coastline"), 0            },      "", MENU_VIEW_SHOW_COASTLINE, 0 },
-    { { N_("Roads & Rails"), 0        },      "", MENU_VIEW_SHOW_ROADS_RAILS, 0 },
+    { { N_("Paths"), 0                },      "", MENU_VIEW_SHOW_PATHS, 0 },
     { { N_("Irrigation"), 0           },      "", MENU_VIEW_SHOW_IRRIGATION, 0 },
     { { N_("Mines"), 0                },      "", MENU_VIEW_SHOW_MINES, 0 },
     { { N_("Bases"), 0                },      "", MENU_VIEW_SHOW_BASES, 0 },
@@ -170,8 +167,7 @@ static struct MenuEntry view_menu_entries[]={
 static struct MenuEntry order_menu_entries[]={
     { { N_("Build City"),
         N_("Add to City"), 0          },     "b", MENU_ORDER_BUILD_CITY, 0 },
-    { { N_("Build Road"),
-        N_("Build Railroad"), 0       },     "r", MENU_ORDER_ROAD, 0 },
+    { { N_("Build Road"), 0           },     "r", MENU_ORDER_ROAD, 0 },
     { { N_("Build Irrigation"),
         N_("Build Farmland"),
         N_("Change to %s"), 0         },     "i", MENU_ORDER_IRRIGATE, 0 },
@@ -365,7 +361,7 @@ void real_menus_update(void)
 			 draw_city_names);
     menu_entry_sensitive(MENU_VIEW, MENU_VIEW_SHOW_TERRAIN, 1);
     menu_entry_sensitive(MENU_VIEW, MENU_VIEW_SHOW_COASTLINE, !draw_terrain);
-    menu_entry_sensitive(MENU_VIEW, MENU_VIEW_SHOW_ROADS_RAILS, 1);
+    menu_entry_sensitive(MENU_VIEW, MENU_VIEW_SHOW_PATHS, 1);
     menu_entry_sensitive(MENU_VIEW, MENU_VIEW_SHOW_IRRIGATION, 1);
     menu_entry_sensitive(MENU_VIEW, MENU_VIEW_SHOW_MINES, 1);
     menu_entry_sensitive(MENU_VIEW, MENU_VIEW_SHOW_BASES, 1);
@@ -396,6 +392,8 @@ void real_menus_update(void)
       struct tile *ptile = NULL;
       bool can_build;
       struct unit_list *punits = get_units_in_focus();
+      bool road_conn_possible;
+      struct road_type *proad;
 
       XtSetSensitive(menus[MENU_ORDER]->button, True);
 
@@ -403,10 +401,8 @@ void real_menus_update(void)
 
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_BUILD_CITY,
                            can_units_do(punits, unit_can_add_or_build_city));
-      menu_entry_sensitive(MENU_ORDER, MENU_ORDER_ROAD, 
-			   can_units_do_activity(punits, ACTIVITY_ROAD)
-			   || can_units_do_activity(punits,
-						    ACTIVITY_RAILROAD));
+      menu_entry_sensitive(MENU_ORDER, MENU_ORDER_ROAD,
+                           can_units_do_any_road(punits));
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_IRRIGATE, 
 			   can_units_do_activity(punits, ACTIVITY_IRRIGATE));
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_MINE, 
@@ -445,15 +441,39 @@ void real_menus_update(void)
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_CONVERT,
 			   units_can_convert(punits));
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_DISBAND,
-			   units_have_flag(punits, F_UNDISBANDABLE, FALSE));
+			   units_have_type_flag(punits, UTYF_UNDISBANDABLE,
+                                                FALSE));
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_AUTO_EXPLORE, 
 			   can_units_do_activity(punits, ACTIVITY_EXPLORE));
+
+      proad = road_by_compat_special(ROCO_ROAD);
+      if (proad != NULL) {
+        struct act_tgt tgt = { .type = ATT_ROAD,
+                               .obj.road = road_number(proad) }; 
+
+        road_conn_possible = can_units_do_connect(punits, ACTIVITY_GEN_ROAD,
+                                                  &tgt);
+      } else {
+        road_conn_possible = FALSE;
+      }
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_CONNECT_ROAD,
-			   can_units_do_connect(punits, ACTIVITY_ROAD));
+			   road_conn_possible);
+
+      proad = road_by_compat_special(ROCO_RAILROAD);
+      if (proad != NULL) {
+        struct act_tgt tgt = { .type = ATT_ROAD,
+                               .obj.road = road_number(proad) }; 
+
+        road_conn_possible = can_units_do_connect(punits, ACTIVITY_GEN_ROAD,
+                                                  &tgt);
+      } else {
+        road_conn_possible = FALSE;
+      }
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_CONNECT_RAIL,
-			   can_units_do_connect(punits, ACTIVITY_RAILROAD));
+                           road_conn_possible);
+
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_CONNECT_IRRIGATE,
-			   can_units_do_connect(punits, ACTIVITY_IRRIGATE));
+			   can_units_do_connect(punits, ACTIVITY_IRRIGATE, NULL));
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_GOTO_CITY,
 			   any_cities);
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_BUILD_WONDER,
@@ -466,7 +486,7 @@ void real_menus_update(void)
 			   can_units_do_diplomat_action(punits,
 							DIPLOMAT_ANY_ACTION));
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_NUKE,
-                           units_have_flag(punits, F_NUCLEAR, TRUE));
+                           units_have_type_flag(punits, UTYF_NUCLEAR, TRUE));
 
       /* FiXME: very odd, iterating for the first entry! */
       unit_list_iterate(punits, punit) {
@@ -477,7 +497,7 @@ void real_menus_update(void)
       tinfo = tile_terrain(ptile);
       can_build = !(tile_city(ptile));
 
-      if (units_have_flag(punits, F_CITIES, TRUE)) {
+      if (units_have_type_flag(punits, UTYF_CITIES, TRUE)) {
 	if (!can_build) {
 	  menu_entry_rename(MENU_ORDER, MENU_ORDER_BUILD_CITY,
 			  TEXT_ORDER_CITY_ADD_TO, NULL);
@@ -522,21 +542,13 @@ void real_menus_update(void)
 			  TEXT_ORDER_TRANSFORM_TERRAIN, NULL);
       }
 
-      if (units_have_flag(punits, F_PARATROOPERS, TRUE)) {
+      if (units_have_type_flag(punits, UTYF_PARATROOPERS, TRUE)) {
 	menu_entry_rename(MENU_ORDER, MENU_ORDER_POLLUTION,
 			  TEXT_ORDER_POLLUTION_PARADROP, NULL);
       } else {
 	menu_entry_rename(MENU_ORDER, MENU_ORDER_POLLUTION,
 			  TEXT_ORDER_POLLUTION_POLLUTION, NULL);
       }
-
-      if (tile_has_special(ptile, S_ROAD)) {
-	menu_entry_rename(MENU_ORDER, MENU_ORDER_ROAD,
-			  TEXT_ORDER_ROAD_RAILROAD, NULL);
-      } else {
-	menu_entry_rename(MENU_ORDER, MENU_ORDER_ROAD,
-			  TEXT_ORDER_ROAD_ROAD, NULL);
-      }
     }
   }
 }
@@ -653,7 +665,7 @@ static void view_menu_callback(Widget w, XtPointer client_data,
   case MENU_VIEW_SHOW_COASTLINE:
     key_coastline_toggle();
     break;
-  case MENU_VIEW_SHOW_ROADS_RAILS:
+  case MENU_VIEW_SHOW_PATHS:
     key_roads_rails_toggle();
     break;
   case MENU_VIEW_SHOW_IRRIGATION:
@@ -767,13 +779,31 @@ static void orders_menu_callback(Widget w, XtPointer client_data,
     key_unit_auto_explore();
     break;
   case MENU_ORDER_CONNECT_ROAD:
-    key_unit_connect(ACTIVITY_ROAD);
+    {
+      struct road_type *proad = road_by_compat_special(ROCO_ROAD);
+
+      if (proad != NULL) {
+        struct act_tgt tgt = { .type = ATT_ROAD,
+                               .obj.road = road_number(proad) }; 
+
+        key_unit_connect(ACTIVITY_GEN_ROAD, &tgt);
+      }
+    }
     break;
   case MENU_ORDER_CONNECT_RAIL:
-    key_unit_connect(ACTIVITY_RAILROAD);
+    {
+      struct road_type *proad = road_by_compat_special(ROCO_RAILROAD);
+
+      if (proad != NULL) {
+        struct act_tgt tgt = { .type = ATT_ROAD,
+                               .obj.road = road_number(proad) }; 
+
+        key_unit_connect(ACTIVITY_GEN_ROAD, &tgt);
+      }
+    }
     break;
   case MENU_ORDER_CONNECT_IRRIGATE:
-    key_unit_connect(ACTIVITY_IRRIGATE);
+    key_unit_connect(ACTIVITY_IRRIGATE, NULL);
     break;
   case MENU_ORDER_PATROL:
     key_unit_patrol();
@@ -1091,8 +1121,8 @@ void menu_entry_rename(enum MenuIndex menu, enum MenuID id, int var,
   char *item;
 
   for (i = 0; pmenu->entries[i].id != MENU_END_OF_LIST; i++) {
-    if(pmenu->entries[i].id==id) {
-      item=menu_entry_text(menu, i, var, terr);
+    if (pmenu->entries[i].id == id) {
+      item = menu_entry_text(menu, i, var, terr);
       XtVaSetValues(pmenu->entries[i].w, XtNlabel, item, NULL);
       return;
     }
