@@ -13,15 +13,18 @@
 #ifndef FC__HELPDLG_G_H
 #define FC__HELPDLG_G_H
 
+#include "gui_proto_constructor.h"
+
 /* This must be in same order as names in helpdata.c */
 enum help_page_type { HELP_ANY, HELP_TEXT, HELP_UNIT, HELP_IMPROVEMENT,
 		      HELP_WONDER, HELP_TECH, HELP_TERRAIN, HELP_BASE,
-                      HELP_SPECIALIST, HELP_GOVERNMENT, HELP_RULESET,
-                      HELP_NATIONS, HELP_LAST };
+                      HELP_ROAD, HELP_SPECIALIST, HELP_GOVERNMENT,
+                      HELP_RULESET, HELP_NATIONS, HELP_LAST };
 
-void popup_help_dialog_string(const char *item);
-void popup_help_dialog_typed(const char *item, enum help_page_type);
-void popdown_help_dialog(void);
+GUI_FUNC_PROTO(void, popup_help_dialog_string, const char *item)
+GUI_FUNC_PROTO(void, popup_help_dialog_typed, const char *item,
+               enum help_page_type)
+GUI_FUNC_PROTO(void, popdown_help_dialog, void)
 
 /* TRANS: "Overview" topic in built-in help */
 #define HELP_OVERVIEW_ITEM N_("?help:Overview")
@@ -41,6 +44,7 @@ void popdown_help_dialog(void);
 #define HELP_COMBAT_ITEM N_("Combat")
 #define HELP_ZOC_ITEM N_("Zones of Control")
 #define HELP_TECHS_ITEM N_("Technology")
+#define HELP_ROADS_ITEM N_("Roads")
 #define HELP_TERRAIN_ITEM N_("Terrain")
 #define HELP_WONDERS_ITEM N_("Wonders of the World")
 #define HELP_GOVERNMENT_ITEM N_("Government")
