@@ -121,7 +121,7 @@ static void define_tiles_within_rectangle(bool append)
   const int inc_y = (rec_h > 0 ? half_H : -half_H);
   int x, y, x2, y2, xx, yy;
   struct unit_list *units = unit_list_new();
-
+  const struct city *pcity;
   bool found_any_cities = FALSE;
 
   if (!append) {
@@ -156,8 +156,8 @@ static void define_tiles_within_rectangle(bool append)
 
       /*  Tile passed all tests; process it.
        */
-      if (NULL != tile_city(ptile)
-          && tile_owner(ptile) == client.conn.playing) {
+      pcity = tile_city(ptile);
+      if (pcity != NULL && city_owner(pcity) == client_player()) {
         mapdeco_set_highlight(ptile, TRUE);
         found_any_cities = tiles_hilited_cities = TRUE;
       }
@@ -483,7 +483,7 @@ void release_goto_button(int canvas_x, int canvas_y)
 
   if (keyboardless_goto_active && hover_state == HOVER_GOTO && ptile) {
     do_unit_goto(ptile);
-    set_hover_state(NULL, HOVER_NONE, ACTIVITY_LAST, ORDER_LAST);
+    set_hover_state(NULL, HOVER_NONE, ACTIVITY_LAST, NULL, ORDER_LAST);
     update_unit_info_label(get_units_in_focus());
   }
   keyboardless_goto_active = FALSE;
@@ -515,6 +515,7 @@ bool get_turn_done_button_state(void)
   return (can_client_issue_orders()
           && !client.conn.playing->ai_controlled
           && !client.conn.playing->phase_done
+          && is_player_phase(client.conn.playing, game.info.phase)
           && !agents_busy());
 }
 
