@@ -129,6 +129,11 @@ bool client_handle_packet(enum packet_type type, const void *packet)
     handle_player_info(packet);
     return TRUE;
 
+  case PACKET_TECH_GAINED:
+    handle_tech_gained(
+      ((const struct packet_tech_gained *)packet)->tech);
+    return TRUE;
+
   case PACKET_PLAYER_ATTRIBUTE_CHUNK:
     handle_player_attribute_chunk(packet);
     return TRUE;
@@ -264,6 +269,14 @@ bool client_handle_packet(enum packet_type type, const void *packet)
     handle_ruleset_unit(packet);
     return TRUE;
 
+  case PACKET_RULESET_UNIT_BONUS:
+    handle_ruleset_unit_bonus(packet);
+    return TRUE;
+
+  case PACKET_RULESET_UNIT_FLAG:
+    handle_ruleset_unit_flag(packet);
+    return TRUE;
+
   case PACKET_RULESET_GAME:
     handle_ruleset_game(packet);
     return TRUE;
@@ -280,6 +293,10 @@ bool client_handle_packet(enum packet_type type, const void *packet)
     handle_ruleset_tech(packet);
     return TRUE;
 
+  case PACKET_RULESET_TECH_FLAG:
+    handle_ruleset_tech_flag(packet);
+    return TRUE;
+
   case PACKET_RULESET_GOVERNMENT:
     handle_ruleset_government(packet);
     return TRUE;
@@ -292,6 +309,10 @@ bool client_handle_packet(enum packet_type type, const void *packet)
     handle_rulesets_ready();
     return TRUE;
 
+  case PACKET_RULESET_NATION_SETS:
+    handle_ruleset_nation_sets(packet);
+    return TRUE;
+
   case PACKET_RULESET_NATION_GROUPS:
     handle_ruleset_nation_groups(packet);
     return TRUE;
@@ -300,6 +321,13 @@ bool client_handle_packet(enum packet_type type, const void *packet)
     handle_ruleset_nation(packet);
     return TRUE;
 
+  case PACKET_NATION_AVAILABILITY:
+    handle_nation_availability(
+      ((const struct packet_nation_availability *)packet)->ncount,
+      ((const struct packet_nation_availability *)packet)->is_pickable,
+      ((const struct packet_nation_availability *)packet)->nationset_change);
+    return TRUE;
+
   case PACKET_RULESET_CITY:
     handle_ruleset_city(packet);
     return TRUE;
@@ -312,6 +340,10 @@ bool client_handle_packet(enum packet_type type, const void *packet)
     handle_ruleset_terrain(packet);
     return TRUE;
 
+  case PACKET_RULESET_TERRAIN_FLAG:
+    handle_ruleset_terrain_flag(packet);
+    return TRUE;
+
   case PACKET_RULESET_UNIT_CLASS:
     handle_ruleset_unit_class(packet);
     return TRUE;
@@ -320,6 +352,18 @@ bool client_handle_packet(enum packet_type type, const void *packet)
     handle_ruleset_base(packet);
     return TRUE;
 
+  case PACKET_RULESET_ROAD:
+    handle_ruleset_road(packet);
+    return TRUE;
+
+  case PACKET_RULESET_DISASTER:
+    handle_ruleset_disaster(packet);
+    return TRUE;
+
+  case PACKET_RULESET_TRADE:
+    handle_ruleset_trade(packet);
+    return TRUE;
+
   case PACKET_RULESET_CONTROL:
     handle_ruleset_control(packet);
     return TRUE;
