@@ -291,10 +291,10 @@ void cm_init(void)
 #ifdef GATHER_TIME_STATS
   memset(&performance, 0, sizeof(performance));
 
-  performance.greedy.wall_timer = new_timer(TIMER_USER, TIMER_ACTIVE);
+  performance.greedy.wall_timer = timer_new(TIMER_USER, TIMER_ACTIVE);
   performance.greedy.name = "greedy";
 
-  performance.opt.wall_timer = new_timer(TIMER_USER, TIMER_ACTIVE);
+  performance.opt.wall_timer = timer_new(TIMER_USER, TIMER_ACTIVE);
   performance.opt.name = "opt";
 #endif /* GATHER_TIME_STATS */
 }
@@ -326,8 +326,8 @@ void cm_free(void)
   print_performance(&performance.greedy);
   print_performance(&performance.opt);
 
-  free_timer(performance.greedy.wall_timer);
-  free_timer(performance.opt.wall_timer);
+  timer_destroy(performance.greedy.wall_timer);
+  timer_destroy(performance.opt.wall_timer);
   memset(&performance, 0, sizeof(performance));
 #endif /* GATHER_TIME_STATS */
 }
@@ -755,8 +755,17 @@ static struct cm_fitness evaluate_solution(struct cm_state *state,
 
   /* if this solution is not content, we have an estimate on min. luxuries */
   if (disorder) {
-    /* we have to consider the influence of specialists making one unhappy 
-       citizen content (because all other citizens are already unhappy) */
+    /* We have to consider the influence of each specialist in this
+       solution possibly 'hiding' a potential unhappy citizen who
+       could require luxuries.
+       Since we know the city is in disorder, we can discount most
+       effects that make citizens content, since they clearly weren't
+       sufficient.
+       This may not be sufficient luxury to make the city content (due
+       to military unhappiness etc), but certainly no less will do.
+       (Specialists may also be making angry citizens content, requiring
+       additional luxuries, but we don't try to consider that here; this
+       just means we might explore some solutions unnecessarily.) */
     state->min_luxury = surplus[O_LUXURY] 
        + game.info.happy_cost*MAX( city_specialists(pcity) - player_content_citizens(city_owner(pcity)), 0)
        + 1;
@@ -1619,9 +1628,16 @@ static bool choice_is_promising(struct cm_state *state, int newchoice)
     }
   } output_type_iterate_end;
  
- /* if we don't get the city content, we assume using every idle worker 
-     as specialist and the maximum producible luxury already computed 
-     see also evaluate_solution, where min_luxury is set */
+  /* If we don't get the city content, we assume using every idle worker 
+     as specialist and the maximum producible luxury already computed.
+     If this is less than the amount of luxury we calculated in
+     evaluate_solution() (where min_luxury is set), when we observed the
+     city in disorder, then this is clearly not worth pursuing.
+     (Since we're comparing to evaluate_solution()'s calculation, we
+     don't need to take effects, angry citizens etc into account here
+     either.)
+     FIXME: this heuristic will break in rulesets where specialists can
+     influence happiness other than by direct production of luxury. */
   int max_luxury = production[O_LUXURY]
         + game.info.happy_cost*MAX( specialists_in_solution(state, &state->current) + state->current.idle - player_content_citizens(city_owner(state->pcity)), 0);
  
@@ -1851,7 +1867,7 @@ static void begin_search(struct cm_state *state,
 			 const struct cm_parameter *parameter)
 {
 #ifdef GATHER_TIME_STATS
-  start_timer(performance.current->wall_timer);
+  timer_start(performance.current->wall_timer);
   performance.current->query_count++;
 #endif
 
@@ -1876,7 +1892,7 @@ static void begin_search(struct cm_state *state,
 static void end_search(struct cm_state *state)
 {
 #ifdef GATHER_TIME_STATS
-  stop_timer(performance.current->wall_timer);
+  timer_stop(performance.current->wall_timer);
 
 #ifdef PRINT_TIME_STATS_EVERY_QUERY
   print_performance(performance.current);
@@ -2240,7 +2256,7 @@ static void print_performance(struct one_perf *counts)
   double q;
   int queries, applies;
 
-  s = read_timer_seconds(counts->wall_timer);
+  s = timer_read_seconds(counts->wall_timer);
   ms = 1000.0 * s;
 
   queries = counts->query_count;
