@@ -40,16 +40,33 @@ static inline int single_move_cost(const struct pf_parameter *param,
                                    const struct tile *dest_tile)
 {
   if (!tile_city(dest_tile)
-      && BV_ISSET(param->unit_flags, F_TRIREME)
+      && BV_ISSET(param->unit_flags, UTYF_TRIREME)
       && !is_safe_ocean(dest_tile)) {
     return PF_IMPOSSIBLE_MC;
-  } else if (uclass_has_flag(param->uclass, UCF_TERRAIN_SPEED)) {
-    return map_move_cost(param->owner, src_tile, dest_tile);
   } else {
-    return SINGLE_MOVE;
+    return map_move_cost(param->owner, param->uclass, src_tile, dest_tile);
   }
 }
 
+/***************************************************************************
+  Whether there is potentially a transport that can be used as a waypoint
+  for the planned move.
+***************************************************************************/
+static bool
+pf_potential_unit_class_transport(const struct pf_parameter *param,
+                                  const struct tile *ptile)
+{
+  unit_list_iterate(ptile->units, punit) {
+    if (pplayers_allied(unit_owner(punit), param->owner)
+        && can_unit_type_transport(unit_type(punit), param->uclass)
+        && !unit_has_orders(punit)) {
+      return TRUE;
+    }
+  } unit_list_iterate_end;
+
+  return FALSE;
+}
+
 /* ===================== Move Cost Callbacks ========================= */
 
 /*************************************************************
@@ -163,7 +180,7 @@ static int normal_move_unit(const struct tile *ptile, enum direction8 dir,
   int move_cost;
 
   if (!is_native_tile_to_class(param->uclass, ptile1)) {
-    if (unit_class_transporter_capacity(ptile1, param->owner, param->uclass) > 0) {
+    if (pf_potential_unit_class_transport(param, ptile1)) {
       move_cost = SINGLE_MOVE;
     } else if (tile_city(ptile1)
                && (uclass_has_flag(param->uclass, UCF_BUILD_ANYWHERE)
@@ -175,7 +192,8 @@ static int normal_move_unit(const struct tile *ptile, enum direction8 dir,
       move_cost = PF_IMPOSSIBLE_MC;
     }
   } else if (!is_native_tile_to_class(param->uclass, ptile)) {
-    if (!BV_ISSET(param->unit_flags, F_MARINES)
+    if (!BV_ISSET(param->unit_flags, UTYF_MARINES)
+        && !uclass_has_flag(param->uclass, UCF_ATT_FROM_NON_NATIVE)
         && (is_non_allied_unit_tile(ptile1, param->owner) 
             || is_non_allied_city_tile(ptile1, param->owner))) {
       move_cost = PF_IMPOSSIBLE_MC;
@@ -203,8 +221,7 @@ static int land_attack_move(const struct tile *src_tile,
   if (!is_native_tile_to_class(param->uclass, tgt_tile)) {
 
     /* Any-to-Sea */
-    if (unit_class_transporter_capacity(tgt_tile, param->owner,
-                                        param->uclass) > 0) {
+    if (pf_potential_unit_class_transport(param, tgt_tile)) {
       move_cost = SINGLE_MOVE;
     } else {
       move_cost = PF_IMPOSSIBLE_MC;
@@ -215,7 +232,8 @@ static int land_attack_move(const struct tile *src_tile,
     if (!is_non_allied_unit_tile(tgt_tile, param->owner)
         && !is_non_allied_city_tile(tgt_tile, param->owner)) {
       move_cost = tile_terrain(tgt_tile)->movement_cost * SINGLE_MOVE;
-    } else if (BV_ISSET(param->unit_flags, F_MARINES)) {
+    } else if (BV_ISSET(param->unit_flags, UTYF_MARINES)
+               || uclass_has_flag(param->uclass, UCF_ATT_FROM_NON_NATIVE)) {
       /* Can attack!! */
       move_cost = single_move_cost(param, src_tile, tgt_tile);
     } else {
@@ -317,24 +335,27 @@ static int igter_move_unit(const struct tile *ptile,
   if (!is_native_tile_to_class(param->uclass, ptile1)) {
     if (unit_class_transporter_capacity(ptile1, param->owner,
                                         param->uclass) > 0) {
-      move_cost = MOVE_COST_ROAD;
+      move_cost = MOVE_COST_IGTER;
     } else {
       move_cost = PF_IMPOSSIBLE_MC;
     }
   } else if (!is_native_tile_to_class(param->uclass, ptile)) {
-    if (!BV_ISSET(param->unit_flags, F_MARINES)
+    if (!BV_ISSET(param->unit_flags, UTYF_MARINES)
+        && !uclass_has_flag(param->uclass, UCF_ATT_FROM_NON_NATIVE)
         && (is_non_allied_unit_tile(ptile1, param->owner) 
             || is_non_allied_city_tile(ptile1, param->owner))) {
       move_cost = PF_IMPOSSIBLE_MC;
+    } else if (game.info.slow_invasions
+               && tile_city(ptile1) == NULL) {
+      move_cost = param->move_rate;
     } else {
-      move_cost = MOVE_COST_ROAD;
+      move_cost = MOVE_COST_IGTER;
     }
-  } else if (uclass_has_flag(param->uclass, UCF_TERRAIN_SPEED)) {
-    move_cost = (map_move_cost(param->owner, ptile, ptile1) != 0
-                 ? MOVE_COST_ROAD : 0);
   } else {
-    move_cost = SINGLE_MOVE;
+    move_cost = map_move_cost(param->owner, param->uclass, ptile, ptile1);
+    move_cost = MIN(move_cost, MOVE_COST_IGTER);
   }
+
   return move_cost;
 }
 
@@ -354,16 +375,16 @@ static int reverse_igter_move_unit(const struct tile *tile0,
     if (unit_class_transporter_capacity(ptile, param->owner,
                                         param->uclass) > 0) {
       /* Landing */
-      move_cost = MOVE_COST_ROAD;
+      move_cost = MOVE_COST_IGTER;
     } else {
       move_cost = PF_IMPOSSIBLE_MC;
     }
   } else if (!is_native_tile_to_class(param->uclass, tile0)) {
     /* Boarding */
-    move_cost = MOVE_COST_ROAD;
+    move_cost = MOVE_COST_IGTER;
   } else {
     move_cost =
-	(ptile->move_cost[DIR_REVERSE(dir)] != 0 ? MOVE_COST_ROAD : 0);
+	(ptile->move_cost[DIR_REVERSE(dir)] != 0 ? MOVE_COST_IGTER : 0);
   }
   return move_cost;
 }
@@ -392,7 +413,7 @@ static int amphibious_move(const struct tile *ptile, enum direction8 dir,
     cost = amphibious->sea.get_MC(ptile, dir, ptile1, &amphibious->sea);
     scale = amphibious->sea_scale;
   } else if (src_ferry && dst_psng) {
-    /* Disembark; use land movement function to handle F_MARINES */
+    /* Disembark; use land movement function to handle UTYF_MARINES */
     cost = amphibious->land.get_MC(ptile, dir, ptile1, &amphibious->land);
     scale = amphibious->land_scale;
   } else if (src_ferry) {
@@ -684,18 +705,14 @@ static void pft_fill_default_parameter(struct pf_parameter *parameter,
 
   if (uclass_has_flag(punitclass, UCF_TERRAIN_SPEED)) {
     /* Unit is subject to terrain movement costs */
-    bv_special specials;
     bv_bases bases;
+    bv_roads roads;
 
-    BV_CLR_ALL(specials); /* This works at the moment, since road is
-                           * only special that affects is_native_terrain()
-                           * Even if tile contains road, we can safely
-                           * ignore it since movement cost for it is
-                           * certainly less than SINGLE_MOVE. */
     BV_CLR_ALL(bases);
+    BV_CLR_ALL(roads);
 
     terrain_type_iterate(pterrain) {
-      if (is_native_terrain(punittype, pterrain, specials, bases)) {
+      if (is_native_terrain(punittype, pterrain, bases, roads)) {
         /* Exact movement cost matters only if we can enter
          * the tile. */
         int mr = 2 * pterrain->movement_cost;
@@ -746,7 +763,7 @@ static void pft_fill_utype_default_parameter(struct pf_parameter *parameter,
   }
   parameter->owner = powner;
 
-  if (!BV_ISSET(parameter->unit_flags, F_CIVILIAN)) {
+  if (!BV_ISSET(parameter->unit_flags, UTYF_CIVILIAN)) {
     parameter->can_invade_tile = player_can_invade_tile;
   } else {
     parameter->can_invade_tile = NULL;
@@ -775,7 +792,7 @@ static void pft_fill_unit_default_parameter(struct pf_parameter *parameter,
   }
   parameter->owner = unit_owner(punit);
 
-  if (!BV_ISSET(parameter->unit_flags, F_CIVILIAN)) {
+  if (!BV_ISSET(parameter->unit_flags, UTYF_CIVILIAN)) {
     parameter->can_invade_tile = player_can_invade_tile;
   } else {
     parameter->can_invade_tile = NULL;
@@ -792,7 +809,7 @@ static void pft_fill_parameter(struct pf_parameter *parameter,
 {
   switch (utype_move_type(punittype)) {
   case UMT_LAND:
-    if (utype_has_flag(punittype, F_IGTER)) {
+    if (utype_has_flag(punittype, UTYF_IGTER)) {
       parameter->get_MC = igter_move_unit;
     } else {
       parameter->get_MC = normal_move_unit;
