@@ -24,6 +24,7 @@
 #endif
 
 #include <limits.h>
+#include <math.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -123,12 +124,7 @@ static bool enough_space(struct data_out *dout, size_t size)
 **************************************************************************/
 static bool enough_data(struct data_in *din, size_t size)
 {
-  if (dio_input_remaining(din) < size) {
-    din->too_short = TRUE;
-    return FALSE;
-  } else {
-    return TRUE;
-  }
+  return dio_input_remaining(din) >= size;
 }
 
 /**************************************************************************
@@ -171,10 +167,6 @@ void dio_input_init(struct data_in *din, const void *src, size_t src_size)
   din->src = src;
   din->src_size = src_size;
   din->current = 0;
-  din->too_short = FALSE;
-  din->bad_boolean = FALSE;
-  din->bad_string = FALSE;
-  din->bad_bit_string = FALSE;
 }
 
 /**************************************************************************
@@ -194,6 +186,29 @@ size_t dio_input_remaining(struct data_in *din)
   return din->src_size - din->current;
 }
 
+/**************************************************************************
+  Return the size of the data_type in bytes.
+**************************************************************************/
+size_t data_type_size(enum data_type type)
+{
+  switch (type) {
+  case DIOT_UINT8:
+  case DIOT_SINT8:
+    return 1;
+  case DIOT_UINT16:
+  case DIOT_SINT16:
+    return 2;
+  case DIOT_UINT32:
+  case DIOT_SINT32:
+    return 4;
+  case DIOT_LAST:
+    break;
+  }
+
+  fc_assert_msg(FALSE, "data_type %d not handled.", type);
+  return 0;
+}
+
 /**************************************************************************
    Skips 'n' bytes.
 **************************************************************************/
@@ -212,10 +227,16 @@ bool dio_input_skip(struct data_in *din, size_t size)
 **************************************************************************/
 void dio_put_uint8(struct data_out *dout, int value)
 {
-  if (enough_space(dout, 1)) {
-    uint8_t x = value;
+  uint8_t x = value;
+  FC_STATIC_ASSERT(sizeof(x) == 1, uint8_not_1_byte);
 
-    fc_assert(sizeof(x) == 1);
+  if ((int) x != value) {
+    log_error("Trying to put %d into 8 bits; "
+              "it will result %d at receiving side.",
+              value, (int) x);
+  }
+
+  if (enough_space(dout, 1)) {
     memcpy(ADD_TO_POINTER(dout->dest, dout->current), &x, 1);
     dout->current++;
   }
@@ -226,29 +247,100 @@ void dio_put_uint8(struct data_out *dout, int value)
 **************************************************************************/
 void dio_put_uint16(struct data_out *dout, int value)
 {
-  if (enough_space(dout, 2)) {
-    uint16_t x = htons(value);
+  uint16_t x = htons(value);
+  FC_STATIC_ASSERT(sizeof(x) == 2, uint16_not_2_bytes);
 
-    fc_assert(sizeof(x) == 2);
+  if ((int) ntohs(x) != value) {
+    log_error("Trying to put %d into 16 bits; "
+              "it will result %d at receiving side.",
+              value, (int) ntohs(x));
+  }
+
+  if (enough_space(dout, 2)) {
     memcpy(ADD_TO_POINTER(dout->dest, dout->current), &x, 2);
     dout->current += 2;
   }
 }
 
 /**************************************************************************
-  Insert value using 16 bits.
+  Insert value using 32 bits. May overflow.
 **************************************************************************/
 void dio_put_uint32(struct data_out *dout, int value)
 {
-  if (enough_space(dout, 4)) {
-    uint32_t x = htonl(value);
+  uint32_t x = htonl(value);
+  FC_STATIC_ASSERT(sizeof(x) == 4, uint32_not_4_bytes);
 
-    fc_assert(sizeof(x) == 4);
+  if ((int) ntohl(x) != value) {
+    log_error("Trying to put %d into 32 bits; "
+              "it will result %d at receiving side.",
+              value, (int) ntohl(x));
+  }
+
+  if (enough_space(dout, 4)) {
     memcpy(ADD_TO_POINTER(dout->dest, dout->current), &x, 4);
     dout->current += 4;
   }
 }
 
+/**************************************************************************
+  Insert value using 'size' bits. May overflow.
+**************************************************************************/
+void dio_put_type(struct data_out *dout, enum data_type type, int value)
+{
+  switch (type) {
+  case DIOT_UINT8:
+    dio_put_uint8(dout, value);
+    return;
+  case DIOT_UINT16:
+    dio_put_uint16(dout, value);
+    return;
+  case DIOT_UINT32:
+    dio_put_uint32(dout, value);
+    return;
+  case DIOT_SINT8:
+    dio_put_sint8(dout, value);
+    return;
+  case DIOT_SINT16:
+    dio_put_sint16(dout, value);
+    return;
+  case DIOT_SINT32:
+    dio_put_sint32(dout, value);
+    return;
+  case DIOT_LAST:
+    break;
+  }
+
+  fc_assert_msg(FALSE, "data_type %d not handled.", type);
+}
+
+/**************************************************************************
+  Insert value using 8 bits. May overflow.
+**************************************************************************/
+void dio_put_sint8(struct data_out *dout, int value)
+{
+  dio_put_uint8(dout, 0 <= value ? value : value + 0x100);
+}
+
+/**************************************************************************
+  Insert value using 16 bits. May overflow.
+**************************************************************************/
+void dio_put_sint16(struct data_out *dout, int value)
+{
+  dio_put_uint16(dout, 0 <= value ? value : value + 0x10000);
+}
+
+/**************************************************************************
+  Insert value using 32 bits. May overflow.
+**************************************************************************/
+void dio_put_sint32(struct data_out *dout, int value)
+{
+#if SIZEOF_INT == 4
+  dio_put_uint32(dout, value);
+#else
+  dio_put_uint32(dout, (0 <= value ? value : value + 0x100000000));
+#endif
+}
+
 /**************************************************************************
   Insert value 0 or 1 using 8 bits.
 **************************************************************************/
@@ -256,7 +348,7 @@ void dio_put_bool8(struct data_out *dout, bool value)
 {
   if (value != TRUE && value != FALSE) {
     log_error("Trying to put a non-boolean: %d", (int) value);
-    value = FALSE;
+    value = (value != FALSE);
   }
 
   dio_put_uint8(dout, value ? 1 : 0);
@@ -269,12 +361,46 @@ void dio_put_bool32(struct data_out *dout, bool value)
 {
   if (value != TRUE && value != FALSE) {
     log_error("Trying to put a non-boolean: %d", (int) value);
-    value = FALSE;
+    value = (value != FALSE);
   }
 
   dio_put_uint32(dout, value ? 1 : 0);
 }
 
+/****************************************************************************
+  Insert a float number, which is multiplied by 'float_factor' before
+  being encoded into an uint32.
+****************************************************************************/
+void dio_put_ufloat(struct data_out *dout, float value, int float_factor)
+{
+  uint32_t v = value * float_factor;
+
+  if (fabsf((float) v / float_factor - value) >= 1.0 / float_factor) {
+    log_error("Trying to put %f with factor %d in 32 bits; "
+              "it will result %f at receiving side.",
+              value, float_factor, (float) v / float_factor);
+  }
+
+  dio_put_uint32(dout, v);
+}
+
+/****************************************************************************
+  Insert a float number, which is multiplied by 'float_factor' before
+  being encoded into a sint32.
+****************************************************************************/
+void dio_put_sfloat(struct data_out *dout, float value, int float_factor)
+{
+  int32_t v = value * float_factor;
+
+  if (fabsf((float) v / float_factor - value) >= 1.0 / float_factor) {
+    log_error("Trying to put %f with factor %d in 32 bits; "
+              "it will result %f at receiving side.",
+              value, float_factor, (float) v / float_factor);
+  }
+
+  dio_put_sint32(dout, v);
+}
+
 /**************************************************************************
   Insert number of values brefore stop_value using 8 bits. Then
   insert values using 8 bits for each. stop_value is not required to
@@ -352,43 +478,6 @@ void dio_put_string(struct data_out *dout, const char *value)
   }
 }
 
-/**************************************************************************
-  Insert number of bits as 16 bit value and then insert bits. In incoming
-  value string each bit is represented by one character, value '1' indicating
-  TRUE bit.
-**************************************************************************/
-void dio_put_bit_string(struct data_out *dout, const char *value)
-{
-  /* Note that size_t is often an unsigned type, so we must be careful
-   * with the math when calculating 'bytes'. */
-  size_t bits = strlen(value), bytes;
-  size_t max = (unsigned short)(-1);
-
-  if (bits > max) {
-    fc_assert_msg(FALSE, "Bit string too long: %lu bits.",
-                  (unsigned long) bits);
-    bits = max;
-  }
-  bytes = (bits + 7) / 8;
-
-  if (enough_space(dout, bytes + 1)) {
-    size_t i;
-
-    dio_put_uint16(dout, bits);
-
-    for (i = 0; i < bits;) {
-      int bit, data = 0;
-
-      for (bit = 0; bit < 8 && i < bits; bit++, i++) {
-	if (value[i] == '1') {
-	  data |= (1 << bit);
-	}
-      }
-      dio_put_uint8(dout, data);
-    }
-  }
-}
-
 /**************************************************************************
   Insert tech numbers from value array as 8 bit values until there is value
   A_LAST or MAX_NUM_TECH_LIST tech numbers have been inserted.
@@ -461,11 +550,14 @@ bool dio_get_uint8(struct data_in *din, int *dest)
 {
   uint8_t x;
 
+  FC_STATIC_ASSERT(sizeof(x) == 1, uint8_not_byte);
+
   if (!enough_data(din, 1)) {
+    log_packet("Packet too short to read 1 byte");
+
     return FALSE;
   }
 
-  fc_assert(sizeof(x) == 1);
   memcpy(&x, ADD_TO_POINTER(din->src, din->current), 1);
   *dest = x;
   din->current++;
@@ -479,11 +571,14 @@ bool dio_get_uint16(struct data_in *din, int *dest)
 {
   uint16_t x;
 
+  FC_STATIC_ASSERT(sizeof(x) == 2, uint16_not_2_bytes);
+
   if (!enough_data(din, 2)) {
+    log_packet("Packet too short to read 2 bytes");
+
     return FALSE;
   }
 
-  fc_assert(sizeof(x) == 2);
   memcpy(&x, ADD_TO_POINTER(din->src, din->current), 2);
   *dest = ntohs(x);
   din->current += 2;
@@ -497,17 +592,46 @@ bool dio_get_uint32(struct data_in *din, int *dest)
 {
   uint32_t x;
 
+  FC_STATIC_ASSERT(sizeof(x) == 4, uint32_not_4_bytes);
+
   if (!enough_data(din, 4)) {
+    log_packet("Packet too short to read 4 bytes");
+
     return FALSE;
   }
 
-  fc_assert(sizeof(x) == 4);
   memcpy(&x, ADD_TO_POINTER(din->src, din->current), 4);
   *dest = ntohl(x);
   din->current += 4;
   return TRUE;
 }
 
+/**************************************************************************
+  Receive value using 'size' bits to dest.
+**************************************************************************/
+bool dio_get_type(struct data_in *din, enum data_type type, int *dest)
+{
+  switch (type) {
+  case DIOT_UINT8:
+    return dio_get_uint8(din, dest);
+  case DIOT_UINT16:
+    return dio_get_uint16(din, dest);
+  case DIOT_UINT32:
+    return dio_get_uint32(din, dest);
+  case DIOT_SINT8:
+    return dio_get_sint8(din, dest);
+  case DIOT_SINT16:
+    return dio_get_sint16(din, dest);
+  case DIOT_SINT32:
+    return dio_get_sint32(din, dest);
+  case DIOT_LAST:
+    break;
+  }
+
+  fc_assert_msg(FALSE, "data_type %d not handled.", type);
+  return FALSE;
+}
+
 /**************************************************************************
   Take boolean value from 8 bits.
 **************************************************************************/
@@ -520,7 +644,7 @@ bool dio_get_bool8(struct data_in *din, bool *dest)
   }
 
   if (ival != 0 && ival != 1) {
-    din->bad_boolean = TRUE;
+    log_packet("Got a bad boolean: %d", ival);
     return FALSE;
   }
 
@@ -540,7 +664,7 @@ bool dio_get_bool32(struct data_in *din, bool * dest)
   }
 
   if (ival != 0 && ival != 1) {
-    din->bad_boolean = TRUE;
+    log_packet("Got a bad boolean: %d", ival);
     return FALSE;
   }
 
@@ -548,6 +672,38 @@ bool dio_get_bool32(struct data_in *din, bool * dest)
   return TRUE;
 }
 
+/****************************************************************************
+  Get an unsigned float number, which have been multiplied by 'float_factor'
+  and encoded into an uint32 by dio_put_ufloat().
+****************************************************************************/
+bool dio_get_ufloat(struct data_in *din, float *dest, int float_factor)
+{
+  int ival;
+
+  if (!dio_get_uint32(din, &ival)) {
+    return FALSE;
+  }
+
+  *dest = (float) ival / float_factor;
+  return TRUE;
+}
+
+/****************************************************************************
+  Get a signed float number, which have been multiplied by 'float_factor'
+  and encoded into a sint32 by dio_put_sfloat().
+****************************************************************************/
+bool dio_get_sfloat(struct data_in *din, float *dest, int float_factor)
+{
+  int ival;
+
+  if (!dio_get_sint32(din, &ival)) {
+    return FALSE;
+  }
+
+  *dest = (float) ival / float_factor;
+  return TRUE;
+}
+
 /**************************************************************************
   Take value from 8 bits.
 **************************************************************************/
@@ -571,7 +727,7 @@ bool dio_get_sint8(struct data_in *din, int *dest)
 **************************************************************************/
 bool dio_get_sint16(struct data_in *din, int *dest)
 {
-  int tmp = 0;
+  int tmp;
 
   if (!dio_get_uint16(din, &tmp)) {
     return FALSE;
@@ -584,12 +740,34 @@ bool dio_get_sint16(struct data_in *din, int *dest)
   return TRUE;
 }
 
+/**************************************************************************
+  Take value from 32 bits.
+**************************************************************************/
+bool dio_get_sint32(struct data_in *din, int *dest)
+{
+  int tmp;
+
+  if (!dio_get_uint32(din, &tmp)) {
+    return FALSE;
+  }
+
+#if SIZEOF_INT != 4
+  if (tmp > 0x7fffffff) {
+    tmp -= 0x100000000;
+  }
+#endif
+
+  *dest = tmp;
+  return TRUE;
+}
+
 /**************************************************************************
   Take memory block directly.
 **************************************************************************/
 bool dio_get_memory(struct data_in *din, void *dest, size_t dest_size)
 {
   if (!enough_data(din, dest_size)) {
+    log_packet("Got too short memory");
     return FALSE;
   }
 
@@ -609,6 +787,7 @@ bool dio_get_string(struct data_in *din, char *dest, size_t max_dest_size)
   fc_assert(max_dest_size > 0);
 
   if (!enough_data(din, 1)) {
+    log_packet("Got a bad string");
     return FALSE;
   }
 
@@ -621,13 +800,12 @@ bool dio_get_string(struct data_in *din, char *dest, size_t max_dest_size)
   }
 
   if (offset >= remaining) {
-    din->too_short = TRUE;
-    din->bad_string = TRUE;
+    log_packet("Got a too short string");
     return FALSE;
   }
 
   if (!(*get_conv_callback) (dest, max_dest_size, c, offset)) {
-    din->bad_string = TRUE;
+    log_packet("Got a bad encoded string");
     return FALSE;
   }
 
@@ -635,49 +813,6 @@ bool dio_get_string(struct data_in *din, char *dest, size_t max_dest_size)
   return TRUE;
 }
 
-/**************************************************************************
-  Take bits and produce string containing chars '0' and '1'
-**************************************************************************/
-bool dio_get_bit_string(struct data_in *din, char *dest,
-			size_t max_dest_size)
-{
-  int npack = 0;		/* number claimed in packet */
-  int i;			/* iterate the bytes */
-
-  fc_assert(max_dest_size > 0);
-
-  if (!dio_get_uint16(din, &npack)) {
-    din->bad_bit_string = TRUE;
-    return FALSE;
-  }
-
-  if (npack >= max_dest_size) {
-      log_error("Have size for %lu, got %d",
-                (unsigned long) max_dest_size, npack);
-    din->bad_bit_string = TRUE;
-    return FALSE;
-  }
-
-  for (i = 0; i < npack;) {
-    int bit, byte_value;
-
-    if (!dio_get_uint8(din, &byte_value)) {
-      din->bad_bit_string = TRUE;
-      return FALSE;
-    }
-    for (bit = 0; bit < 8 && i < npack; bit++, i++) {
-      if (TEST_BIT(byte_value, bit)) {
-	dest[i] = '1';
-      } else {
-	dest[i] = '0';
-      }
-    }
-  }
-
-  dest[npack] = '\0';
-  return TRUE;
-}
-
 /**************************************************************************
   Take tech numbers until A_LAST encountered, or MAX_NUM_TECH_LIST techs
   retrieved.
@@ -688,6 +823,7 @@ bool dio_get_tech_list(struct data_in *din, int *dest)
 
   for (i = 0; i < MAX_NUM_TECH_LIST; i++) {
     if (!dio_get_uint8(din, &dest[i])) {
+      log_packet("Got a too short tech list");
       return FALSE;
     }
     if (dest[i] == A_LAST) {
@@ -712,6 +848,7 @@ bool dio_get_unit_list(struct data_in *din, int *dest)
 
   for (i = 0; i < MAX_NUM_UNIT_LIST; i++) {
     if (!dio_get_uint8(din, &dest[i])) {
+      log_packet("Got a too short unit list");
       return FALSE;
     }
     if (dest[i] == U_LAST) {
@@ -736,6 +873,7 @@ bool dio_get_building_list(struct data_in *din, int *dest)
 
   for (i = 0; i < MAX_NUM_BUILDING_LIST; i++) {
     if (!dio_get_uint8(din, &dest[i])) {
+      log_packet("Got a too short building list");
       return FALSE;
     }
     if (dest[i] == B_LAST) {
@@ -761,6 +899,7 @@ bool dio_get_worklist(struct data_in *din, struct worklist *pwl)
   worklist_init(pwl);
 
   if (!dio_get_uint8(din, &length)) {
+    log_packet("Got a bad worklist");
     return FALSE;
   }
 
@@ -770,9 +909,13 @@ bool dio_get_worklist(struct data_in *din, struct worklist *pwl)
 
     if (!dio_get_uint8(din, &kind)
         || !dio_get_uint8(din, &identifier)) {
+      log_packet("Got a too short worklist");
       return FALSE;
     }
 
+    /*
+     * FIXME: the value returned by universal_by_number() should be checked!
+     */
     worklist_append(pwl, universal_by_number(kind, identifier));
   }
 
@@ -843,9 +986,13 @@ bool dio_get_requirement(struct data_in *din, struct requirement *preq)
       || !dio_get_uint8(din, &range)
       || !dio_get_bool8(din, &survives)
       || !dio_get_bool8(din, &negated)) {
+    log_packet("Got a bad requirement");
     return FALSE;
   }
 
+  /*
+   * FIXME: the value returned by req_from_values() should be checked!
+   */
   *preq = req_from_values(type, range, survives, negated, value);
 
   return TRUE;
