@@ -18,11 +18,13 @@
 #include <string.h>
 
 /* utility */
+#include "astring.h"
 #include "fcintl.h"
 #include "log.h"
 #include "mem.h"
 #include "support.h"
 #include "shared.h" /* ARRAY_SIZE */
+#include "string_vector.h"
 
 /* common */
 #include "city.h"
@@ -120,7 +122,7 @@ static struct {
 /**************************************************************************
   Get a list of effects of this type.
 **************************************************************************/
-static struct effect_list *get_effects(enum effect_type effect_type)
+struct effect_list *get_effects(enum effect_type effect_type)
 {
   return ruleset_cache.effects[effect_type];
 }
@@ -291,6 +293,190 @@ void ruleset_cache_free(void)
   initialized = FALSE;
 }
 
+/****************************************************************************
+  Get the maximum effect value in this ruleset for the unit type for_unit
+  (that is, the sum of all positive effects clauses that apply specifically
+  to this type of unit -- this can be an overestimate in the case of
+  mutually exclusive effects).
+  for_unit can be NULL to get max effect value ignoring requirements.
+****************************************************************************/
+int effect_cumulative_max(enum effect_type type, struct unit_type *for_unit)
+{
+  struct effect_list *plist = ruleset_cache.tracker;
+  int value = 0;
+
+  if (plist) {
+    struct unit_class *pclass = NULL;
+
+    if (for_unit != NULL) {
+      pclass = utype_class(for_unit);
+    }
+
+    effect_list_iterate(plist, peffect) {
+      if (peffect->type == type && peffect->value > 0) {
+        if (for_unit == NULL) {
+          value += peffect->value;
+        } else {
+          bool failed = FALSE;
+
+          requirement_list_iterate(peffect->reqs, preq) {
+            if (preq->source.kind == VUT_UTYPE) {
+              if (preq->source.value.utype == for_unit) {
+                failed |= preq->negated;
+              } else {
+                failed |= (!preq->negated);
+              }
+            } else if (preq->source.kind == VUT_UTFLAG) {
+              if (utype_has_flag(for_unit, preq->source.value.unitflag)) {
+                failed |= preq->negated;
+              } else {
+                failed |= (!preq->negated);
+              }
+            } else if (preq->source.kind == VUT_UCLASS) {
+              if (preq->source.value.uclass == pclass) {
+                failed |= preq->negated;
+              } else {
+                failed |= (!preq->negated);
+              }
+            } else if (preq->source.kind == VUT_UCFLAG) {
+              if (uclass_has_flag(pclass, preq->source.value.unitclassflag)) {
+                failed |= preq->negated;
+              } else {
+                failed |= (!preq->negated);
+              }
+            }
+          } requirement_list_iterate_end;
+          requirement_list_iterate(peffect->nreqs, preq) {
+            if (preq->source.kind == VUT_UTYPE) {
+              if (preq->source.value.utype == for_unit) {
+                failed |= (!preq->negated);
+              } else {
+                failed |= preq->negated;
+              }
+            } else if (preq->source.kind == VUT_UTFLAG) {
+              if (utype_has_flag(for_unit, preq->source.value.unitflag)) {
+                failed |= (!preq->negated);
+              } else {
+                failed |= preq->negated;
+              }
+            } else if (preq->source.kind == VUT_UCLASS) {
+              if (preq->source.value.uclass == pclass) {
+                failed |= (!preq->negated);
+              } else {
+                failed |= preq->negated;
+              }
+            } else if (preq->source.kind == VUT_UCFLAG) {
+              if (uclass_has_flag(pclass, preq->source.value.unitclassflag)) {
+                failed |= (!preq->negated);
+              } else {
+                failed |= preq->negated;
+              }
+            }
+          } requirement_list_iterate_end;
+
+          if (!failed) {
+            value += peffect->value;
+          }
+        }
+      }
+    } effect_list_iterate_end;
+  }
+
+  return value;
+}
+
+/****************************************************************************
+  Get the minimum effect value in this ruleset for the unit type for_unit
+  (that is, the sum of all negative effects clauses that apply specifically
+  to this type of unit -- this can be an overestimate in the case of
+  mutually exclusive effects).
+  for_unit can be NULL to get min effect value ignoring requirements.
+****************************************************************************/
+int effect_cumulative_min(enum effect_type type, struct unit_type *for_unit)
+{
+  struct effect_list *plist = ruleset_cache.tracker;
+  int value = 0;
+
+  if (plist) {
+    struct unit_class *pclass = NULL;
+
+    if (for_unit != NULL) {
+      pclass = utype_class(for_unit);
+    }
+
+    effect_list_iterate(plist, peffect) {
+      if (peffect->type == type && peffect->value < 0) {
+        if (for_unit == NULL) {
+          value += peffect->value;
+        } else {
+          bool failed = FALSE;
+
+          requirement_list_iterate(peffect->reqs, preq) {
+            if (preq->source.kind == VUT_UTYPE) {
+              if (preq->source.value.utype == for_unit) {
+                failed |= preq->negated;
+              } else {
+                failed |= (!preq->negated);
+              }
+            } else if (preq->source.kind == VUT_UTFLAG) {
+              if (utype_has_flag(for_unit, preq->source.value.unitflag)) {
+                failed |= preq->negated;
+              } else {
+                failed |= (!preq->negated);
+              }
+            } else if (preq->source.kind == VUT_UCLASS) {
+              if (preq->source.value.uclass == pclass) {
+                failed |= preq->negated;
+              } else {
+                failed |= (!preq->negated);
+              }
+            } else if (preq->source.kind == VUT_UCFLAG) {
+              if (uclass_has_flag(pclass, preq->source.value.unitclassflag)) {
+                failed |= preq->negated;
+              } else {
+                failed |= (!preq->negated);
+              }
+            }
+          } requirement_list_iterate_end;
+          requirement_list_iterate(peffect->nreqs, preq) {
+            if (preq->source.kind == VUT_UTYPE) {
+              if (preq->source.value.utype == for_unit) {
+                failed |= (!preq->negated);
+              } else {
+                failed |= preq->negated;
+              }
+            } else if (preq->source.kind == VUT_UTFLAG) {
+              if (utype_has_flag(for_unit, preq->source.value.unitflag)) {
+                failed |= (!preq->negated);
+              } else {
+                failed |= preq->negated;
+              }
+            } else if (preq->source.kind == VUT_UCLASS) {
+              if (preq->source.value.uclass == pclass) {
+                failed |= (!preq->negated);
+              } else {
+                failed |= preq->negated;
+              }
+            } else if (preq->source.kind == VUT_UCFLAG) {
+              if (uclass_has_flag(pclass, preq->source.value.unitclassflag)) {
+                failed |= (!preq->negated);
+              } else {
+                failed |= preq->negated;
+              }
+            }
+          } requirement_list_iterate_end;
+
+          if (!failed) {
+            value += peffect->value;
+          }
+        }
+      }
+    } effect_list_iterate_end;
+  }
+
+  return value;
+}
+
 /****************************************************************************
   Receives a new effect.  This is called by the client when the packet
   arrives.
@@ -375,66 +561,6 @@ void send_ruleset_cache(struct conn_list *dest)
   } effect_list_iterate_end;
 }
 
-/**************************************************************************
-  Returns a buildable, non-obsolete building that can provide the effect.
-
-  Note: this function is an inefficient hack to be used by the old AI.  It
-  will never find wonders, since that's not what the AI wants.
-**************************************************************************/
-Impr_type_id ai_find_source_building(struct city *pcity,
-				     enum effect_type effect_type,
-                                     struct unit_class *uclass,
-                                     enum unit_move_type move)
-{
-  int greatest_value = 0;
-  struct impr_type *best_building = NULL;
-
-  /* There's no point in defining both of these as uclass is more restrictive
-   * than move_type */
-  fc_assert_ret_val(uclass == NULL || move == unit_move_type_invalid(),
-                    B_LAST);
-
-  effect_list_iterate(get_effects(effect_type), peffect) {
-    if (peffect->value > greatest_value) {
-      struct impr_type *building = NULL;
-      bool wrong_unit = FALSE;
-
-      requirement_list_iterate(peffect->reqs, preq) {
-        if (VUT_IMPROVEMENT == preq->source.kind && !preq->negated) {
-          building = preq->source.value.building;
-
-          if (!can_city_build_improvement_now(pcity, building)
-              || !is_improvement(building)) {          
-            building = NULL;
-            break;
-          }
-        }
-        if (VUT_UCLASS == preq->source.kind) {
-          if (uclass != NULL) {
-            if ((!preq->negated && preq->source.value.uclass != uclass)
-                || (preq->negated && preq->source.value.uclass == uclass)
-                || (move != unit_move_type_invalid()
-                    && uclass_move_type(preq->source.value.uclass) != move)) {
-              /* Effect requires other kind of unit than what we are interested about */
-              wrong_unit = TRUE;
-              break;
-            }
-          }
-        }
-      } requirement_list_iterate_end;
-      if (!wrong_unit && building != NULL) {
-        best_building = building;
-	greatest_value = peffect->value;
-      }
-    }
-  } effect_list_iterate_end;
-
-  if (best_building) {
-    return improvement_number(best_building);
-  }
-  return B_LAST;
-}
-
 /**************************************************************************
   Returns TRUE if the building has any effect bonuses of the given type.
 
@@ -714,7 +840,7 @@ int get_city_bonus(const struct city *pcity, enum effect_type effect_type)
 
   return get_target_bonus_effects(NULL,
 				  city_owner(pcity), pcity, NULL,
-				  NULL, NULL, NULL, NULL,
+				  city_tile(pcity), NULL, NULL, NULL,
 				  effect_type);
 }
 
@@ -984,7 +1110,8 @@ int get_current_construction_bonus(const struct city *pcity,
   Make user-friendly text for the source.  The text is put into a user
   buffer.
 **************************************************************************/
-void get_effect_req_text(struct effect *peffect, char *buf, size_t buf_len)
+void get_effect_req_text(const struct effect *peffect,
+                         char *buf, size_t buf_len)
 {
   buf[0] = '\0';
 
@@ -995,7 +1122,7 @@ void get_effect_req_text(struct effect *peffect, char *buf, size_t buf_len)
       continue;
     }
     if (buf[0] != '\0') {
-      fc_strlcat(buf, "+", buf_len);
+      fc_strlcat(buf, Q_("?req-list-separator:+"), buf_len);
     }
 
     universal_name_translation(&preq->source,
@@ -1003,6 +1130,25 @@ void get_effect_req_text(struct effect *peffect, char *buf, size_t buf_len)
   } requirement_list_iterate_end;
 }
 
+/****************************************************************************
+  Make user-friendly text for an effect list. The text is put into a user
+  astring.
+****************************************************************************/
+void get_effect_list_req_text(const struct effect_list *plist,
+                              struct astring *astr)
+{
+  struct strvec *psv = strvec_new();
+  char req_text[512];
+
+  effect_list_iterate(plist, peffect) {
+    get_effect_req_text(peffect, req_text, sizeof(req_text));
+    strvec_append(psv, req_text);
+  } effect_list_iterate_end;
+
+  strvec_to_and_list(psv, astr);
+  strvec_destroy(psv);
+}
+
 /**************************************************************************
   Iterate through all the effects in cache, and call callback for each.
   This is currently not very generic implementation, as we have only one user;
