@@ -28,21 +28,30 @@ extern "C" {
  * Nothing in this file should require anything else from the common/
  * directory! */
 
-#define MAX_NUM_PLAYER_SLOTS 128
+#define MAX_NUM_PLAYER_SLOTS 128 /* Used in the network protocol. */
 #define MAX_NUM_BARBARIANS   2
 #define MAX_NUM_PLAYERS      MAX_NUM_PLAYER_SLOTS - MAX_NUM_BARBARIANS
+/* Used in the network protocol. */
 #define MAX_NUM_CONNECTIONS (2 * (MAX_NUM_PLAYER_SLOTS))
-#define MAX_NUM_ITEMS   200     /* eg, unit_types */
-#define MAX_NUM_TECH_LIST 10
-#define MAX_NUM_UNIT_LIST 10
-#define MAX_NUM_BUILDING_LIST 10
+/* e.g. unit_types. Used in the network protocol. */
+#define MAX_NUM_ITEMS   200
+#define MAX_NUM_TECH_LIST 10 /* Used in the network protocol. */
+#define MAX_NUM_UNIT_LIST 10 /* Used in the network protocol. */
+#define MAX_NUM_BUILDING_LIST 10 /* Used in the network protocol. */
 #define MAX_LEN_VET_SHORT_NAME 8
-#define MAX_VET_LEVELS 20 /* see diplomat_success_vs_defender() */
-#define MAX_BASE_TYPES 32
-#define MAX_NUM_USER_UNIT_FLAGS 4
-#define MAX_NUM_LEADERS MAX_NUM_ITEMS
-#define MAX_NUM_NATION_GROUPS 128
-#define MAX_NUM_STARTPOS_NATIONS 1024
+/* Used in the network protocol. See diplomat_success_vs_defender() */
+#define MAX_VET_LEVELS 20
+#define MAX_BASE_TYPES 32 /* Used in the network protocol. */
+#define MAX_ROAD_TYPES 8 /* Used in the network protocol. */
+#define MAX_DISASTER_TYPES 10
+#define MAX_NUM_LEADERS MAX_NUM_ITEMS /* Used in the network protocol. */
+#define MAX_NUM_NATION_SETS 32 /* Used in the network protocol.
+                                * RULESET_NATION_SETS packet may become too big
+                                * if increased */
+#define MAX_NUM_NATION_GROUPS 128 /* Used in the network protocol. */
+/* Used in the network protocol -- nation count is a UINT16 */
+#define MAX_NUM_NATIONS MAX_UINT16
+#define MAX_NUM_STARTPOS_NATIONS 1024 /* Used in the network protocol. */
 
 /* Changing these will probably break network compatability. */
 #define MAX_LEN_NAME     48
@@ -59,7 +68,7 @@ extern "C" {
 /* symbol to flag missing numbers for better debugging */
 #define IDENTITY_NUMBER_ZERO (0)
 
-/* A bitvector for all player slots. */
+/* A bitvector for all player slots. Used in the network protocol. */
 BV_DEFINE(bv_player, MAX_NUM_PLAYER_SLOTS);
 
 /* Changing this breaks network compatibility. */
@@ -68,28 +77,51 @@ enum output_type_id {
 };
 
 /* Changing this enum will break savegame and network compatability. */
-enum unit_activity {
-  ACTIVITY_IDLE = 0,
-  ACTIVITY_POLLUTION = 1,
-  ACTIVITY_ROAD = 2,
-  ACTIVITY_MINE = 3,
-  ACTIVITY_IRRIGATE = 4,
-  ACTIVITY_FORTIFIED = 5,
-  ACTIVITY_FORTRESS = 6,
-  ACTIVITY_SENTRY = 7,
-  ACTIVITY_RAILROAD = 8,
-  ACTIVITY_PILLAGE = 9,
-  ACTIVITY_GOTO = 10,
-  ACTIVITY_EXPLORE = 11,
-  ACTIVITY_TRANSFORM = 12,
-  ACTIVITY_UNKNOWN = 13,		/* savegame compatability. */
-  ACTIVITY_AIRBASE = 14,
-  ACTIVITY_FORTIFYING = 15,
-  ACTIVITY_FALLOUT = 16,
-  ACTIVITY_PATROL_UNUSED = 17,		/* savegame compatability. */
-  ACTIVITY_BASE = 18,			/* building base */
-  ACTIVITY_LAST   /* leave this one last */
-};
+#define SPECENUM_NAME unit_activity
+#define SPECENUM_VALUE0 ACTIVITY_IDLE
+#define SPECENUM_VALUE0NAME "Idle"
+#define SPECENUM_VALUE1 ACTIVITY_POLLUTION
+#define SPECENUM_VALUE1NAME "Pollution"
+#define SPECENUM_VALUE2 ACTIVITY_OLD_ROAD
+#define SPECENUM_VALUE2NAME "Unused Road"
+#define SPECENUM_VALUE3 ACTIVITY_MINE
+#define SPECENUM_VALUE3NAME "Mine"
+#define SPECENUM_VALUE4 ACTIVITY_IRRIGATE
+#define SPECENUM_VALUE4NAME "Irrigate"
+#define SPECENUM_VALUE5 ACTIVITY_FORTIFIED
+#define SPECENUM_VALUE5NAME "Fortified"
+#define SPECENUM_VALUE6 ACTIVITY_FORTRESS
+#define SPECENUM_VALUE6NAME "Fortress"
+#define SPECENUM_VALUE7 ACTIVITY_SENTRY
+#define SPECENUM_VALUE7NAME "Sentry"
+#define SPECENUM_VALUE8 ACTIVITY_OLD_RAILROAD
+#define SPECENUM_VALUE8NAME "Unused Railroad"
+#define SPECENUM_VALUE9 ACTIVITY_PILLAGE
+#define SPECENUM_VALUE9NAME "Pillage"
+#define SPECENUM_VALUE10 ACTIVITY_GOTO
+#define SPECENUM_VALUE10NAME "Goto"
+#define SPECENUM_VALUE11 ACTIVITY_EXPLORE
+#define SPECENUM_VALUE11NAME "Explore"
+#define SPECENUM_VALUE12 ACTIVITY_TRANSFORM
+#define SPECENUM_VALUE12NAME "Transform"
+#define SPECENUM_VALUE13 ACTIVITY_UNKNOWN
+#define SPECENUM_VALUE13NAME "Unused"
+#define SPECENUM_VALUE14 ACTIVITY_AIRBASE
+#define SPECENUM_VALUE14NAME "Unused Airbase"
+#define SPECENUM_VALUE15 ACTIVITY_FORTIFYING
+#define SPECENUM_VALUE15NAME "Fortifying"
+#define SPECENUM_VALUE16 ACTIVITY_FALLOUT
+#define SPECENUM_VALUE16NAME "Fallout"
+#define SPECENUM_VALUE17 ACTIVITY_PATROL_UNUSED
+#define SPECENUM_VALUE17NAME "Unused Patrol"
+#define SPECENUM_VALUE18 ACTIVITY_BASE
+#define SPECENUM_VALUE18NAME "Base"
+#define SPECENUM_VALUE19 ACTIVITY_GEN_ROAD
+#define SPECENUM_VALUE19NAME "Road"
+#define SPECENUM_VALUE20 ACTIVITY_CONVERT
+#define SPECENUM_VALUE20NAME "Convert"
+#define SPECENUM_COUNT ACTIVITY_LAST
+#include "specenum_gen.h"
 
 enum adv_unit_task { AUT_NONE, AUT_AUTO_SETTLER, AUT_BUILD_CITY };
 
@@ -105,6 +137,7 @@ typedef int Nation_type_id;
 typedef int Unit_type_id;
 typedef int Base_type_id;
 typedef int Road_type_id;
+typedef int Disaster_type_id;
 typedef unsigned char citizens;
 
 struct advance;
@@ -125,12 +158,12 @@ struct unit;
 #define SP_MAX 20
 #define MAX_NUM_REQS 10
 
-#define MAX_NUM_RULESETS 16
-#define MAX_RULESET_NAME_LENGTH 64
+#define MAX_NUM_RULESETS 16 /* Used in the network protocol. */
+#define MAX_RULESET_NAME_LENGTH 64 /* Used in the network protocol. */
 #define RULESET_SUFFIX ".serv"
 
 /* Unit Class List, also 32-bit vector? */
-#define UCL_LAST 32
+#define UCL_LAST 32 /* Used in the network protocol. */
 typedef int Unit_Class_id;
 
 /* This has to be put here for now, otherwise movement.h and unittype.h
@@ -178,6 +211,16 @@ typedef int Unit_Class_id;
 #define SPECENUM_VALUE7NAME "Southeast"
 #include "specenum_gen.h"
 
+/* Used in the network protocol. */
+#define SPECENUM_NAME free_tech_method
+#define SPECENUM_VALUE0 FTM_GOAL
+#define SPECENUM_VALUE0NAME "Goal"
+#define SPECENUM_VALUE1 FTM_RANDOM
+#define SPECENUM_VALUE1NAME "Random"
+#define SPECENUM_VALUE2 FTM_CHEAPEST
+#define SPECENUM_VALUE2NAME "Cheapest"
+#include "specenum_gen.h"
+
 /* Some code requires compile time value for number of directions, and
  * cannot use specenum function call direction8_max(). */
 #define DIR8_MAGIC_MAX 8
@@ -228,17 +271,20 @@ enum req_problem_type {
 
 /* Originally in requirements.h, bumped up and revised to unify with
  * city_production and worklists.  Functions remain in requirements.c
- */
+ * Used in the network protocol. */
 typedef union {
   struct advance *advance;
   struct government *govern;
   struct impr_type *building;
   struct nation_type *nation;
+  struct nation_type *nationality;
   struct specialist *specialist;
   struct terrain *terrain;
   struct unit_class *uclass;
   struct unit_type *utype;
   struct base_type *base;
+  struct road_type *road;
+  struct resource *resource;
 
   enum ai_level ai_level;
   enum citytile_type citytile;
@@ -250,9 +296,12 @@ typedef union {
   int special;				/* enum tile_special_type */
   int unitclassflag;			/* enum unit_class_flag_id */
   int unitflag;				/* enum unit_flag_id */
+  int terrainflag;                      /* enum terrain_flag_id */
+  int techflag;                         /* enum tech_flag_id */
 } universals_u;
 
-/* The kind of universals_u (value_union_type was req_source_type). */
+/* The kind of universals_u (value_union_type was req_source_type).
+ * Used in the network protocol. */
 #define SPECENUM_NAME universals_n
 #define SPECENUM_VALUE0 VUT_NONE
 #define SPECENUM_VALUE0NAME "None"
@@ -300,9 +349,20 @@ typedef union {
 #define SPECENUM_VALUE19 VUT_CITYTILE
 #define SPECENUM_VALUE19NAME "CityTile"
 /* Keep this last. */
+#define SPECENUM_VALUE20 VUT_ROAD
+#define SPECENUM_VALUE20NAME "Road"
+#define SPECENUM_VALUE21 VUT_RESOURCE
+#define SPECENUM_VALUE21NAME "Resource"
+#define SPECENUM_VALUE22 VUT_TERRFLAG
+#define SPECENUM_VALUE22NAME "TerrainFlag"
+#define SPECENUM_VALUE23 VUT_NATIONALITY
+#define SPECENUM_VALUE23NAME "Nationality"
+#define SPECENUM_VALUE24 VUT_TECHFLAG
+#define SPECENUM_VALUE24NAME "TechFlag"
 #define SPECENUM_COUNT VUT_COUNT
 #include "specenum_gen.h"
 
+/* Used in the network protocol. */
 struct universal {
   universals_u value;
   enum universals_n kind;		/* formerly .type and .is_unit */
@@ -310,9 +370,12 @@ struct universal {
 
 struct ai_choice;			/* incorporates universals_u */
 
+/* Used in the network protocol. */
 BV_DEFINE(bv_bases, MAX_BASE_TYPES);
+BV_DEFINE(bv_roads, MAX_ROAD_TYPES);
 BV_DEFINE(bv_startpos_nations, MAX_NUM_STARTPOS_NATIONS);
 
+/* Used in the network protocol. */
 #define SPECENUM_NAME gui_type
 /* Used for options which do not belong to any gui. */
 #define SPECENUM_VALUE0 GUI_STUB
@@ -327,12 +390,11 @@ BV_DEFINE(bv_startpos_nations, MAX_NUM_STARTPOS_NATIONS);
 #define SPECENUM_VALUE4NAME "xaw"
 #define SPECENUM_VALUE5 GUI_QT
 #define SPECENUM_VALUE5NAME "qt"
-#define SPECENUM_VALUE6 GUI_WIN32
-#define SPECENUM_VALUE6NAME "win32"
-#define SPECENUM_VALUE7 GUI_FTWL
-#define SPECENUM_VALUE7NAME "ftwl"
+#define SPECENUM_VALUE6 GUI_WEB
+#define SPECENUM_VALUE6NAME "web"
 #include "specenum_gen.h"
 
+/* Used in the network protocol. */
 #define SPECENUM_NAME airlifting_style
 #define SPECENUM_BITWISE
 /* Like classical Freeciv.  One unit per turn. */
@@ -359,12 +421,43 @@ BV_DEFINE(bv_startpos_nations, MAX_NUM_STARTPOS_NATIONS);
 #define SPECENUM_VALUE1 REVEAL_MAP_DEAD
 #include "specenum_gen.h"
 
+/* Used in the network protocol. */
+#define SPECENUM_NAME gameloss_style
+#define SPECENUM_BITWISE
+/* Like classical Freeciv. No special effects. */
+#define SPECENUM_ZERO   GAMELOSS_STYLE_CLASSICAL
+/* Remaining cities are taken by barbarians. */
+#define SPECENUM_VALUE0 GAMELOSS_STYLE_BARB
+#define SPECENUM_VALUE0NAME "Barbarians"
+/* Try civil war. */
+#define SPECENUM_VALUE1 GAMELOSS_STYLE_CWAR
+#define SPECENUM_VALUE1NAME "CivilWar"
+/* Do some looting */
+#define SPECENUM_VALUE2 GAMELOSS_STYLE_LOOT
+#define SPECENUM_VALUE2NAME "Loot"
+#include "specenum_gen.h"
+
+/* Used in the network protocol. */
+#define SPECENUM_NAME tech_upkeep_style
+/* No upkeep */
+#define SPECENUM_VALUE0 TECH_UPKEEP_NONE
+#define SPECENUM_VALUE0NAME "None"
+/* Normal tech upkeep */
+#define SPECENUM_VALUE1 TECH_UPKEEP_BASIC
+#define SPECENUM_VALUE1NAME "Basic"
+/* Tech upkeep multiplied by number of cities */
+#define SPECENUM_VALUE2 TECH_UPKEEP_PER_CITY
+#define SPECENUM_VALUE2NAME "Cities"
+#include "specenum_gen.h"
+
+/* Used in the network protocol. */
 enum phase_mode_types {
   PMT_CONCURRENT = 0,
   PMT_PLAYERS_ALTERNATE,
   PMT_TEAMS_ALTERNATE
 };
 
+/* Used in the network protocol. */
 enum borders_mode {
   BORDERS_DISABLED = 0,
   BORDERS_ENABLED,
@@ -372,30 +465,35 @@ enum borders_mode {
   BORDERS_EXPAND,
 };
 
+/* Used in the network protocol. */
 enum diplomacy_mode {
   DIPLO_FOR_ALL,
   DIPLO_FOR_HUMANS,
   DIPLO_FOR_AIS,
+  DIPLO_NO_AIS,
+  DIPLO_NO_MIXED,
   DIPLO_FOR_TEAMS,
   DIPLO_DISABLED,
 };
 
+/* Used in the network protocol. */
 enum tile_special_type {
-  S_ROAD,
   S_IRRIGATION,
-  S_RAILROAD,
   S_MINE,
   S_POLLUTION,
   S_HUT,
-  S_OLD_FORTRESS,
-  S_RIVER,
   S_FARMLAND,
-  S_OLD_AIRBASE,
   S_FALLOUT,
 
   /* internal values not saved */
   S_LAST,
   S_RESOURCE_VALID = S_LAST,
+
+  S_OLD_FORTRESS,
+  S_OLD_AIRBASE,
+  S_OLD_ROAD,
+  S_OLD_RAILROAD,
+  S_OLD_RIVER
 };
 
 #ifdef __cplusplus
@@ -408,4 +506,27 @@ enum test_result {
   TR_ALREADY_SOLD
 };
 
+enum act_tgt_type { ATT_SPECIAL, ATT_BASE, ATT_ROAD };
+
+union act_tgt_obj {
+  enum tile_special_type spe;
+  Base_type_id base;
+  Road_type_id road;
+};
+
+struct act_tgt {
+  enum act_tgt_type type;
+  union act_tgt_obj obj;
+};
+
+/* Road type compatibility with old specials based roads.
+ * Used in the network protocol. */
+enum road_compat { ROCO_ROAD, ROCO_RAILROAD, ROCO_RIVER, ROCO_NONE };
+
+/*
+ * Maximum number of trade routes a city can have in any situation.
+ * Changing this changes network protocol.
+ */
+#define MAX_TRADE_ROUTES        5
+
 #endif /* FC__FC_TYPES_H */
