@@ -23,9 +23,12 @@ extern "C" {
 #include <sys/time.h>
 #endif
 
+/* utility */
+#include "fcthread.h"
 #include "shared.h"
 #include "timing.h"
 
+/* common */
 #include "connection.h"		/* struct conn_list */
 #include "fc_types.h"
 #include "player.h"
@@ -58,6 +61,12 @@ enum autosave_type {
   AS_INTERRUPT
 };
 
+struct user_flag
+{
+  char *name;
+  char *helptxt;
+};
+
 #define CONTAMINATION_POLLUTION 1
 #define CONTAMINATION_FALLOUT   2
 
@@ -124,6 +133,7 @@ struct civ_game {
       int killunhomed;    /* slowly killing unhomed units */
       int maxconnectionsperhost;
       int max_players;
+      char nationset[MAX_LEN_NAME];
       int mgr_distance;
       bool mgr_foodneeded;
       int mgr_nationchance;
@@ -145,14 +155,16 @@ struct civ_game {
       int revolution_length;
       int save_compress_level;
       enum fz_method save_compress_type;
-      int saveversion; /* save game version to use */
       int save_nturns;
       unsigned autosaves; /* FIXME: char would be enough, but current settings.c code wants to
                              write sizeof(unsigned) bytes */
       bool savepalace;
       bool homecaughtunits;
       char start_units[MAX_LEN_STARTUNIT];
+      bool start_city;
       int start_year;
+      int techloss_forgiveness;
+      int techloss_restore;
       int techlost_donor;
       int techlost_recv;
       int tcptimeout;
@@ -218,6 +230,10 @@ struct civ_game {
         char user_message[256];
       } meta_info;
 
+      struct {
+        fc_mutex city_list;
+      } mutexes;
+
       int first_timeout;
     } server;
   };
@@ -262,6 +278,9 @@ const char *textyear(int year);
 int generate_save_name(const char *format, char *buf, int buflen,
                        const char *reason);
 
+void user_flag_init(struct user_flag *flag);
+void user_flag_free(struct user_flag *flag);
+
 extern struct civ_game game;
 
 #define GAME_DEFAULT_SEED        0
@@ -273,6 +292,7 @@ extern struct civ_game game;
 #define GAME_MAX_GOLD            50000
 
 #define GAME_DEFAULT_START_UNITS  "ccwwx"
+#define GAME_DEFAULT_START_CITY  FALSE
 
 #define GAME_DEFAULT_DISPERSION  0
 #define GAME_MIN_DISPERSION      0
@@ -300,6 +320,8 @@ extern struct civ_game game;
 #define GAME_MIN_AIFILL              0
 #define GAME_MAX_AIFILL              GAME_MAX_MAX_PLAYERS
 
+#define GAME_DEFAULT_NATIONSET       ""
+
 #define GAME_DEFAULT_FOODBOX         100
 #define GAME_MIN_FOODBOX             1
 #define GAME_MAX_FOODBOX             10000
@@ -341,6 +363,14 @@ extern struct civ_game game;
 #define GAME_MIN_CONQUERCOST         0
 #define GAME_MAX_CONQUERCOST         100
 
+#define GAME_DEFAULT_TECHLOSSFG      -1
+#define GAME_MIN_TECHLOSSFG          -1
+#define GAME_MAX_TECHLOSSFG          200
+
+#define GAME_DEFAULT_TECHLOSSREST    50
+#define GAME_MIN_TECHLOSSREST        -1
+#define GAME_MAX_TECHLOSSREST        100
+
 #define GAME_DEFAULT_CITYMINDIST     2
 #define GAME_MIN_CITYMINDIST         1
 #define GAME_MAX_CITYMINDIST         9
@@ -361,6 +391,10 @@ extern struct civ_game game;
 #define GAME_DEFAULT_RAPTUREDELAY    1
 #define GAME_MIN_RAPTUREDELAY        1
 #define GAME_MAX_RAPTUREDELAY        99 /* 99 practicaly disables rapturing */
+
+#define GAME_DEFAULT_DISASTERS       10
+#define GAME_MIN_DISASTERS           0
+#define GAME_MAX_DISASTERS           1000
  
 #define GAME_DEFAULT_SAVEPALACE      TRUE
 
@@ -395,7 +429,8 @@ extern struct civ_game game;
 #define GAME_MIN_AQUEDUCTLOSS        0
 #define GAME_MAX_AQUEDUCTLOSS        100
 
-#define GAME_DEFAULT_KILLCITIZEN     (1 << UMT_LAND)
+#define GAME_DEFAULT_KILLSTACK       TRUE
+#define GAME_DEFAULT_KILLCITIZEN     TRUE
 
 #define GAME_DEFAULT_KILLUNHOMED     0
 #define GAME_MIN_KILLUNHOMED         0
@@ -500,9 +535,12 @@ extern struct civ_game game;
 
 #define GAME_DEFAULT_AUTOATTACK      FALSE
 
-#define GAME_DEFAULT_RULESETDIR      "default"
+#ifdef FREECIV_WEB
+#define GAME_DEFAULT_RULESETDIR      "fcweb"
+#else  /* FREECIV_WEB */
+#define GAME_DEFAULT_RULESETDIR      "classic"
+#endif /* FREECIV_WEB */
 
-#define GAME_DEFAULT_SAVEVERSION     0
 #define GAME_DEFAULT_SAVE_NAME       "freeciv"
 #define GAME_DEFAULT_SAVETURNS       1
 #define GAME_MIN_SAVETURNS           1
@@ -518,11 +556,11 @@ extern struct civ_game game;
 
 #define GAME_DEFAULT_EVENT_CACHE_TURNS    1
 #define GAME_MIN_EVENT_CACHE_TURNS        0
-#define GAME_MAX_EVENT_CACHE_TURNS        9
+#define GAME_MAX_EVENT_CACHE_TURNS        (GAME_MAX_END_TURN + 1)
 
 #define GAME_DEFAULT_EVENT_CACHE_MAX_SIZE   256
 #define GAME_MIN_EVENT_CACHE_MAX_SIZE        10
-#define GAME_MAX_EVENT_CACHE_MAX_SIZE      1000
+#define GAME_MAX_EVENT_CACHE_MAX_SIZE     20000
 
 #define GAME_DEFAULT_EVENT_CACHE_CHAT     TRUE
 
@@ -565,6 +603,9 @@ extern struct civ_game game;
 /* Max distance from the capital used to calculat the bribe cost. */
 #define GAME_UNIT_BRIBE_DIST_MAX 32
 
+/* Max number of recursive transports. */
+#define GAME_TRANSPORT_MAX_RECURSIVE 5
+
 /* ruleset settings */
 
 #define RS_MAX_VALUE                             10000
@@ -662,8 +703,6 @@ extern struct civ_game game;
 
 #define RS_DEFAULT_TIRED_ATTACK                  FALSE
 
-#define RS_DEFAULT_KILLSTACK                     TRUE
-
 #define RS_DEFAULT_BASE_BRIBE_COST               750
 #define RS_MIN_BASE_BRIBE_COST                   0
 #define RS_MAX_BASE_BRIBE_COST                   RS_MAX_VALUE
@@ -678,10 +717,6 @@ extern struct civ_game game;
 #define RS_MIN_UPGRADE_VETERAN_LOSS              0
 #define RS_MAX_UPGRADE_VETERAN_LOSS              MAX_VET_LEVELS
 
-#define RS_DEFAULT_TECH_UPKEEP_STYLE     0
-#define RS_MIN_TECH_UPKEEP_STYLE         0
-#define RS_MAX_TECH_UPKEEP_STYLE         1
-
 #define RS_DEFAULT_TECH_UPKEEP_DIVIDER   100
 #define RS_MIN_TECH_UPKEEP_DIVIDER       1
 #define RS_MAX_TECH_UPKEEP_DIVIDER       100000
