@@ -20,10 +20,8 @@ extern "C" {
 #include "fc_types.h"
 #include "tile.h"
 
-#define SINGLE_MOVE     3
-#define MOVE_COST_RIVER 1
-#define MOVE_COST_RAIL  0
-#define MOVE_COST_ROAD  1
+#define SINGLE_MOVE     (terrain_control.move_fragments)
+#define MOVE_COST_IGTER (terrain_control.igter_cost)
 
 struct unit_type;
 struct terrain;
@@ -33,7 +31,7 @@ enum unit_move_result {
   MR_DEATH,
   MR_PAUSE,
   MR_BAD_TYPE_FOR_CITY_TAKE_OVER,
-  MR_BAD_TYPE_FOR_CITY_TAKE_OVER_FROM_SEA,
+  MR_BAD_TYPE_FOR_CITY_TAKE_OVER_FROM_NON_NATIVE,
   MR_NO_WAR,    /* Can't move here without declaring war. */
   MR_PEACE,     /* Can't move here because of a peace treaty. */
   MR_ZOC,
@@ -44,6 +42,7 @@ enum unit_move_result {
   MR_DESTINATION_OCCUPIED_BY_NON_ALLIED_UNIT,
   MR_NO_TRANSPORTER_CAPACITY,
   MR_TRIREME,
+  MR_CANNOT_DISEMBARK,
 };
 
 int unit_move_rate(const struct unit *punit);
@@ -66,10 +65,10 @@ bool is_native_tile_to_class(const struct unit_class *punitclass,
                              const struct tile *ptile);
 bool is_native_terrain(const struct unit_type *punittype,
                        const struct terrain *pterrain,
-                       bv_special special, bv_bases bases);
+                       bv_bases bases, bv_roads roads);
 bool is_native_to_class(const struct unit_class *punitclass,
                         const struct terrain *pterrain,
-                        bv_special special, bv_bases bases);
+                        bv_bases bases, bv_roads roads);
 bool is_native_near_tile(const struct unit_class *uclass, const struct tile *ptile);
 bool can_exist_at_tile(const struct unit_type *utype,
                        const struct tile *ptile);
@@ -85,8 +84,7 @@ bool unit_can_move_to_tile(const struct unit *punit,
                            const struct tile *ptile,
                            bool igzoc);
 enum unit_move_result
-unit_move_to_tile_test(const struct unit_type *punittype,
-                       const struct player *unit_owner,
+unit_move_to_tile_test(const struct unit *punit,
                        enum unit_activity activity,
                        const struct tile *src_tile,
                        const struct tile *dst_tile,
@@ -97,10 +95,13 @@ bool can_unit_type_transport(const struct unit_type *transporter,
 int unit_class_transporter_capacity(const struct tile *ptile,
                                     const struct player *pplayer,
                                     const struct unit_class *pclass);
-struct unit *transport_from_tile(struct unit *punit, struct tile *ptile);
+struct unit *transport_from_tile(const struct unit *punit,
+                                 const struct tile *ptile);
 
-const char *move_points_text(int mp, const char *prefix, const char *none,
-                             bool align);
+void init_move_fragments(void);
+const char *move_points_text_full(int mp, bool reduce, const char *prefix,
+                                  const char *none, bool align);
+const char *move_points_text(int mp, bool reduce);
 
 #ifdef __cplusplus
 }
