@@ -31,10 +31,17 @@
 #include "government.h"
 #include "player.h"
 #include "tech.h"
+#include "traits.h"
 
 #include "nation.h"
 
 
+/* Nation set structure. */
+struct nation_set {
+  struct name_translation name;
+  char description[MAX_LEN_MSG];
+};
+
 /* Nation group structure. */
 struct nation_group {
   struct name_translation name;
@@ -45,9 +52,6 @@ struct nation_group {
 
       /* How much the AI will try to select a nation in the same group */
       int match;
-
-      /* Whether to treat the group as a self-contained subset */
-      bool set;
     } server;
 
     struct {
@@ -59,6 +63,8 @@ struct nation_group {
 
 static struct nation_type *nations = NULL;
 
+static int num_nation_sets;
+static struct nation_set nation_sets[MAX_NUM_NATION_SETS];
 static int num_nation_groups;
 static struct nation_group nation_groups[MAX_NUM_NATION_GROUPS];
 
@@ -190,6 +196,17 @@ const char *nation_plural_for_player(const struct player *pplayer)
   return nation_plural_translation(nation_of_player(pplayer));
 }
 
+/****************************************************************************
+  Return whether a nation is "pickable" -- whether players can select it
+  at game start.
+  (Client only function -- on the server, use client_can_pick_nation().)
+****************************************************************************/
+bool is_nation_pickable(const struct nation_type *nation)
+{
+  fc_assert_ret_val(!is_server(), FALSE);
+  return nation->client.is_pickable;
+}
+
 /****************************************************************************
   Return whether a nation is "playable"; i.e., whether a human player can
   choose this nation.  Barbarian and observer nations are not playable.
@@ -292,6 +309,18 @@ bool nation_leader_is_male(const struct nation_leader *pleader)
   return pleader->is_male;
 }
 
+/****************************************************************************
+  Return translated version of nation legend.
+****************************************************************************/
+const char *nation_legend_translation(const struct nation_type *pnation,
+                                      const char *legend)
+{
+  if (pnation->translation_domain == NULL) {
+    return _(legend);
+  }
+
+  return DG_(pnation->translation_domain, legend);
+}
 
 /****************************************************************************
   Nation default cities. The nation_city structure holds information about
@@ -560,7 +589,9 @@ static void nation_init(struct nation_type *pnation)
   memset(pnation, 0, sizeof(*pnation));
 
   pnation->item_number = pnation - nations;
+  pnation->translation_domain = NULL;
   pnation->leaders = nation_leader_list_new_full(nation_leader_destroy);
+  pnation->sets = nation_set_list_new();
   pnation->groups = nation_group_list_new();
 
   if (is_server()) {
@@ -569,6 +600,9 @@ static void nation_init(struct nation_type *pnation)
     pnation->server.civilwar_nations = nation_list_new();
     pnation->server.parent_nations = nation_list_new();
     pnation->server.conflicts_with = nation_list_new();
+    /* server.rgb starts out NULL */
+    pnation->server.traits = fc_calloc(TRAIT_COUNT,
+                                       sizeof(*pnation->server.traits));
   }
 }
 
@@ -578,7 +612,9 @@ static void nation_init(struct nation_type *pnation)
 static void nation_free(struct nation_type *pnation)
 {
   free(pnation->legend);
+  FC_FREE(pnation->translation_domain);
   nation_leader_list_destroy(pnation->leaders);
+  nation_set_list_destroy(pnation->sets);
   nation_group_list_destroy(pnation->groups);
 
   if (is_server()) {
@@ -586,6 +622,8 @@ static void nation_free(struct nation_type *pnation)
     nation_list_destroy(pnation->server.civilwar_nations);
     nation_list_destroy(pnation->server.parent_nations);
     nation_list_destroy(pnation->server.conflicts_with);
+    rgbcolor_destroy(pnation->server.rgb);
+    FC_FREE(pnation->server.traits);
   }
 
   memset(pnation, 0, sizeof(*pnation));
@@ -635,6 +673,246 @@ int city_style_of_nation(const struct nation_type *pnation)
   return pnation->city_style;
 }
 
+/****************************************************************************
+  Returns nation's player color preference, or NULL if none.
+  Server only function.
+****************************************************************************/
+const struct rgbcolor *nation_color(const struct nation_type *pnation)
+{
+  NATION_CHECK(pnation, return NULL);
+  return pnation->server.rgb;
+}
+
+/****************************************************************************
+  Return the number of nation sets.
+****************************************************************************/
+int nation_set_count(void)
+{
+  return num_nation_sets;
+}
+
+/****************************************************************************
+  Return the nation set index.
+****************************************************************************/
+int nation_set_index(const struct nation_set *pset)
+{
+  fc_assert_ret_val(NULL != pset, -1);
+  return pset - nation_sets;
+}
+
+/****************************************************************************
+  Return the nation set index.
+****************************************************************************/
+int nation_set_number(const struct nation_set *pset)
+{
+  return nation_set_index(pset);
+}
+
+/****************************************************************************
+  Add new set into the array of nation sets.
+****************************************************************************/
+struct nation_set *nation_set_new(const char *set_name,
+                                  const char *set_rule_name,
+                                  const char *set_description)
+{
+  struct nation_set *pset;
+
+  if (MAX_NUM_NATION_SETS <= num_nation_sets) {
+    log_error("Too many nation sets (%d is the maximum).",
+              MAX_NUM_NATION_SETS);
+    return NULL;
+  }
+
+  /* Print the name and truncate if needed. */
+  pset = nation_sets + num_nation_sets;
+  names_set(&pset->name, NULL, set_name, set_rule_name);
+  (void) sz_loud_strlcpy(pset->description, set_description,
+                         "Nation set description \"%s\" too long; truncating.");
+  if (NULL != nation_set_by_rule_name(rule_name(&pset->name))) {
+    log_error("Duplicate nation set name %s.", rule_name(&pset->name));
+    return NULL;
+  }
+  if (NULL != nation_group_by_rule_name(rule_name(&pset->name))) {
+    log_error("Nation set name %s is already used for a group.",
+              rule_name(&pset->name));
+    return NULL;
+  }
+
+  num_nation_sets++;
+
+  return pset;
+}
+
+/****************************************************************************
+  Return the nation set with the given index.
+
+  This function returns NULL for an out-of-range index (some callers
+  rely on this).
+****************************************************************************/
+struct nation_set *nation_set_by_number(int id)
+{
+  if (id < 0 || id >= num_nation_sets) {
+    return NULL;
+  }
+  return nation_sets + id;
+}
+
+/****************************************************************************
+  Return the nation set that has the given (untranslated) rule name.
+  Returns NULL if no set is found.
+****************************************************************************/
+struct nation_set *nation_set_by_rule_name(const char *name)
+{
+  const char *qname = Qn_(name);
+
+  nation_sets_iterate(pset) {
+    if (0 == fc_strcasecmp(rule_name(&pset->name), qname)) {
+      return pset;
+    }
+  } nation_sets_iterate_end;
+
+  return NULL;
+}
+
+/****************************************************************************
+  Return the untranslated name of a nation set (including qualifier, if any).
+  You usually want nation_set_rule_name() instead.
+  You don't have to free the return pointer.
+****************************************************************************/
+const char *nation_set_untranslated_name(const struct nation_set *pset)
+{
+  fc_assert_ret_val(NULL != pset, NULL);
+  return untranslated_name(&pset->name);
+}
+
+/****************************************************************************
+  Return the (untranslated) rule name of a nation set.
+  You don't have to free the return pointer.
+****************************************************************************/
+const char *nation_set_rule_name(const struct nation_set *pset)
+{
+  fc_assert_ret_val(NULL != pset, NULL);
+  return rule_name(&pset->name);
+}
+
+/****************************************************************************
+  Return the translated name of a nation set.
+  You don't have to free the return pointer.
+****************************************************************************/
+const char *nation_set_name_translation(const struct nation_set *pset)
+{
+  fc_assert_ret_val(NULL != pset, NULL);
+  return name_translation(&pset->name);
+}
+
+/****************************************************************************
+  Return the (untranslated) user description of a nation set.
+  You don't have to free the return pointer.
+****************************************************************************/
+const char *nation_set_description(const struct nation_set *pset)
+{
+  fc_assert_ret_val(NULL != pset, NULL);
+  return pset->description;
+}
+
+/****************************************************************************
+  Check if the given nation is in a given set
+****************************************************************************/
+bool nation_is_in_set(const struct nation_type *pnation,
+                      const struct nation_set *pset)
+{
+  fc_assert_ret_val(NULL != pnation, FALSE);
+
+  nation_set_list_iterate(pnation->sets, aset) {
+    if (aset == pset) {
+      return TRUE;
+    }
+  } nation_set_list_iterate_end;
+  return FALSE;
+}
+
+/****************************************************************************
+  Returns the nation set that would be selected by the given value of the
+  'nationset' server setting.
+  This differs from nation_set_by_rule_name() for the empty string, where
+  the first (ruleset default) nationset will be used; and similarly for
+  a nationset not matched in the ruleset.
+  The knowledge of the interpretation of this setting encapsulated here is
+  required on both server and client.
+****************************************************************************/
+struct nation_set *nation_set_by_setting_value(const char *setting)
+{
+  struct nation_set *pset = NULL;
+
+  if (strlen(setting) > 0) {
+    pset = nation_set_by_rule_name(setting);
+  }
+  if (pset == NULL) {
+    /* Either no nation set specified, or the specified one isn't in the
+     * current ruleset. Default to the first nation set specified by
+     * the ruleset. */
+    pset = nation_set_by_number(0);
+  }
+  fc_assert(pset != NULL);
+
+  return pset;
+}
+
+/****************************************************************************
+  Nation set iterator.
+****************************************************************************/
+struct nation_set_iter {
+  struct iterator vtable;
+  struct nation_set *p, *end;
+};
+#define NATION_SET_ITER(p) ((struct nation_set_iter *)(p))
+
+/****************************************************************************
+  Implementation of iterator 'sizeof' function.
+****************************************************************************/
+size_t nation_set_iter_sizeof(void)
+{
+  return sizeof(struct nation_set_iter);
+}
+
+/****************************************************************************
+  Implementation of iterator 'next' function.
+****************************************************************************/
+static void nation_set_iter_next(struct iterator *iter)
+{
+  NATION_SET_ITER(iter)->p++;
+}
+
+/****************************************************************************
+  Implementation of iterator 'get' function.
+****************************************************************************/
+static void *nation_set_iter_get(const struct iterator *iter)
+{
+  return NATION_SET_ITER(iter)->p;
+}
+
+/****************************************************************************
+  Implementation of iterator 'valid' function.
+****************************************************************************/
+static bool nation_set_iter_valid(const struct iterator *iter)
+{
+  struct nation_set_iter *it = NATION_SET_ITER(iter);
+  return it->p < it->end;
+}
+
+/****************************************************************************
+  Implementation of iterator 'init' function.
+****************************************************************************/
+struct iterator *nation_set_iter_init(struct nation_set_iter *it)
+{
+  it->vtable.next = nation_set_iter_next;
+  it->vtable.get = nation_set_iter_get;
+  it->vtable.valid = nation_set_iter_valid;
+  it->p = nation_sets;
+  it->end = nation_sets + nation_set_count();
+  return ITERATOR(it);
+}
+
 /****************************************************************************
   Return the number of nation groups.
 ****************************************************************************/
@@ -668,22 +946,26 @@ struct nation_group *nation_group_new(const char *name)
   struct nation_group *pgroup;
 
   if (MAX_NUM_NATION_GROUPS <= num_nation_groups) {
-    log_error("Too many groups/sets of nations (%d is the maximum).",
+    log_error("Too many nation groups (%d is the maximum).",
               MAX_NUM_NATION_GROUPS);
     return NULL;
   }
 
   /* Print the name and truncate if needed. */
   pgroup = nation_groups + num_nation_groups;
-  name_set(&pgroup->name, name);
+  name_set(&pgroup->name, NULL, name);
   if (NULL != nation_group_by_rule_name(rule_name(&pgroup->name))) {
-    log_error("Duplicate nation group/set name %s.", rule_name(&pgroup->name));
+    log_error("Duplicate nation group name %s.", rule_name(&pgroup->name));
+    return NULL;
+  }
+  if (NULL != nation_set_by_rule_name(rule_name(&pgroup->name))) {
+    log_error("Nation group name %s is already used for a set.",
+              rule_name(&pgroup->name));
     return NULL;
   }
 
   if (is_server()) {
     pgroup->server.match = 0;
-    pgroup->server.set = FALSE;
   }
   num_nation_groups++;
 
@@ -733,29 +1015,7 @@ void nation_group_set_match(struct nation_group *pgroup, int match)
 }
 
 /****************************************************************************
-  Set whether the group is considered a self-contained subset.
-  Server only function.
-****************************************************************************/
-void nation_group_set_set(struct nation_group *pgroup, bool is_set)
-{
-  fc_assert_ret(is_server());
-  fc_assert_ret(NULL != pgroup);
-  pgroup->server.set = is_set;
-}
-
-/****************************************************************************
-  Return whether the group is considered a self-contained subset.
-  Server only function.
-****************************************************************************/
-bool nation_group_is_a_set(const struct nation_group *pgroup)
-{
-  fc_assert_ret_val(is_server(), FALSE);
-  fc_assert_ret_val(NULL != pgroup, FALSE);
-  return pgroup->server.set;
-}
-
-/****************************************************************************
-  Return the untranslated name of a group of the nation (including qualifier,
+  Return the untranslated name of a nation group (including qualifier,
   if any).
   You usually want nation_group_rule_name() instead.
   You don't have to free the return pointer.
@@ -767,7 +1027,7 @@ const char *nation_group_untranslated_name(const struct nation_group *pgroup)
 }
 
 /****************************************************************************
-  Return the (untranslated) rule name of a group of the nation.
+  Return the (untranslated) rule name of a nation group.
   You don't have to free the return pointer.
 ****************************************************************************/
 const char *nation_group_rule_name(const struct nation_group *pgroup)
@@ -777,7 +1037,7 @@ const char *nation_group_rule_name(const struct nation_group *pgroup)
 }
 
 /****************************************************************************
-  Return the translated name of a group of the nation.
+  Return the translated name of a nation group.
   You don't have to free the return pointer.
 ****************************************************************************/
 const char *nation_group_name_translation(const struct nation_group *pgroup)
@@ -802,7 +1062,6 @@ bool nation_is_in_group(const struct nation_type *pnation,
   return FALSE;
 }
 
-
 /****************************************************************************
   Nation group iterator.
 ****************************************************************************/
@@ -859,19 +1118,19 @@ struct iterator *nation_group_iter_init(struct nation_group_iter *it)
 }
 
 /****************************************************************************
-  Initializes all nation group data.
+  Initializes all nation set/group data.
 ****************************************************************************/
-void nation_groups_init(void)
+void nation_sets_groups_init(void)
 {
-  num_nation_groups = 0;
+  num_nation_sets = num_nation_groups = 0;
 }
 
 /****************************************************************************
-  Frees and resets all nation group data.
+  Frees and resets all nation set/group data.
 ****************************************************************************/
-void nation_groups_free(void)
+void nation_sets_groups_free(void)
 {
-  num_nation_groups = 0;
+  num_nation_sets = num_nation_groups = 0;
 }
 
 /****************************************************************************
@@ -887,29 +1146,6 @@ bool can_conn_edit_players_nation(const struct connection *pconn,
 	           || pconn->access_level >= ALLOW_CTRL)));
 }
 
-/****************************************************************************
-  Set the nation groups that random nation selection is allowed to pick from.
-  If NULL, there are no restrictions.
-  Ownership of plist is passed to this module. Any previous list is freed.
-****************************************************************************/
-static struct nation_group_list *allowed_nation_groups = NULL;
-void set_allowed_nation_groups(struct nation_group_list *plist)
-{
-  if (allowed_nation_groups) {
-    nation_group_list_destroy(allowed_nation_groups);
-  }
-  allowed_nation_groups = plist;
-}
-
-/****************************************************************************
-  Get the nation groups that random nation selection is allowed to pick from.
-  If NULL, there are no restrictions.
-****************************************************************************/
-const struct nation_group_list *get_allowed_nation_groups(void)
-{
-  return allowed_nation_groups;
-}
-
 /****************************************************************************
   Returns how much two nations look good in the same game.
   Nations in the same group are considered to match, if that nation group
