@@ -68,6 +68,29 @@ static const char *ai_level_names[] = {
   NULL, N_("Hard"), N_("Cheating"), NULL, N_("Experimental")
 };
 
+/***************************************************************
+  Return the diplomatic state that cancelling a pact will
+  end up in.
+***************************************************************/
+enum diplstate_type cancel_pact_result(enum diplstate_type oldstate)
+{
+  switch(oldstate) {
+  case DS_NO_CONTACT: /* possible if someone declares war on our ally */
+  case DS_WAR: /* no change */
+  case DS_ARMISTICE:
+  case DS_CEASEFIRE:
+  case DS_PEACE:
+    return DS_WAR;
+  case DS_ALLIANCE:
+    return DS_ARMISTICE;
+  case DS_TEAM: /* no change */
+    return DS_TEAM;
+  default:
+    log_error("non-pact diplstate %d in cancel_pact_result", oldstate);
+    return DS_WAR; /* arbitrary */
+  }
+}
+
 /***************************************************************
   Returns true iff p1 can cancel treaty on p2.
 
@@ -89,7 +112,7 @@ enum dipl_reason pplayer_can_cancel_treaty(const struct player *p1,
   }
   if (player_diplstate_get(p1, p2)->has_reason_to_cancel == 0
       && get_player_bonus(p1, EFT_HAS_SENATE) > 0
-      && get_player_bonus(p1, EFT_NO_ANARCHY) == 0) {
+      && get_player_bonus(p1, EFT_NO_ANARCHY) <= 0) {
     return DIPL_SENATE_BLOCKING;
   }
   return DIPL_OK;
@@ -138,11 +161,12 @@ enum dipl_reason pplayer_can_make_treaty(const struct player *p1,
 {
   enum diplstate_type existing = player_diplstate_get(p1, p2)->type;
 
-  if (p1 == p2) {
-    return DIPL_ERROR; /* duh! */
+  if (players_on_same_team(p1, p2)) {
+    /* This includes the case p1 == p2 */
+    return DIPL_ERROR;
   }
-  if (get_player_bonus(p1, EFT_NO_DIPLOMACY)
-      || get_player_bonus(p2, EFT_NO_DIPLOMACY)) {
+  if (get_player_bonus(p1, EFT_NO_DIPLOMACY) > 0
+      || get_player_bonus(p2, EFT_NO_DIPLOMACY) > 0) {
     return DIPL_ERROR;
   }
   if (treaty == DS_WAR 
@@ -545,6 +569,7 @@ static void player_defaults(struct player *pplayer)
   player_slots_iterate(pslot) {
     pplayer->ai_common.love[player_slot_index(pslot)] = 1;
   } player_slots_iterate_end;
+  pplayer->ai_common.traits = NULL;
 
   pplayer->ai = NULL;
   pplayer->was_created = FALSE;
@@ -753,6 +778,7 @@ struct player *player_by_number(const int player_id)
 /****************************************************************************
   Set the player's nation to the given nation (may be NULL).  Returns TRUE
   iff there was a change.
+  Doesn't check if the nation is legal wrt nationset.
 ****************************************************************************/
 bool player_set_nation(struct player *pplayer, struct nation_type *pnation)
 {
@@ -855,7 +881,8 @@ struct player *player_by_user(const char *name)
 ****************************************************************************/
 bool can_player_see_unit_at(const struct player *pplayer,
 			    const struct unit *punit,
-			    const struct tile *ptile)
+                            const struct tile *ptile,
+                            bool is_transported)
 {
   struct city *pcity;
 
@@ -867,7 +894,7 @@ bool can_player_see_unit_at(const struct player *pplayer,
   /* Don't show non-allied units that are in transports.  This is logical
    * because allied transports can also contain our units.  Shared vision
    * isn't taken into account. */
-  if (unit_transported(punit) && unit_owner(punit) != pplayer
+  if (is_transported && unit_owner(punit) != pplayer
       && !pplayers_allied(pplayer, unit_owner(punit))) {
     return FALSE;
   }
@@ -899,7 +926,8 @@ bool can_player_see_unit_at(const struct player *pplayer,
 bool can_player_see_unit(const struct player *pplayer,
 			 const struct unit *punit)
 {
-  return can_player_see_unit_at(pplayer, punit, unit_tile(punit));
+  return can_player_see_unit_at(pplayer, punit, unit_tile(punit),
+                                unit_transported(punit));
 }
 
 /****************************************************************************
@@ -1152,6 +1180,9 @@ bool ai_fuzzy(const struct player *pplayer, bool normal_decision)
 const char *love_text(const int love)
 {
   if (love <= - MAX_AI_LOVE * 90 / 100) {
+    /* TRANS: These words should be adjectives which can fit in the sentence
+       "The x are y towards us"
+       "The Babylonians are respectful towards us" */
     return Q_("?attitude:Genocidal");
   } else if (love <= - MAX_AI_LOVE * 70 / 100) {
     return Q_("?attitude:Belligerent");
@@ -1294,7 +1325,7 @@ bool pplayers_non_attack(const struct player *pplayer,
 bool players_on_same_team(const struct player *pplayer1,
                           const struct player *pplayer2)
 {
-  return (pplayer1->team && pplayer1->team == pplayer2->team);
+  return pplayer1->team == pplayer2->team;
 }
 
 /**************************************************************************
