@@ -29,6 +29,7 @@ extern "C" {
 #include "shared.h"
 #include "spaceship.h"
 #include "tech.h"
+#include "traits.h"
 #include "unitlist.h"
 #include "vision.h"
 
@@ -43,7 +44,8 @@ enum plrcolor_mode {
   PLRCOL_PLR_ORDER,
   PLRCOL_PLR_RANDOM,
   PLRCOL_PLR_SET,
-  PLRCOL_TEAM_ORDER
+  PLRCOL_TEAM_ORDER,
+  PLRCOL_NATION_ORDER
 };
 
 struct player_slot;
@@ -131,6 +133,8 @@ struct player_ai {
   enum barbarian_type barbarian_type;
 
   int love[MAX_NUM_PLAYER_SLOTS];
+
+  struct ai_trait *traits;
 };
 
 /* Diplomatic states (how one player views another).
@@ -327,7 +331,8 @@ bool can_player_see_unit(const struct player *pplayer,
 			 const struct unit *punit);
 bool can_player_see_unit_at(const struct player *pplayer,
 			    const struct unit *punit,
-			    const struct tile *ptile);
+                            const struct tile *ptile,
+                            bool is_transported);
 
 bool can_player_see_units_in_city(const struct player *pplayer,
 				  const struct city *pcity);
@@ -360,6 +365,8 @@ bool ai_fuzzy(const struct player *pplayer, bool normal_decision);
 const char *diplstate_text(const enum diplstate_type type);
 const char *love_text(const int love);
 
+enum diplstate_type cancel_pact_result(enum diplstate_type oldstate);
+
 struct player_diplstate *player_diplstate_get(const struct player *plr1,
                                               const struct player *plr2);
 bool are_diplstates_equal(const struct player_diplstate *pds1,
