@@ -24,6 +24,7 @@
 #include "fc_interface.h"
 #include "game.h"
 #include "movement.h"
+#include "road.h"
 #include "unit.h"
 #include "unitlist.h"
 
@@ -198,6 +199,20 @@ bv_bases tile_bases(const struct tile *ptile)
   return ptile->bases;
 }
 
+/****************************************************************************
+  Returns a bit vector of the roads present at the tile.
+****************************************************************************/
+bv_roads tile_roads(const struct tile *ptile)
+{
+  if (!ptile) {
+    bv_roads empty;
+    BV_CLR_ALL(empty);
+    return empty;
+  }
+
+  return ptile->roads;
+}
+
 /****************************************************************************
   Set the bases on the tile to those present in the given bit vector.
 ****************************************************************************/
@@ -275,20 +290,70 @@ bool tile_has_claimable_base(const struct tile *ptile,
 }
 
 /****************************************************************************
-  Calculate defense bonus given by bases
+  Calculate defense bonus given for unit type by bases and roads
 ****************************************************************************/
-int tile_bases_defense_bonus(const struct tile *ptile,
-                             const struct unit_type *punittype)
+int tile_extras_defense_bonus(const struct tile *ptile,
+                              const struct unit_type *punittype)
+{
+  return tile_extras_class_defense_bonus(ptile, utype_class(punittype));
+}
+
+/****************************************************************************
+  Calculate defense bonus given for unit class by bases and roads
+****************************************************************************/
+int tile_extras_class_defense_bonus(const struct tile *ptile,
+                                    const struct unit_class *pclass)
 {
   int bonus = 0;
 
   base_type_iterate(pbase) {
     if (tile_has_base(ptile, pbase)
-        && is_native_base_to_uclass(pbase, utype_class(punittype))) {
+        && is_native_base_to_uclass(pbase, pclass)) {
       bonus += pbase->defense_bonus;
     }
   } base_type_iterate_end;
 
+  road_type_iterate(proad) {
+    if (tile_has_road(ptile, proad)
+        && is_native_road_to_uclass(proad, pclass)) {
+      bonus += proad->defense_bonus;
+    }
+  } road_type_iterate_end;
+
+  return bonus;
+}
+
+/****************************************************************************
+  Calculate output increment given by roads
+****************************************************************************/
+int tile_roads_output_incr(const struct tile *ptile, enum output_type_id o)
+{
+  int const_incr = 0;
+  int incr = 0;
+
+  road_type_iterate(proad) {
+    if (tile_has_road(ptile, proad)) {
+      const_incr += proad->tile_incr_const[o];
+      incr += proad->tile_incr[o];
+    }
+  } road_type_iterate_end;
+
+  return const_incr + incr * tile_terrain(ptile)->road_output_incr_pct[o] / 100;
+}
+
+/****************************************************************************
+  Calculate output bonus given by roads
+****************************************************************************/
+int tile_roads_output_bonus(const struct tile *ptile, enum output_type_id o)
+{
+  int bonus = 0;
+
+  road_type_iterate(proad) {
+    if (tile_has_road(ptile, proad)) {
+      bonus += proad->tile_bonus[o];
+    }
+  } road_type_iterate_end;
+
   return bonus;
 }
 
@@ -416,18 +481,15 @@ int tile_activity_time(enum unit_activity activity, const struct tile *ptile)
 
   /* ACTIVITY_BASE not handled here */
   fc_assert_ret_val(activity != ACTIVITY_BASE, FC_INFINITY);
+  fc_assert_ret_val(activity != ACTIVITY_GEN_ROAD, FC_INFINITY);
 
   switch (activity) {
   case ACTIVITY_POLLUTION:
     return pterrain->clean_pollution_time * ACTIVITY_FACTOR;
-  case ACTIVITY_ROAD:
-    return pterrain->road_time * ACTIVITY_FACTOR;
   case ACTIVITY_MINE:
     return pterrain->mining_time * ACTIVITY_FACTOR;
   case ACTIVITY_IRRIGATE:
     return pterrain->irrigation_time * ACTIVITY_FACTOR;
-  case ACTIVITY_RAILROAD:
-    return pterrain->rail_time * ACTIVITY_FACTOR;
   case ACTIVITY_TRANSFORM:
     return pterrain->transform_time * ACTIVITY_FACTOR;
   case ACTIVITY_FALLOUT:
@@ -438,39 +500,25 @@ int tile_activity_time(enum unit_activity activity, const struct tile *ptile)
 }
 
 /****************************************************************************
-  Time to complete the given activity on the given tile.
+  Time to complete the base building activity on the given tile.
 ****************************************************************************/
 int tile_activity_base_time(const struct tile *ptile,
                             Base_type_id base)
 {
-  return base_by_number(base)->build_time * ACTIVITY_FACTOR;
+  struct terrain *pterrain = tile_terrain(ptile);
+
+  return terrain_base_time(pterrain, base) * ACTIVITY_FACTOR;
 }
 
 /****************************************************************************
-  Clear all infrastructure (man-made specials) from the tile.
+  Time to complete the road building activity on the given tile.
 ****************************************************************************/
-static void tile_clear_unsupported_infrastructure(struct tile *ptile)
+int tile_activity_road_time(const struct tile *ptile,
+                            Road_type_id road)
 {
-  int i;
-  bool city_present = tile_city(ptile) != NULL;
-  struct terrain *pterr = tile_terrain(ptile);
-  bool ocean = is_ocean(pterr);
+  struct terrain *pterrain = tile_terrain(ptile);
 
-  for (i = 0; infrastructure_specials[i] != S_LAST; i++) {
-    switch (infrastructure_specials[i]) {
-    case S_ROAD:
-    case S_RAILROAD:
-      if (!city_present && pterr->road_time == 0) {
-	tile_clear_special(ptile, infrastructure_specials[i]);
-      }
-      break;
-    default:
-      if (ocean) {
-	tile_clear_special(ptile, infrastructure_specials[i]);
-      }
-      break;
-    }
-  }
+  return terrain_road_time(pterrain, road) * ACTIVITY_FACTOR;
 }
 
 /****************************************************************************
@@ -490,11 +538,9 @@ static void tile_clear_dirtiness(struct tile *ptile)
 void tile_change_terrain(struct tile *ptile, struct terrain *pterrain)
 {
   tile_set_terrain(ptile, pterrain);
-  tile_clear_unsupported_infrastructure(ptile);
 
   if (is_ocean(pterrain)) {
     /* The code can't handle these specials in ocean. */
-    tile_clear_special(ptile, S_RIVER);
     tile_clear_special(ptile, S_HUT);
   }
 
@@ -518,13 +564,20 @@ void tile_change_terrain(struct tile *ptile, struct terrain *pterrain)
     if (tile_has_base(ptile, pbase)
         && !is_native_tile_to_base(pbase, ptile)) {
       if (fc_funcs->destroy_base != NULL) {
+        /* Assume callback calls tile_remove_base() itself. */
 	fc_funcs->destroy_base(ptile, pbase);
       } else {
 	tile_remove_base(ptile, pbase);
       }
-      tile_remove_base(ptile, pbase);
     }
   } base_type_iterate_end;
+
+  road_type_iterate(proad) {
+    if (tile_has_road(ptile, proad)
+        && !is_native_tile_to_road(proad, ptile)) {
+      tile_remove_road(ptile, proad);
+    }
+  } road_type_iterate_end;
 }
 
 /****************************************************************************
@@ -534,6 +587,7 @@ void tile_change_terrain(struct tile *ptile, struct terrain *pterrain)
 void tile_add_special(struct tile *ptile, enum tile_special_type special)
 {
   fc_assert_ret(special != S_OLD_FORTRESS && special != S_OLD_AIRBASE);
+  fc_assert_ret(special != S_OLD_ROAD && special != S_OLD_RAILROAD && special != S_OLD_RIVER);
 
   tile_set_special(ptile, special);
 
@@ -544,18 +598,13 @@ void tile_add_special(struct tile *ptile, enum tile_special_type special)
   case S_IRRIGATION:
     tile_clear_special(ptile, S_MINE);
     break;
-  case S_RAILROAD:
-    tile_add_special(ptile, S_ROAD);
-    break;
   case S_MINE:
     tile_clear_special(ptile, S_IRRIGATION);
     tile_clear_special(ptile, S_FARMLAND);
     break;
 
-  case S_ROAD:
   case S_POLLUTION:
   case S_HUT:
-  case S_RIVER:
   case S_FALLOUT:
   default:
     break;
@@ -569,6 +618,7 @@ void tile_add_special(struct tile *ptile, enum tile_special_type special)
 void tile_remove_special(struct tile *ptile, enum tile_special_type special)
 {
   fc_assert_ret(special != S_OLD_FORTRESS && special != S_OLD_AIRBASE);
+  fc_assert_ret(special != S_OLD_ROAD && special != S_OLD_RAILROAD && special != S_OLD_RIVER);
 
   tile_clear_special(ptile, special);
 
@@ -576,15 +626,10 @@ void tile_remove_special(struct tile *ptile, enum tile_special_type special)
   case S_IRRIGATION:
     tile_clear_special(ptile, S_FARMLAND);
     break;
-  case S_ROAD:
-    tile_clear_special(ptile, S_RAILROAD);
-    break;
 
-  case S_RAILROAD:
   case S_MINE:
   case S_POLLUTION:
   case S_HUT:
-  case S_RIVER:
   case S_FARMLAND:
   case S_FALLOUT:
   default:
@@ -655,7 +700,7 @@ bool tile_apply_activity(struct tile *ptile, Activity_type_id act)
   case ACTIVITY_FALLOUT: 
     tile_clear_dirtiness(ptile);
     return TRUE;
-    
+
   case ACTIVITY_MINE:
     tile_mine(ptile);
     return TRUE;
@@ -664,31 +709,20 @@ bool tile_apply_activity(struct tile *ptile, Activity_type_id act)
     tile_irrigate(ptile);
     return TRUE;
 
-  case ACTIVITY_ROAD: 
-    if (!is_ocean_tile(ptile)
-	&& !tile_has_special(ptile, S_ROAD)) {
-      tile_set_special(ptile, S_ROAD);
-      return TRUE;
-    }
-    return FALSE;
-
-  case ACTIVITY_RAILROAD:
-    if (!is_ocean_tile(ptile)
-	&& !tile_has_special(ptile, S_RAILROAD)
-	&& tile_has_special(ptile, S_ROAD)) {
-      tile_set_special(ptile, S_RAILROAD);
-      return TRUE;
-    }
-    return FALSE;
-
   case ACTIVITY_TRANSFORM:
     tile_transform(ptile);
     return TRUE;
-    
+
+  case ACTIVITY_OLD_ROAD:
+  case ACTIVITY_OLD_RAILROAD:
   case ACTIVITY_FORTRESS:
-  case ACTIVITY_PILLAGE: 
-  case ACTIVITY_AIRBASE:   
+  case ACTIVITY_AIRBASE:
+    fc_assert(FALSE);
+    return FALSE;
+
+  case ACTIVITY_PILLAGE:
   case ACTIVITY_BASE:
+  case ACTIVITY_GEN_ROAD:
     /* do nothing  - not implemented */
     return FALSE;
 
@@ -697,6 +731,7 @@ bool tile_apply_activity(struct tile *ptile, Activity_type_id act)
   case ACTIVITY_SENTRY:
   case ACTIVITY_GOTO:
   case ACTIVITY_EXPLORE:
+  case ACTIVITY_CONVERT:
   case ACTIVITY_UNKNOWN:
   case ACTIVITY_FORTIFYING:
   case ACTIVITY_PATROL_UNUSED:
@@ -761,15 +796,18 @@ const char *tile_get_info_text(const struct tile *ptile,
     lb = TRUE;
   }
 
-  if (tile_has_special(ptile, S_RIVER)) {
-    if (lb) {
-      sz_strlcat(s, "\n");
-      lb = FALSE;
-    } else {
-      sz_strlcat(s, "/");
+  road_type_iterate(proad) {
+    if (tile_has_road(ptile, proad)
+        && road_has_flag(proad, RF_NATURAL)) {
+      if (lb) {
+        sz_strlcat(s, "\n");
+        lb = FALSE;
+      } else {
+        sz_strlcat(s, "/");
+      }
+      sz_strlcat(s, road_name_translation(proad));
     }
-    sz_strlcat(s, special_name_translation(S_RIVER));
-  }
+  } road_type_iterate_end;
   if (linebreaks & TILE_LB_RIVER_RESOURCE) {
     /* New linebreak requested */
     lb = TRUE;
@@ -812,6 +850,21 @@ bool tile_has_base(const struct tile *ptile, const struct base_type *pbase)
   return BV_ISSET(ptile->bases, base_index(pbase));
 }
 
+/****************************************************************************
+  Returns TRUE if the given tile has a base conflicting with the given one.
+****************************************************************************/
+bool tile_has_conflicting_base(const struct tile *ptile, const struct base_type *pbase)
+{
+  base_type_iterate(pconfl) {
+    if (BV_ISSET(pbase->conflicts, base_index(pconfl))
+        && tile_has_base(ptile, pconfl)) {
+      return TRUE;
+    }
+  } base_type_iterate_end;
+
+  return FALSE;
+}
+
 /****************************************************************************
   Returns TRUE if the given tile has any bases on it.
 ****************************************************************************/
@@ -823,6 +876,49 @@ bool tile_has_any_bases(const struct tile *ptile)
   return BV_ISSET_ANY(ptile->bases);
 }
 
+/****************************************************************************
+  Returns TRUE if the given tile has a road of given type on it.
+****************************************************************************/
+bool tile_has_road(const struct tile *ptile, const struct road_type *proad)
+{
+  return BV_ISSET(ptile->roads, road_index(proad));
+}
+
+/****************************************************************************
+  Tile has any river type
+****************************************************************************/
+bool tile_has_river(const struct tile *ptile)
+{
+  road_type_iterate(priver) {
+    if (tile_has_road(ptile, priver)
+        && road_has_flag(priver, RF_RIVER)) {
+      return TRUE;
+    }
+  } road_type_iterate_end;
+
+  return FALSE;
+}
+
+/****************************************************************************
+  Adds road to tile
+****************************************************************************/
+void tile_add_road(struct tile *ptile, const struct road_type *proad)
+{
+  if (proad != NULL) {
+    BV_SET(ptile->roads, road_index(proad));
+  }
+}
+
+/****************************************************************************
+  Removes road from tile if such exist
+****************************************************************************/
+void tile_remove_road(struct tile *ptile, const struct road_type *proad)
+{
+  if (proad != NULL) {
+    BV_CLR(ptile->roads, road_index(proad));
+  }
+}
+
 /****************************************************************************
   Returns a virtual tile. If ptile is given, the properties of this tile are
   copied, else it is completely blank (except for the unit list
@@ -841,6 +937,7 @@ struct tile *tile_virtual_new(const struct tile *ptile)
 
   BV_CLR_ALL(vtile->special);
   BV_CLR_ALL(vtile->bases);
+  BV_CLR_ALL(vtile->roads);
   vtile->resource = NULL;
   vtile->terrain = NULL;
   vtile->units = unit_list_new();
@@ -871,6 +968,12 @@ struct tile *tile_virtual_new(const struct tile *ptile)
       }
     } base_type_iterate_end;
 
+    road_type_iterate(proad) {
+      if (tile_has_road(ptile, proad)) {
+        tile_add_road(vtile, proad);
+      }
+    } road_type_iterate_end;
+
     vtile->resource = ptile->resource;
     vtile->terrain = ptile->terrain;
     vtile->worked = ptile->worked;
