@@ -44,28 +44,7 @@ static struct unit_class unit_classes[UCL_LAST];
    client/packhand.c (for the client)
 */
 
-static const char *flag_names[] = {
-  "TradeRoute", "HelpWonder", "IgZOC", "NonMil", "IgTer", 
-  "OneAttack", "Pikemen", "Horse", "IgWall", "FieldUnit", 
-  "AEGIS", "Marines", "Partial_Invis", "Settlers", "Diplomat",
-  "Trireme", "Nuclear", "Spy", "Transform", "Paratroopers",
-  "Cities", "No_Land_Attack",
-  "AddToCity", "Fanatic", "GameLoss", "Unique", "Unbribable", 
-  "Undisbandable", "SuperSpy", "NoHome", "NoVeteran", "Bombarder",
-  "CityBuster", "NoBuild", "BadWallAttacker", "BadCityDefender",
-  "Helicopter", "AirUnit", "Fighter", "BarbarianOnly", "Shield2Gold",
-  "Capturable", "Capturer"
-};
-static char *user_flag_names[MAX_NUM_USER_UNIT_FLAGS]
-  = { NULL, NULL, NULL, NULL };
-static const char *role_names[] = {
-  "FirstBuild", "Explorer", "Hut", "HutTech", "Partisan",
-  "DefendOk", "DefendGood", "AttackFast", "AttackStrong",
-  "Ferryboat", "Barbarian", "BarbarianTech", "BarbarianBoat",
-  "BarbarianBuild", "BarbarianBuildTech", "BarbarianLeader",
-  "BarbarianSea", "BarbarianSeaTech", "Cities", "Settlers",
-  "GameLoss", "Diplomat", "Hunter"
-};
+static struct user_flag user_type_flags[MAX_NUM_USER_UNIT_FLAGS];
 
 /**************************************************************************
   Return the first item of unit_types.
@@ -150,8 +129,8 @@ int utype_upkeep_cost(const struct unit_type *ut, struct player *pplayer,
 {
   int val = ut->upkeep[otype], gold_upkeep_factor;
 
-  if (get_player_bonus(pplayer, EFT_FANATICS)
-      && BV_ISSET(ut->flags, F_FANATIC)) {
+  if (BV_ISSET(ut->flags, UTYF_FANATIC)
+      && get_player_bonus(pplayer, EFT_FANATICS) > 0) {
     /* Special case: fanatics have no upkeep under fanaticism. */
     return 0;
   }
@@ -159,11 +138,11 @@ int utype_upkeep_cost(const struct unit_type *ut, struct player *pplayer,
   /* switch shield upkeep to gold upkeep if
      - the effect 'EFT_SHIELD2GOLD_FACTOR' is non-zero (it gives the
        conversion factor in percent) and
-     - the unit has the corresponding flag set (F_SHIELD2GOLD)
+     - the unit has the corresponding flag set (UTYF_SHIELD2GOLD)
      FIXME: Should the ai know about this? */
   gold_upkeep_factor = get_player_bonus(pplayer, EFT_SHIELD2GOLD_FACTOR);
   gold_upkeep_factor = (gold_upkeep_factor > 0) ? gold_upkeep_factor : 0;
-  if (gold_upkeep_factor > 0 && utype_has_flag(ut, F_SHIELD2GOLD)) {
+  if (gold_upkeep_factor > 0 && utype_has_flag(ut, UTYF_SHIELD2GOLD)) {
     switch (otype) {
       case O_GOLD:
         val = ceil((0.01 * gold_upkeep_factor) * ut->upkeep[O_SHIELD]);
@@ -207,14 +186,14 @@ bool uclass_has_flag(const struct unit_class *punitclass,
 **************************************************************************/
 bool utype_has_flag(const struct unit_type *punittype, int flag)
 {
-  fc_assert_ret_val(flag >= 0 && flag < F_LAST, FALSE);
+  fc_assert_ret_val(unit_type_flag_id_is_valid(flag), FALSE);
   return BV_ISSET(punittype->flags, flag);
 }
 
 /**************************************************************************
   Return whether the unit has the given flag.
 **************************************************************************/
-bool unit_has_type_flag(const struct unit *punit, enum unit_flag_id flag)
+bool unit_has_type_flag(const struct unit *punit, enum unit_type_flag_id flag)
 {
   return utype_has_flag(unit_type(punit), flag);
 }
@@ -251,7 +230,31 @@ bool unit_can_take_over(const struct unit *punit)
 bool utype_can_take_over(const struct unit_type *punittype)
 {
   return (uclass_has_flag(utype_class(punittype), UCF_CAN_OCCUPY_CITY)
-          && !utype_has_flag(punittype, F_CIVILIAN));
+          && !utype_has_flag(punittype, UTYF_CIVILIAN));
+}
+
+/****************************************************************************
+  Return TRUE iff the given cargo type has no restrictions on when it can
+  load onto the given transporter.
+  (Does not check that cargo is valid for transport!)
+****************************************************************************/
+bool utype_can_freely_load(const struct unit_type *pcargotype,
+                           const struct unit_type *ptranstype)
+{
+  return BV_ISSET(pcargotype->embarks,
+                  uclass_index(utype_class(ptranstype)));
+}
+
+/****************************************************************************
+  Return TRUE iff the given cargo type has no restrictions on when it can
+  unload from the given transporter.
+  (Does not check that cargo is valid for transport!)
+****************************************************************************/
+bool utype_can_freely_unload(const struct unit_type *pcargotype,
+                             const struct unit_type *ptranstype)
+{
+  return BV_ISSET(pcargotype->disembarks,
+                  uclass_index(utype_class(ptranstype)));
 }
 
 /****************************************************************************
@@ -384,11 +387,11 @@ const char *utype_values_string(const struct unit_type *punittype)
   fc_snprintf(buffer, sizeof(buffer), "%d/%d/%s",
               punittype->attack_strength,
               punittype->defense_strength,
-              move_points_text(punittype->move_rate, NULL, NULL, FALSE));
+              move_points_text(punittype->move_rate, TRUE));
   if (utype_fuel(punittype)) {
     cat_snprintf(buffer, sizeof(buffer), "(%s)",
                  move_points_text(punittype->move_rate * utype_fuel(punittype),
-                                  NULL, NULL, FALSE));
+                                  TRUE));
   }
   return buffer;
 }
@@ -553,91 +556,92 @@ struct unit_class *unit_class_by_rule_name(const char *s)
   return NULL;
 }
 
+/**************************************************************************
+  Initialize user unit type flags.
+**************************************************************************/
+void user_unit_type_flags_init(void)
+{
+  int i;
+
+  for (i = 0; i < MAX_NUM_USER_UNIT_FLAGS; i++) {
+    user_flag_init(&user_type_flags[i]);
+  }
+}
+
 /**************************************************************************
   Sets user defined name for unit flag.
 **************************************************************************/
-void set_user_unit_flag_name(enum unit_flag_id id, const char *name)
+void set_user_unit_type_flag_name(enum unit_type_flag_id id, const char *name,
+                                  const char *helptxt)
 {
-  int ufid = id - F_USER_FLAG_1;
+  int ufid = id - UTYF_USER_FLAG_1;
 
-  fc_assert_ret(id >= F_USER_FLAG_1 && id < F_LAST);
+  fc_assert_ret(id >= UTYF_USER_FLAG_1 && id <= UTYF_LAST_USER_FLAG);
 
-  if (user_flag_names[ufid] != NULL) {
-    free(user_flag_names[ufid]);
-    user_flag_names[ufid] = NULL;
+  if (user_type_flags[ufid].name != NULL) {
+    FC_FREE(user_type_flags[ufid].name);
+    user_type_flags[ufid].name = NULL;
   }
 
   if (name && name[0] != '\0') {
-    user_flag_names[ufid] = fc_strdup(name);
+    user_type_flags[ufid].name = fc_strdup(name);
+  }
+
+  if (user_type_flags[ufid].helptxt != NULL) {
+    free(user_type_flags[ufid].helptxt);
+    user_type_flags[ufid].helptxt = NULL;
+  }
+
+  if (helptxt && helptxt[0] != '\0') {
+    user_type_flags[ufid].helptxt = fc_strdup(helptxt);
   }
 }
 
 /**************************************************************************
-  Convert flag names to enum; case insensitive;
-  returns F_LAST if can't match.
+  Unit type flag name callback, called from specenum code.
 **************************************************************************/
-enum unit_flag_id unit_flag_by_rule_name(const char *s)
+char *unit_type_flag_id_name_cb(enum unit_type_flag_id flag)
 {
-  enum unit_flag_id i;
+  if (flag < UTYF_USER_FLAG_1 || flag > UTYF_LAST_USER_FLAG) {
+    return NULL;
+  }
 
-  fc_assert_ret_val(ARRAY_SIZE(flag_names) == F_USER_FLAG_1, F_LAST);
+  return user_type_flags[flag - UTYF_USER_FLAG_1].name;
+}
 
-  for (i = 0; i < F_USER_FLAG_1; i++) {
-    if (fc_strcasecmp(flag_names[i], s) == 0) {
-      return i;
+/**************************************************************************
+  Return the (untranslated) helptxt of the user unit flag.
+**************************************************************************/
+const char *unit_type_flag_helptxt(enum unit_type_flag_id id)
+{
+  fc_assert(id >= UTYF_USER_FLAG_1 && id <= UTYF_LAST_USER_FLAG);
+
+  return user_type_flags[id - UTYF_USER_FLAG_1].helptxt;
+}
+
+/**************************************************************************
+  Returns TRUE iff the unit type is unique and the player already has one.
+**************************************************************************/
+bool utype_player_already_has_this_unique(const struct player *pplayer,
+                                          const struct unit_type *putype)
+{
+  if (!utype_has_flag(putype, UTYF_UNIQUE)) {
+    /* This isn't a unique unit type. */
+    return FALSE;
+  }
+
+  unit_list_iterate(pplayer->units, existing_unit) {
+    if (putype == unit_type(existing_unit)) {
+      /* FIXME: This could be slow if we have lots of units. We could
+       * consider keeping an array of unittypes updated with this info
+       * instead. */
+
+      return TRUE;
     }
-  }
-  for (i = 0; i < MAX_NUM_USER_UNIT_FLAGS; i++) {
-    if (user_flag_names[i] != NULL
-        && fc_strcasecmp(user_flag_names[i], s) == 0) {
-      return i + F_USER_FLAG_1;
-    }
-  }
+  } unit_list_iterate_end;
 
-  return F_LAST;
-}
-
-/**************************************************************************
-  Return the (untranslated) rule name of the unit flag.
-**************************************************************************/
-const char *unit_flag_rule_name(enum unit_flag_id id)
-{
-  fc_assert_ret_val(ARRAY_SIZE(flag_names) == F_USER_FLAG_1, NULL);
-  fc_assert_ret_val(id >= 0 && id < F_LAST, NULL);
-
-  if (id < F_USER_FLAG_1) {
-    return flag_names[id];
-  }
-
-  return user_flag_names[id - F_USER_FLAG_1];
-}
-
-/**************************************************************************
-  Convert role names to enum; case insensitive;
-  returns L_LAST if can't match.
-**************************************************************************/
-enum unit_role_id unit_role_by_rule_name(const char *s)
-{
-  enum unit_role_id i;
-
-  fc_assert_ret_val(ARRAY_SIZE(role_names) == (L_LAST - L_FIRST), L_LAST);
-
-  for(i = L_FIRST; i < L_LAST; i++) {
-    if (fc_strcasecmp(role_names[i - L_FIRST], s) == 0) {
-      return i;
-    }
-  }
-  return L_LAST;
-}
-
-/**************************************************************************
-  Return the (untranslated) rule name of the unit role.
-**************************************************************************/
-const char *unit_role_rule_name(enum unit_role_id id)
-{
-  fc_assert_ret_val(ARRAY_SIZE(role_names) == L_LAST, NULL);
-  fc_assert_ret_val(id >= 0 && id < L_LAST, NULL);
-  return role_names[id];
+  /* The player doesn't already have one. */
+  return FALSE;
 }
 
 /**************************************************************************
@@ -657,15 +661,15 @@ bool can_player_build_unit_direct(const struct player *p,
     return FALSE;
   }
 
-  if (utype_has_flag(punittype, F_NUCLEAR)
-      && !get_player_bonus(p, EFT_ENABLE_NUKE) > 0) {
+  if (utype_has_flag(punittype, UTYF_NUCLEAR)
+      && get_player_bonus(p, EFT_ENABLE_NUKE) <= 0) {
     return FALSE;
   }
-  if (utype_has_flag(punittype, F_NOBUILD)) {
+  if (utype_has_flag(punittype, UTYF_NOBUILD)) {
     return FALSE;
   }
 
-  if (utype_has_flag(punittype, F_BARBARIAN_ONLY)
+  if (utype_has_flag(punittype, UTYF_BARBARIAN_ONLY)
       && !is_barbarian(p)) {
     /* Unit can be built by barbarians only and this player is
      * not barbarian */
@@ -703,17 +707,11 @@ bool can_player_build_unit_direct(const struct player *p,
         return FALSE;
       }
     }
-    
   }
-  if (utype_has_flag(punittype, F_UNIQUE)) {
-    /* FIXME: This could be slow if we have lots of units. We could
-     * consider keeping an array of unittypes updated with this info 
-     * instead. */
-    unit_list_iterate(p->units, punit) {
-      if (unit_type(punit) == punittype) { 
-        return FALSE;
-      }
-    } unit_list_iterate_end;
+
+  if (utype_player_already_has_this_unique(p, punittype)) {
+    /* A player can only have one unit of each unique unit type. */
+    return FALSE;
   }
 
   /* If the unit has a building requirement, we check to see if the player
@@ -767,7 +765,7 @@ bool can_player_build_unit_later(const struct player *p,
                                  const struct unit_type *punittype)
 {
   fc_assert_ret_val(NULL != punittype, FALSE);
-  if (utype_has_flag(punittype, F_NOBUILD)) {
+  if (utype_has_flag(punittype, UTYF_NOBUILD)) {
     return FALSE;
   }
   while ((punittype = punittype->obsoleted_by) != U_NOT_OBSOLETED) {
@@ -798,6 +796,7 @@ static void precalc_one(int i,
   int j;
 
   /* Count: */
+  fc_assert(n_with_role[i] == 0);
   unit_type_iterate(u) {
     if (func_has(u, i)) {
       n_with_role[i]++;
@@ -847,10 +846,10 @@ void role_unit_precalcs(void)
     role_unit_precalcs_free();
   }
 
-  for(i=0; i<F_LAST; i++) {
+  for (i = 0; i <= UTYF_LAST_USER_FLAG; i++) {
     precalc_one(i, utype_has_flag);
   }
-  for(i=L_FIRST; i<L_LAST; i++) {
+  for (i = L_FIRST; i < L_LAST; i++) {
     precalc_one(i, utype_has_role);
   }
   first_init = FALSE;
@@ -861,19 +860,56 @@ How many unit types have specified role/flag.
 **************************************************************************/
 int num_role_units(int role)
 {
-  fc_assert_ret_val((role >= 0 && role < F_LAST)
+  fc_assert_ret_val((role >= 0 && role <= UTYF_LAST_USER_FLAG)
                     || (role >= L_FIRST && role < L_LAST), -1);
   fc_assert_ret_val(!first_init, -1);
   return n_with_role[role];
 }
 
+/**************************************************************************
+  Iterate over all the role units and feed them to callback.
+  Once callback returns TRUE, no further units are feeded to it and
+  we return the unit that caused callback to return TRUE
+**************************************************************************/
+struct unit_type *role_units_iterate(int role, role_unit_callback cb, void *data)
+{
+  int i;
+
+  for (i = 0; i < n_with_role[role]; i++) {
+    if (cb(with_role[role][i], data)) {
+      return with_role[role][i];
+    }
+  }
+
+  return NULL;
+}
+
+/**************************************************************************
+  Iterate over all the role units and feed them to callback, starting
+  from the last one.
+  Once callback returns TRUE, no further units are feeded to it and
+  we return the unit that caused callback to return TRUE
+**************************************************************************/
+struct unit_type *role_units_iterate_backwards(int role, role_unit_callback cb, void *data)
+{
+  int i;
+
+  for (i = n_with_role[role] - 1; i >= 0; i--) {
+    if (cb(with_role[role][i], data)) {
+     return with_role[role][i];
+    }
+  }
+
+  return NULL;
+}
+
 /**************************************************************************
 Return index-th unit with specified role/flag.
 Index -1 means (n-1), ie last one.
 **************************************************************************/
 struct unit_type *get_role_unit(int role, int index)
 {
-  fc_assert_ret_val((role >= 0 && role < F_LAST)
+  fc_assert_ret_val((role >= 0 && role <= UTYF_LAST_USER_FLAG)
                     || (role >= L_FIRST && role < L_LAST), NULL);
   fc_assert_ret_val(!first_init, NULL);
   if (index==-1) {
@@ -892,7 +928,7 @@ struct unit_type *best_role_unit(const struct city *pcity, int role)
   struct unit_type *u;
   int j;
 
-  fc_assert_ret_val((role >= 0 && role < F_LAST)
+  fc_assert_ret_val((role >= 0 && role <= UTYF_LAST_USER_FLAG)
                     || (role >= L_FIRST && role < L_LAST), NULL);
   fc_assert_ret_val(!first_init, NULL);
 
@@ -916,7 +952,7 @@ struct unit_type *best_role_unit_for_player(const struct player *pplayer,
 {
   int j;
 
-  fc_assert_ret_val((role >= 0 && role < F_LAST)
+  fc_assert_ret_val((role >= 0 && role <= UTYF_LAST_USER_FLAG)
                     || (role >= L_FIRST && role < L_LAST), NULL);
   fc_assert_ret_val(!first_init, NULL);
 
@@ -940,7 +976,7 @@ struct unit_type *first_role_unit_for_player(const struct player *pplayer,
 {
   int j;
 
-  fc_assert_ret_val((role >= 0 && role < F_LAST)
+  fc_assert_ret_val((role >= 0 && role <= UTYF_LAST_USER_FLAG)
                     || (role >= L_FIRST && role < L_LAST), NULL);
   fc_assert_ret_val(!first_init, NULL);
 
@@ -966,7 +1002,9 @@ void unit_types_init(void)
    * num_unit_types isn't known yet. */
   for (i = 0; i < ARRAY_SIZE(unit_types); i++) {
     unit_types[i].item_number = i;
+    unit_types[i].helptext = NULL;
     unit_types[i].veteran = NULL;
+    unit_types[i].bonuses = combat_bonus_list_new();
   }
 }
 
@@ -981,6 +1019,10 @@ static void unit_type_free(struct unit_type *punittype)
   }
 
   veteran_system_destroy(punittype->veteran);
+  combat_bonus_list_iterate(punittype->bonuses, pbonus) {
+    FC_FREE(pbonus);
+  } combat_bonus_list_iterate_end;
+  combat_bonus_list_destroy(punittype->bonuses);
 }
 
 /***************************************************************
@@ -988,23 +1030,24 @@ static void unit_type_free(struct unit_type *punittype)
 ***************************************************************/
 void unit_types_free(void)
 {
-  unit_type_iterate(punittype) {
-    unit_type_free(punittype);
-  } unit_type_iterate_end;
+  int i;
+
+  /* Can't use unit_type_iterate or utype_by_number here because
+   * we want to free what unit_types_init() has allocated. */
+  for (i = 0; i < ARRAY_SIZE(unit_types); i++) {
+    unit_type_free(unit_types + i);
+  }
 }
 
 /***************************************************************
   Frees the memory associated with all unit type flags
 ***************************************************************/
-void unit_flags_free(void)
+void unit_type_flags_free(void)
 {
   int i;
 
   for (i = 0; i < MAX_NUM_USER_UNIT_FLAGS; i++) {
-    if (user_flag_names[i] != 0) {
-      free(user_flag_names[i]);
-      user_flag_names[i] = NULL;
-    }
+    user_flag_free(&user_type_flags[i]);
   }
 }
 
@@ -1102,7 +1145,9 @@ void unit_classes_init(void)
    * num_unit_classes isn't known yet. */
   for (i = 0; i < ARRAY_SIZE(unit_classes); i++) {
     unit_classes[i].item_number = i;
+    unit_classes[i].cache.refuel_bases = NULL;
     unit_classes[i].cache.native_tile_bases = NULL;
+    unit_classes[i].cache.native_tile_roads = NULL;
   }
 }
 
@@ -1114,12 +1159,18 @@ void unit_classes_free(void)
   int i;
 
   for (i = 0; i < ARRAY_SIZE(unit_classes); i++) {
-    if (unit_classes[i].cache.native_tile_bases != NULL) {
+    if (unit_classes[i].cache.refuel_bases != NULL) {
       base_type_list_destroy(unit_classes[i].cache.refuel_bases);
       unit_classes[i].cache.refuel_bases = NULL;
+    }
+    if (unit_classes[i].cache.native_tile_bases != NULL) {
       base_type_list_destroy(unit_classes[i].cache.native_tile_bases);
       unit_classes[i].cache.native_tile_bases = NULL;
     }
+    if (unit_classes[i].cache.native_tile_roads != NULL) {
+      road_type_list_destroy(unit_classes[i].cache.native_tile_roads);
+      unit_classes[i].cache.native_tile_roads = NULL;
+    }
   }
 }
 
@@ -1242,20 +1293,38 @@ void veteran_system_definition(struct veteran_system *vsystem, int level,
 
   vlevel = vsystem->definitions + level;
 
-  names_set(&vlevel->name, vlist_name, NULL);
+  names_set(&vlevel->name, NULL, vlist_name, NULL);
   vlevel->power_fact = vlist_power;
   vlevel->move_bonus = vlist_move;
   vlevel->raise_chance = vlist_raise;
   vlevel->work_raise_chance = vlist_wraise;
 }
 
+/**************************************************************************
+  Return pointer to ai data of given unit type and ai type.
+**************************************************************************/
+void *utype_ai_data(const struct unit_type *ptype, const struct ai_type *ai)
+{
+  return ptype->ais[ai_type_number(ai)];
+}
+
+/**************************************************************************
+  Attach ai data to unit type
+**************************************************************************/
+void utype_set_ai_data(struct unit_type *ptype, const struct ai_type *ai,
+                       void *data)
+{
+  ptype->ais[ai_type_number(ai)] = data;
+}
+
 /****************************************************************************
   Set caches for unit class.
 ****************************************************************************/
 void set_unit_class_caches(struct unit_class *pclass)
 {
-  pclass->cache.native_tile_bases = base_type_list_new();
   pclass->cache.refuel_bases = base_type_list_new();
+  pclass->cache.native_tile_bases = base_type_list_new();
+  pclass->cache.native_tile_roads = road_type_list_new();
 
   base_type_iterate(pbase) {
     if (is_native_base_to_uclass(pbase, pclass)) {
@@ -1265,4 +1334,11 @@ void set_unit_class_caches(struct unit_class *pclass)
       }
     }
   } base_type_iterate_end;
+
+  road_type_iterate(proad) {
+    if (is_native_road_to_uclass(proad, pclass)
+        && road_has_flag(proad, RF_NATIVE_TILE)) {
+      road_type_list_append(pclass->cache.native_tile_roads, proad);
+    }
+  } road_type_iterate_end;
 }
