@@ -29,6 +29,9 @@
 #include "fc_svnrev_gen.h"
 #endif /* SVNREV */
 
+#ifdef GITREV
+#include "fc_gitrev_gen.h"
+#endif /* GITREV */
 
 /**********************************************************************
   Return string containing both name of Freeciv and version.
@@ -43,6 +46,9 @@ const char *freeciv_name_version(void)
 #elif defined(SVNREV) && !defined(FC_SVNREV_OFF)
   fc_snprintf(msgbuf, sizeof (msgbuf), _("Freeciv version %s (%s)"),
               VERSION_STRING, fc_svn_revision());
+#elif defined(GITREV) && !defined(FC_GITREV_OFF)
+  fc_snprintf(msgbuf, sizeof (msgbuf), _("Freeciv version %s (%s)"),
+              VERSION_STRING, fc_git_revision());
 #else
   fc_snprintf(msgbuf, sizeof (msgbuf), _("Freeciv version %s"),
               VERSION_STRING);
@@ -82,8 +88,29 @@ const char *fc_svn_revision(void)
 #endif /* FC_SVNREV_OFF */
 }
 
+/**********************************************************************
+  Returns string with git revision information if it is possible to
+  determine. Can return also some fallback string or even NULL.
+***********************************************************************/
+const char *fc_git_revision(void)
+{
+#if defined(GITREV) && !defined(FC_GITREV_OFF)
+  static char buf[100];
+  bool translate = FC_GITREV1[0] != '\0';
+
+  fc_snprintf(buf, sizeof(buf), "%s%s",
+              translate ? _(FC_GITREV1) : FC_GITREV1, FC_GITREV2);
+
+  return buf; /* Either revision, or modified revision */
+#else  /* FC_GITREV_OFF */
+  return NULL;
+#endif /* FC_GITREV_OFF */
+}
+
 /**********************************************************************
   Returns version string that can be used to compare two freeciv builds.
+  This does not handle git revisions, as there's no way to compare
+  which of the two commits is "higher".
 ***********************************************************************/
 const char *fc_comparable_version(void)
 {
@@ -148,3 +175,28 @@ const char *freeciv_motto(void)
 {
   return _("'Cause civilization should be free!");
 }
+
+/***************************************************************************
+  Return version string in a format suitable to be written to created
+  datafiles as human readable information.
+***************************************************************************/
+const char *freeciv_datafile_version(void)
+{
+  static char buf[500] = { '\0' };
+
+  if (buf[0] == '\0') {
+    const char *ver_rev;
+
+    ver_rev = fc_svn_revision();
+    if (ver_rev == NULL) {
+      ver_rev = fc_git_revision();
+    }
+    if (ver_rev != NULL) {
+      fc_snprintf(buf, sizeof(buf), "%s (%s)", VERSION_STRING, ver_rev);
+    } else {
+      fc_snprintf(buf, sizeof(buf), "%s", VERSION_STRING);
+    }
+  }
+
+  return buf;
+}
