@@ -87,7 +87,7 @@ bool is_sea_barbarian(struct player *pplayer)
 
   Dead barbarians forget the map and lose the money.
 **************************************************************************/
-static struct player *create_barbarian_player(enum barbarian_type type)
+struct player *create_barbarian_player(enum barbarian_type type)
 {
   struct player *barbarians;
   struct nation_type *nation;
@@ -124,8 +124,17 @@ static struct player *create_barbarian_player(enum barbarian_type type)
   }
   server_player_init(barbarians, TRUE, TRUE);
 
-  nation = pick_a_nation(NULL, FALSE, TRUE, type);
+  nation = pick_a_nation(NULL, FALSE, FALSE, type);
+
+  /* Ruleset loading time checks should guarantee that there always is
+     suitable nation available */
+  fc_assert(nation != NULL);
+
   player_nation_defaults(barbarians, nation, TRUE);
+  if (game_was_started()) {
+    /* Find a color for the new player. */
+    assign_player_colors();
+  }
 
   server.nbarbarians++;
 
@@ -169,24 +178,6 @@ static struct player *create_barbarian_player(enum barbarian_type type)
   return barbarians;
 }
 
-/**************************************************************************
-  Check if a tile is land and free of enemy units
-**************************************************************************/
-static bool is_free_land(struct tile *ptile, struct player *who)
-{
-  return (!is_ocean_tile(ptile)
-	  && !is_non_allied_unit_tile((ptile), who));
-}
-
-/**************************************************************************
-  Check if a tile is sea and free of enemy units
-**************************************************************************/
-static bool is_free_sea(struct tile *ptile, struct player *who)
-{
-  return (is_ocean_tile(ptile)
-	  && !is_non_allied_unit_tile((ptile), who));
-}
-
 /**************************************************************************
   (Re)initialize direction checked status array based on terrain class.
 **************************************************************************/
@@ -294,12 +285,14 @@ bool unleash_barbarians(struct tile *ptile)
     dir_tiles[dir] = mapstep(ptile, dir);
     if (dir_tiles[dir] == NULL) {
       terrainc[dir] = terrain_class_invalid();
-    } else if (is_free_land(dir_tiles[dir], barbarians)) {
-      terrainc[dir] = TC_LAND;
-      land_tiles++;
-    } else if (is_free_sea(dir_tiles[dir], barbarians)) {
-      terrainc[dir] = TC_OCEAN;
-      ocean_tiles++;
+    } else if (!is_non_allied_unit_tile(dir_tiles[dir], barbarians)) {
+      if (is_ocean_tile(dir_tiles[dir])) {
+        terrainc[dir] = TC_OCEAN;
+        ocean_tiles++;
+      } else {
+        terrainc[dir] = TC_LAND;
+        land_tiles++;
+      }
     } else {
       terrainc[dir] = terrain_class_invalid();
     }
@@ -586,7 +579,7 @@ static void try_summon_barbarians(void)
     boat = find_a_unit_type(L_BARBARIAN_BOAT,-1);
 
     if (is_native_tile(boat, utile)
-        && (!utype_has_flag(boat, F_TRIREME) || is_safe_ocean(utile))) {
+        && (!utype_has_flag(boat, UTYF_TRIREME) || is_safe_ocean(utile))) {
       int cap;
 
       ptrans = create_unit(barbarians, utile, boat, 0, 0, -1);
