@@ -91,6 +91,7 @@ static struct command commands[] = {
              "list map image definitions\n"
              "list players\n"
              "list scenarios\n"
+             "list nationsets\n"
              "list teams\n"
              "list votes\n"),
    N_("Show a list of various things."),
@@ -102,6 +103,7 @@ static struct command commands[] = {
       " - the list of defined map images,\n"
       " - the list of the players in the game,\n"
       " - the available scenarios,\n"
+      " - the available nationsets in this ruleset,\n"
       " - the teams of players or\n"
       " - the running votes.\n"
       "The argument may be abbreviated, and defaults to 'players' if "
@@ -545,6 +547,21 @@ static struct command commands[] = {
       "and use the 'start' command once players have reconnected."), NULL,
    CMD_ECHO_ADMINS, VCF_NONE, 0
   },
+#ifdef DEBUG
+  {"scensave",	ALLOW_ADMIN,
+   /* TRANS: translate text between <> only */
+   N_("scensave\n"
+      "scensave <file-name>"),
+   N_("Save game to file as scenario."),
+   N_("Save the current game to file <file-name> as scenario. If no file-name "
+      "argument is given saves to \"<auto-save name prefix><year>m.sav[.gz]\". "
+      "To reload a savegame created by 'scensave', start the server with "
+      "the command-line argument:\n"
+      "    '--file <filename>' or '-f <filename>'\n"
+      "and use the 'start' command once players have reconnected."), NULL,
+   CMD_ECHO_ADMINS, VCF_NONE, 0
+  },
+#endif /* DEBUG */
   {"load",      ALLOW_CTRL,
    /* TRANS: translate text between <> only */
    N_("load\n"
@@ -596,15 +613,6 @@ static struct command commands[] = {
        "able to reconnect is controlled by the 'kicktime' setting."), NULL,
    CMD_ECHO_ADMINS, VCF_NOPASSALONE, 50
   },
-#ifdef DEBUG
-  {"oldsave", ALLOW_ADMIN,
-   /* TRANS: translate text between <> only */
-   N_("save\n"
-      "save <file-name>"),
-   N_("Save game to file using the old format."), NULL, NULL,
-   CMD_ECHO_ADMINS, VCF_NONE, 0
-  },
-#endif /* DEBUG */
   {"delegate", ALLOW_BASIC,
    /* TRANS: translate only text between [] and <> */
    N_("delegate to <username> [player-name]\n"
