@@ -31,6 +31,7 @@
 #include "map.h"
 #include "packets.h"
 #include "player.h"
+#include "research.h"
 #include "unit.h"
 
 /* common/scriptcore */
@@ -175,7 +176,7 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
     clause_list_iterate(ptreaty->clauses, pclause) {
       struct city *pcity = NULL;
 
-      if (pclause->from == pplayer) {
+      if (pclause->from == pplayer || is_pact_clause(pclause->type)) {
 	switch(pclause->type) {
 	case CLAUSE_EMBASSY:
           if (player_has_real_embassy(pother, pplayer)) {
@@ -397,6 +398,7 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
       struct player_diplstate *ds_destgiver
         = player_diplstate_get(pdest, pgiver);
       enum diplstate_type old_diplstate = ds_giverdest->type;
+      struct unit_list *pgiver_seen_units, *pdest_seen_units;
 
       switch (pclause->type) {
       case CLAUSE_EMBASSY:
@@ -430,12 +432,8 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
                            nation_plural_for_player(pdest),
                            advance_name_for_player(pdest, pclause->value),
                            nation_plural_for_player(pgiver));
-
-          script_server_signal_emit("tech_researched", 3,
-                                    API_TYPE_TECH_TYPE,
-                                    advance_by_number(pclause->value),
-                                    API_TYPE_PLAYER, pdest,
-                                    API_TYPE_STRING, "traded");
+          script_tech_learned(pdest, advance_by_number(pclause->value),
+                              "traded");
           do_dipl_cost(pdest, pclause->value);
           found_new_tech(pdest, pclause->value, FALSE, TRUE);
         }
@@ -487,11 +485,15 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
                         _("You give the city of %s to %s."),
                         city_link(pcity), player_name(pdest));
 
-          transfer_city(pdest, pcity, -1, TRUE, TRUE, FALSE,
-                        !is_barbarian(pdest));
+          (void) transfer_city(pdest, pcity, -1, TRUE, TRUE, FALSE,
+                               !is_barbarian(pdest));
 	  break;
 	}
       case CLAUSE_CEASEFIRE:
+        if (old_diplstate == DS_ALLIANCE) {
+          pgiver_seen_units = get_seen_units(pgiver, pdest);
+          pdest_seen_units = get_seen_units(pdest, pgiver);
+        }
         ds_giverdest->type = DS_CEASEFIRE;
         ds_giverdest->turns_left = TURNS_LEFT;
         ds_destgiver->type = DS_CEASEFIRE;
@@ -502,13 +504,21 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
         notify_player(pdest, NULL, E_TREATY_CEASEFIRE, ftc_server,
                       _("You agree on a cease-fire with %s."),
                       player_name(pgiver));
-	if (old_diplstate == DS_ALLIANCE) {
-	  update_players_after_alliance_breakup(pgiver, pdest);
-	}
+        if (old_diplstate == DS_ALLIANCE) {
+          update_players_after_alliance_breakup(pgiver, pdest,
+                                                pgiver_seen_units,
+                                                pdest_seen_units);
+          unit_list_destroy(pgiver_seen_units);
+          unit_list_destroy(pdest_seen_units);
+        }
 
         worker_refresh_required = TRUE;
 	break;
       case CLAUSE_PEACE:
+        if (old_diplstate == DS_ALLIANCE) {
+          pgiver_seen_units = get_seen_units(pgiver, pdest);
+          pdest_seen_units = get_seen_units(pdest, pgiver);
+        }
         ds_giverdest->type = DS_ARMISTICE;
         ds_destgiver->type = DS_ARMISTICE;
         ds_giverdest->turns_left = TURNS_LEFT;
@@ -539,9 +549,13 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
                       nation_plural_for_player(pgiver),
                       TURNS_LEFT,
                       nation_adjective_for_player(pgiver));
-	if (old_diplstate == DS_ALLIANCE) {
-	  update_players_after_alliance_breakup(pgiver, pdest);
-	}
+        if (old_diplstate == DS_ALLIANCE) {
+          update_players_after_alliance_breakup(pgiver, pdest,
+                                                pgiver_seen_units,
+                                                pdest_seen_units);
+          unit_list_destroy(pgiver_seen_units);
+          unit_list_destroy(pdest_seen_units);
+        }
 
         worker_refresh_required = TRUE;
 	break;
@@ -756,8 +770,8 @@ void handle_diplomacy_init_meeting_req(struct player *pplayer,
     return;
   }
 
-  if (get_player_bonus(pplayer, EFT_NO_DIPLOMACY)
-      || get_player_bonus(pother, EFT_NO_DIPLOMACY)) {
+  if (get_player_bonus(pplayer, EFT_NO_DIPLOMACY) > 0
+      || get_player_bonus(pother, EFT_NO_DIPLOMACY) > 0) {
     notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
                   _("Your diplomatic envoy was decapitated!"));
     return;
