@@ -94,7 +94,7 @@ void spy_poison(struct player *pplayer, struct unit *pdiplomat,
   log_debug("poison: unit: %d", pdiplomat->id);
 
   /* If not a Spy, can't poison. */
-  if (!unit_has_type_flag(pdiplomat, F_SPY))
+  if (!unit_has_type_flag(pdiplomat, UTYF_SPY))
     return;
 
   /* Check if the Diplomat/Spy succeeds against defending Diplomats/Spies. */
@@ -211,10 +211,10 @@ void diplomat_investigate(struct player *pplayer, struct unit *pdiplomat,
                        city_tile(pcity), city_link(pcity));
 
   /* Spies always survive. Diplomats never do. */
-  if (!unit_has_type_flag(pdiplomat, F_SPY)) {
+  if (!unit_has_type_flag(pdiplomat, UTYF_SPY)) {
     wipe_unit(pdiplomat, ULR_USED, NULL);
   } else {
-    send_unit_info (pplayer, pdiplomat);
+    send_unit_info(NULL, pdiplomat);
   }
 }
 
@@ -312,10 +312,10 @@ void diplomat_embassy(struct player *pplayer, struct unit *pdiplomat,
                        city_tile(pcity), city_link(pcity));
 
   /* Spies always survive. Diplomats never do. */
-  if (!unit_has_type_flag(pdiplomat, F_SPY)) {
+  if (!unit_has_type_flag(pdiplomat, UTYF_SPY)) {
     wipe_unit(pdiplomat, ULR_USED, NULL);
   } else {
-    send_unit_info (pplayer, pdiplomat);
+    send_unit_info(NULL, pdiplomat);
   }
 }
 
@@ -349,7 +349,7 @@ void spy_sabotage_unit(struct player *pplayer, struct unit *pdiplomat,
   log_debug("sabotage-unit: unit: %d", pdiplomat->id);
 
   /* If not a Spy, can't sabotage unit. */
-  if (!unit_has_type_flag(pdiplomat, F_SPY))
+  if (!unit_has_type_flag(pdiplomat, UTYF_SPY))
     return;
 
   /* N.B: unit_link() always returns the same pointer. */
@@ -435,6 +435,15 @@ void diplomat_bribe(struct player *pplayer, struct unit *pdiplomat,
     return;
   }
 
+  /* Sanity check: The victim isn't a unique unit the actor player already
+   * has. */
+  if (utype_player_already_has_this_unique(pplayer,
+                                           unit_type(pvictim))) {
+    log_debug("bribe-unit: already got unique unit");
+
+    return;
+  }
+
   log_debug("bribe-unit: unit: %d", pdiplomat->id);
 
   /* Check for unit from a bribable government. */
@@ -459,7 +468,7 @@ void diplomat_bribe(struct player *pplayer, struct unit *pdiplomat,
     return;
   }
 
-  if (unit_has_type_flag(pvictim, F_UNBRIBABLE)) {
+  if (unit_has_type_flag(pvictim, UTYF_UNBRIBABLE)) {
     notify_player(pplayer, unit_tile(pdiplomat),
                   E_MY_DIPLOMAT_FAILED, ftc_server,
                   _("You cannot bribe the %s!"),
@@ -508,7 +517,7 @@ void diplomat_bribe(struct player *pplayer, struct unit *pdiplomat,
     pdiplomat->moves_left = 0;
   }
   if (NULL != player_unit_by_number(pplayer, diplomat_id)) {
-    send_unit_info(pplayer, pdiplomat);
+    send_unit_info(NULL, pdiplomat);
   }
 
   /* Update clients. */
@@ -580,7 +589,7 @@ void diplomat_get_tech(struct player *pplayer, struct unit *pdiplomat,
   log_debug("steal-tech: unit: %d", pdiplomat->id);
 
   /* If not a Spy, do something random. */
-  if (!unit_has_type_flag(pdiplomat, F_SPY)) {
+  if (!unit_has_type_flag(pdiplomat, UTYF_SPY)) {
     technology = A_UNSET;
   }
 
@@ -595,7 +604,7 @@ void diplomat_get_tech(struct player *pplayer, struct unit *pdiplomat,
   /* Check if the Diplomat/Spy succeeds with his/her task. */
   /* (Twice as difficult if target is specified.) */
   /* (If already stolen from, impossible for Diplomats and harder for Spies.) */
-  if (pcity->server.steal > 0 && !unit_has_type_flag(pdiplomat, F_SPY)) {
+  if (pcity->server.steal > 0 && !unit_has_type_flag(pdiplomat, UTYF_SPY)) {
     /* Already stolen from: Diplomat always fails! */
     count = 1;
     log_debug("steal-tech: difficulty: impossible");
@@ -617,7 +626,7 @@ void diplomat_get_tech(struct player *pplayer, struct unit *pdiplomat,
   }
   
   if (count > 0) {
-    if (pcity->server.steal > 0 && !unit_has_type_flag(pdiplomat, F_SPY)) {
+    if (pcity->server.steal > 0 && !unit_has_type_flag(pdiplomat, UTYF_SPY)) {
       notify_player(pplayer, city_tile(pcity),
                     E_MY_DIPLOMAT_FAILED, ftc_server,
                     _("%s was expecting your attempt to steal technology "
@@ -653,7 +662,7 @@ void diplomat_get_tech(struct player *pplayer, struct unit *pdiplomat,
                   _("No new technology found in %s."),
                   city_link(pcity));
     diplomat_charge_movement (pdiplomat, pcity->tile);
-    send_unit_info (pplayer, pdiplomat);
+    send_unit_info(NULL, pdiplomat);
     return;
   }
 
@@ -693,6 +702,8 @@ void diplomat_incite(struct player *pplayer, struct unit *pdiplomat,
 		     struct city *pcity)
 {
   struct player *cplayer;
+  struct tile *ctile;
+  const char *clink;
   int revolt_cost;
 
   /* Fetch target civilization's player.  Sanity checks. */
@@ -702,6 +713,9 @@ void diplomat_incite(struct player *pplayer, struct unit *pdiplomat,
   if (!cplayer || pplayers_allied(cplayer, pplayer))
     return;
 
+  ctile = city_tile(pcity);
+  clink = city_link(pcity);
+
   log_debug("incite: unit: %d", pdiplomat->id);
 
   /* See if the city is subvertable. */
@@ -718,10 +732,9 @@ void diplomat_incite(struct player *pplayer, struct unit *pdiplomat,
 
   /* If player doesn't have enough gold, can't incite a revolt. */
   if (pplayer->economic.gold < revolt_cost) {
-    notify_player(pplayer, city_tile(pcity),
-                  E_MY_DIPLOMAT_FAILED, ftc_server,
+    notify_player(pplayer, ctile, E_MY_DIPLOMAT_FAILED, ftc_server,
                   _("You don't have enough gold to subvert %s."),
-                  city_link(pcity));
+                  clink);
     log_debug("incite: not enough gold");
     return;
   }
@@ -736,18 +749,16 @@ void diplomat_incite(struct player *pplayer, struct unit *pdiplomat,
 
   /* Check if the Diplomat/Spy succeeds with his/her task. */
   if (fc_rand (100) >= game.server.diplchance) {
-    notify_player(pplayer, city_tile(pcity),
-                  E_MY_DIPLOMAT_FAILED, ftc_server,
+    notify_player(pplayer, ctile, E_MY_DIPLOMAT_FAILED, ftc_server,
                   _("Your %s was caught in the attempt"
                     " of inciting a revolt!"),
                   unit_tile_link(pdiplomat));
-    notify_player(cplayer, city_tile(pcity),
-                  E_ENEMY_DIPLOMAT_FAILED, ftc_server,
+    notify_player(cplayer, ctile, E_ENEMY_DIPLOMAT_FAILED, ftc_server,
                   _("You caught %s %s attempting"
                     " to incite a revolt in %s!"),
                   nation_adjective_for_player(pplayer),
                   unit_tile_link(pdiplomat),
-                  city_link(pcity));
+                  clink);
     wipe_unit(pdiplomat, ULR_CAUGHT, cplayer);
     return;
   }
@@ -765,14 +776,12 @@ void diplomat_incite(struct player *pplayer, struct unit *pdiplomat,
   pplayer->economic.gold -= revolt_cost;
 
   /* Notify everybody involved. */
-  notify_player(pplayer, city_tile(pcity),
-                E_MY_DIPLOMAT_INCITE, ftc_server,
+  notify_player(pplayer, ctile, E_MY_DIPLOMAT_INCITE, ftc_server,
                 _("Revolt incited in %s, you now rule the city!"),
-                city_link(pcity));
-  notify_player(cplayer, city_tile(pcity),
-                E_ENEMY_DIPLOMAT_INCITE, ftc_server,
+                clink);
+  notify_player(cplayer, ctile, E_ENEMY_DIPLOMAT_INCITE, ftc_server,
                 _("%s has revolted, %s influence suspected."),
-                city_link(pcity),
+                clink,
                 nation_adjective_for_player(pplayer));
 
   pcity->shield_stock = 0;
@@ -782,12 +791,14 @@ void diplomat_incite(struct player *pplayer, struct unit *pdiplomat,
   steal_a_tech (pplayer, cplayer, A_UNSET);
 
   /* this may cause a diplomatic incident */
-  maybe_cause_incident(DIPLOMAT_INCITE, pplayer, cplayer,
-                       city_tile(pcity), city_link(pcity));
+  maybe_cause_incident(DIPLOMAT_INCITE, pplayer, cplayer, ctile, clink);
 
   /* Transfer city and units supported by this city (that
      are within one square of the city) to the new owner. */
-  transfer_city(pplayer, pcity, 1, TRUE, TRUE, FALSE, !is_barbarian(pplayer));
+  if (!transfer_city(pplayer, pcity, 1, TRUE, TRUE, FALSE,
+                     !is_barbarian(pplayer))) {
+    pcity = NULL; /* City destroyed, probably due to a script. */
+  }
 
   /* Check if a spy survives her mission. Diplomats never do.
    * _After_ transferring the city, or the city area is first fogged
@@ -835,7 +846,7 @@ void diplomat_sabotage(struct player *pplayer, struct unit *pdiplomat,
   log_debug("sabotage: unit: %d", pdiplomat->id);
 
   /* If not a Spy, do something random. */
-  if (!unit_has_type_flag(pdiplomat, F_SPY))
+  if (!unit_has_type_flag(pdiplomat, UTYF_SPY))
     improvement = B_LAST;
 
   /* Twice as difficult if target is specified. */
@@ -898,7 +909,7 @@ void diplomat_sabotage(struct player *pplayer, struct unit *pdiplomat,
                     unit_link(pdiplomat),
                     city_link(pcity));
       diplomat_charge_movement(pdiplomat, pcity->tile);
-      send_unit_info(pplayer, pdiplomat);
+      send_unit_info(NULL, pdiplomat);
       log_debug("sabotage: random: nothing to do");
       return;
     }
@@ -950,7 +961,7 @@ void diplomat_sabotage(struct player *pplayer, struct unit *pdiplomat,
                       _("You cannot sabotage a %s!"),
                       improvement_name_translation(pimprove));
         diplomat_charge_movement(pdiplomat, pcity->tile);
-        send_unit_info(pplayer, pdiplomat);
+        send_unit_info(NULL, pdiplomat);
         log_debug("sabotage: disallowed target improvement: %d (%s)",
                   improvement, improvement_rule_name(pimprove));
         return;
@@ -963,7 +974,7 @@ void diplomat_sabotage(struct player *pplayer, struct unit *pdiplomat,
                     improvement_name_translation(pimprove),
                     city_link(pcity));
       diplomat_charge_movement(pdiplomat, pcity->tile);
-      send_unit_info(pplayer, pdiplomat);
+      send_unit_info(NULL, pdiplomat);
       log_debug("sabotage: target improvement not found: %d (%s)",
                 improvement, improvement_rule_name(pimprove));
       return;
@@ -1085,18 +1096,18 @@ static bool diplomat_success_vs_defender(struct unit *pattacker,
 {
   int chance = 50; /* Base 50% chance */
 
-  if (unit_has_type_flag(pattacker, F_SUPERSPY)) {
+  if (unit_has_type_flag(pattacker, UTYF_SUPERSPY)) {
     return TRUE;
   }
-  if (unit_has_type_flag(pdefender, F_SUPERSPY)) {
+  if (unit_has_type_flag(pdefender, UTYF_SUPERSPY)) {
     return FALSE;
   }
 
   /* Add or remove 25% if spy flag. */
-  if (unit_has_type_flag(pattacker, F_SPY)) {
+  if (unit_has_type_flag(pattacker, UTYF_SPY)) {
     chance += 25;
   }
-  if (unit_has_type_flag(pdefender, F_SPY)) {
+  if (unit_has_type_flag(pdefender, UTYF_SPY)) {
     chance -= 25;
   }
 
@@ -1164,16 +1175,16 @@ static bool diplomat_infiltrate_tile(struct player *pplayer,
       continue;
     }
 
-    if (unit_has_type_flag(punit, F_DIPLOMAT)
-        || unit_has_type_flag(punit, F_SUPERSPY)) {
-      /* A F_SUPERSPY unit may not actually be a spy, but a superboss
+    if (unit_has_type_flag(punit, UTYF_DIPLOMAT)
+        || unit_has_type_flag(punit, UTYF_SUPERSPY)) {
+      /* A UTYF_SUPERSPY unit may not actually be a spy, but a superboss
        * which we cannot allow puny diplomats from getting the better
        * of. Note that diplomat_success_vs_defender() is always TRUE
-       * if the attacker is F_SUPERSPY. Hence F_SUPERSPY vs F_SUPERSPY
+       * if the attacker is UTYF_SUPERSPY. Hence UTYF_SUPERSPY vs UTYF_SUPERSPY
        * in a diplomatic contest always kills the attacker. */
 
       if (diplomat_success_vs_defender(pdiplomat, punit, ptile) 
-          && !unit_has_type_flag(punit, F_SUPERSPY)) {
+          && !unit_has_type_flag(punit, UTYF_SUPERSPY)) {
         /* Defending Spy/Diplomat dies. */
 
         /* N.B.: *_link() always returns the same pointer. */
@@ -1234,7 +1245,7 @@ static bool diplomat_infiltrate_tile(struct player *pplayer,
         if (maybe_make_veteran(pdiplomat)) {
           notify_unit_experience(pdiplomat);
         }
-        send_unit_info(pplayer, pdiplomat);
+        send_unit_info(NULL, pdiplomat);
         wipe_unit(punit, ULR_ELIMINATED, pplayer);
         return FALSE;
       } else {
@@ -1339,8 +1350,8 @@ static void diplomat_escape(struct player *pplayer, struct unit *pdiplomat,
                               FALSE, FALSE, TRUE, FALSE, NULL);
 
   if (spyhome
-      && unit_has_type_flag(pdiplomat, F_SPY)
-      && (unit_has_type_flag(pdiplomat, F_SUPERSPY)
+      && unit_has_type_flag(pdiplomat, UTYF_SPY)
+      && (unit_has_type_flag(pdiplomat, UTYF_SUPERSPY)
           || fc_rand (100) < escapechance)) {
     /* Attacking Spy/Diplomat survives. */
     notify_player(pplayer, ptile, E_MY_DIPLOMAT_ESCAPE, ftc_server,
@@ -1354,7 +1365,7 @@ static void diplomat_escape(struct player *pplayer, struct unit *pdiplomat,
 
     /* being teleported costs all movement */
     if (!teleport_unit_to_city (pdiplomat, spyhome, -1, FALSE)) {
-      send_unit_info (pplayer, pdiplomat);
+      send_unit_info(NULL, pdiplomat);
       log_error("Bug in diplomat_escape: Spy can't teleport.");
       return;
     }
@@ -1451,9 +1462,11 @@ int count_diplomats_on_tile(struct tile *ptile)
 {
   int count = 0;
 
-  unit_list_iterate((ptile)->units, punit)
-    if (unit_has_type_flag(punit, F_DIPLOMAT))
+  unit_list_iterate((ptile)->units, punit) {
+    if (unit_has_type_flag(punit, UTYF_DIPLOMAT)) {
       count++;
-  unit_list_iterate_end;
+    }
+  } unit_list_iterate_end;
+
   return count;
 }
