@@ -37,6 +37,7 @@
 #include "packets.h"
 
 /* server */
+#include "citytools.h"
 #include "connecthand.h"
 #include "ggzserver.h"
 #include "maphand.h"
@@ -61,6 +62,23 @@
                           list, plink, psp)
 #define startpos_list_iterate_end LIST_BOTH_ITERATE_END
 
+struct team_placement_config {
+  struct tile **startpos;
+  int flexible_startpos_num;
+  int usable_startpos_num;
+  int total_startpos_num;
+};
+
+struct team_placement_state {
+  int *startpos;
+  long score;
+};
+
+#define SPECPQ_TAG team_placement
+#define SPECPQ_DATA_TYPE struct team_placement_state *
+#define SPECPQ_PRIORITY_TYPE long
+#include "specpq.h"
+
 /****************************************************************************
   Get unit_type for given role character
 ****************************************************************************/
@@ -202,6 +220,200 @@ static struct tile *find_dispersed_position(struct player *pplayer,
   return ptile;
 }
 
+/* Calculate the distance between tiles, according to the 'teamplacement'
+ * setting set to 'CLOSEST'. */
+#define team_placement_closest sq_map_distance
+
+/****************************************************************************
+  Calculate the distance between tiles, according to the 'teamplacement'
+  setting set to 'CONTINENT'.
+****************************************************************************/
+static int team_placement_continent(const struct tile *ptile1,
+                                    const struct tile *ptile2)
+{
+  return (ptile1->continent == ptile2->continent
+          ? sq_map_distance(ptile1, ptile2)
+          : sq_map_distance(ptile1, ptile2) + MAP_INDEX_SIZE);
+}
+
+/****************************************************************************
+  Calculate the distance between tiles, according to the 'teamplacement'
+  setting set to 'HORIZONTAL'.
+****************************************************************************/
+static int team_placement_horizontal(const struct tile *ptile1,
+                                     const struct tile *ptile2)
+{
+  int dx, dy;
+
+  map_distance_vector(&dx, &dy, ptile1, ptile2);
+  /* Map vector to natural vector (Y axis). */
+  return abs(MAP_IS_ISOMETRIC ? dx + dy : dy);
+}
+
+/****************************************************************************
+  Calculate the distance between tiles, according to the 'teamplacement'
+  setting set to 'VERTICAL'.
+****************************************************************************/
+static int team_placement_vertical(const struct tile *ptile1,
+                                   const struct tile *ptile2)
+{
+  int dx, dy;
+
+  map_distance_vector(&dx, &dy, ptile1, ptile2);
+  /* Map vector to natural vector (X axis). */
+  return abs(MAP_IS_ISOMETRIC ? dx - dy : dy);
+}
+
+/****************************************************************************
+  Destroys a team_placement_state structure.
+****************************************************************************/
+static void team_placement_state_destroy(struct team_placement_state *pstate)
+{
+  free(pstate->startpos);
+  free(pstate);
+}
+
+/****************************************************************************
+  Find the best team placement, according to the 'team_placement' setting.
+****************************************************************************/
+static void do_team_placement(const struct team_placement_config *pconfig,
+                              struct team_placement_state *pbest_state,
+                              int iter_max)
+{
+  const size_t state_array_size = (sizeof(*pbest_state->startpos)
+                                   * pconfig->total_startpos_num);
+  struct team_placement_pq *pqueue =
+      team_placement_pq_new(pconfig->total_startpos_num * 4);
+  int (*distance)(const struct tile *, const struct tile *) = NULL;
+  struct team_placement_state *pstate, *pnew;
+  const struct tile *ptile1, *ptile2;
+  long base_delta, delta;
+  bool base_delta_calculated;
+  int iter = 0;
+  bool repeat;
+  int i, j, k, t1, t2;
+
+  switch (map.server.team_placement) {
+  case TEAM_PLACEMENT_CLOSEST:
+    distance = team_placement_closest;
+    break;
+  case TEAM_PLACEMENT_CONTINENT:
+    distance = team_placement_continent;
+    break;
+  case TEAM_PLACEMENT_HORIZONTAL:
+    distance = team_placement_horizontal;
+    break;
+  case TEAM_PLACEMENT_VERTICAL:
+    distance = team_placement_vertical;
+    break;
+  case TEAM_PLACEMENT_DISABLED:
+    break;
+  }
+  fc_assert_ret_msg(distance != NULL, "Wrong team_placement variant (%d)",
+                    map.server.team_placement);
+
+  /* Initialize starting state. */
+  pstate = fc_malloc(sizeof(*pstate));
+  pstate->startpos = fc_malloc(state_array_size);
+  memcpy(pstate->startpos, pbest_state->startpos, state_array_size);
+  pstate->score = pbest_state->score;
+
+  do {
+    repeat = FALSE;
+    for (i = 0; i < pconfig->usable_startpos_num; i++) {
+      t1 = pstate->startpos[i];
+      if (t1 == -1) {
+        continue; /* Not used. */
+      }
+      ptile1 = pconfig->startpos[i];
+      base_delta_calculated = FALSE;
+      for (j = i + 1; j < (i >= pconfig->flexible_startpos_num
+                           ? pconfig->usable_startpos_num
+                           : pconfig->flexible_startpos_num); j++) {
+        t2 = pstate->startpos[j];
+        if (t2 == -1) {
+          /* Not assigned yet. */
+          ptile2 = pconfig->startpos[j];
+          if (base_delta_calculated) {
+            delta = base_delta;
+            for (k = 0; k < pconfig->total_startpos_num; k++) {
+              if (k != i && t1 == pstate->startpos[k]) {
+                delta += distance(ptile2, pconfig->startpos[k]);
+              }
+            }
+          } else {
+            delta = 0;
+            base_delta = 0;
+            for (k = 0; k < pconfig->total_startpos_num; k++) {
+              if (k != i && t1 == pstate->startpos[k]) {
+                base_delta -= distance(ptile1, pconfig->startpos[k]);
+                delta += distance(ptile2, pconfig->startpos[k]);
+              }
+            }
+            delta += base_delta;
+            base_delta_calculated = TRUE;
+          }
+        } else if (t1 < t2) {
+          ptile2 = pconfig->startpos[j];
+          if (base_delta_calculated) {
+            delta = base_delta;
+            for (k = 0; k < pconfig->total_startpos_num; k++) {
+              if (k != i && t1 == pstate->startpos[k]) {
+                delta += distance(ptile2, pconfig->startpos[k]);
+              } else if (k != j && t2 == pstate->startpos[k]) {
+                delta -= distance(ptile2, pconfig->startpos[k]);
+                delta += distance(ptile1, pconfig->startpos[k]);
+              }
+            }
+          } else {
+            delta = 0;
+            base_delta = 0;
+            for (k = 0; k < pconfig->total_startpos_num; k++) {
+              if (k != i && t1 == pstate->startpos[k]) {
+                base_delta -= distance(ptile1, pconfig->startpos[k]);
+                delta += distance(ptile2, pconfig->startpos[k]);
+              } else if (k != j && t2 == pstate->startpos[k]) {
+                delta -= distance(ptile2, pconfig->startpos[k]);
+                delta += distance(ptile1, pconfig->startpos[k]);
+              }
+            }
+            delta += base_delta;
+            base_delta_calculated = TRUE;
+          }
+        } else {
+          continue;
+        }
+
+        if (delta <= 0) {
+          repeat = TRUE;
+          pnew = fc_malloc(sizeof(*pnew));
+          pnew->startpos = fc_malloc(state_array_size);
+          memcpy(pnew->startpos, pstate->startpos, state_array_size);
+          pnew->startpos[i] = t2;
+          pnew->startpos[j] = t1;
+          pnew->score = pstate->score + delta;
+          team_placement_pq_insert(pqueue, pnew, -pnew->score);
+
+          if (pnew->score < pbest_state->score) {
+            memcpy(pbest_state->startpos, pnew->startpos, state_array_size);
+            pbest_state->score = pnew->score;
+          }
+        }
+      }
+    }
+
+    team_placement_state_destroy(pstate);
+    if (iter++ >= iter_max) {
+      log_normal(_("Didn't find optimal solution for team placement "
+                   "in %d iterations."), iter);
+      break;
+    }
+
+  } while (repeat && team_placement_pq_remove(pqueue, &pstate));
+
+  team_placement_pq_destroy_full(pqueue, team_placement_state_destroy);
+}
+
 /****************************************************************************
   Initialize a new game: place the players' units onto the map, etc.
 ****************************************************************************/
@@ -215,6 +427,12 @@ void init_new_game(void)
   randomize_base64url_string(server.game_identifier,
                              sizeof(server.game_identifier));
 
+  /* Assign players to starting positions on the map.
+   * (In scenarios with restrictions on which nations can use which predefined
+   * start positions, this process tries to satisfy those restrictions, but
+   * does not guarantee to. Even if there is a solution to the matching
+   * problem, this algorithm may not find it.) */
+
   fc_assert(player_count() <= map_startpos_count());
 
   /* Convert the startposition hash table in a linked lists, as we mostly
@@ -238,8 +456,8 @@ void init_new_game(void)
   memset(player_startpos, 0, sizeof(player_startpos));
   log_verbose("Placing players at start positions.");
 
-  /* First assign start positions which requires certain nations only this
-   * one. */
+  /* First assign start positions which have restrictions on which nations
+   * can use them. */
   if (0 < startpos_list_size(targeted_list)) {
     log_verbose("Assigning matching nations.");
 
@@ -270,6 +488,8 @@ void init_new_game(void)
         } startpos_list_iterate_end;
 
         if (NULL != choice) {
+          /* Assign this start position to this player and remove
+           * both from consideration. */
           struct tile *ptile =
               startpos_tile(startpos_list_link_data(choice));
 
@@ -277,14 +497,16 @@ void init_new_game(void)
           startpos_list_erase(targeted_list, choice);
           players_to_place--;
           removed = TRUE;
-          log_verbose("Start position (%d, %d) matches player %s (%s).",
+          log_verbose("Start position (%d, %d) exactly matches player %s (%s).",
                       TILE_XY(ptile), player_name(pplayer),
                       nation_rule_name(pnation));
         }
       } players_iterate_end;
 
       if (!removed) {
-        /* Make arbitrary choice for a player. */
+        /* Didn't find any 1:1 matches. For the next restricted start
+         * position, assign a random matching player. (This may create
+         * restrictions such that more 1:1 matches are possible.) */
         struct startpos *psp = startpos_list_back(targeted_list);
         struct tile *ptile = startpos_tile(psp);
         struct player *rand_plr = NULL;
@@ -310,7 +532,9 @@ void init_new_game(void)
                       TILE_XY(ptile), player_name(rand_plr),
                       nation_rule_name(nation_of_player(rand_plr)));
         } else {
-          /* This start position cannot be assigned. */
+          /* This start position cannot be assigned, given the assignments
+           * made so far. We may have to fall back to mismatched
+           * assignments. */
           log_verbose("Start position (%d, %d) cannot be assigned for "
                       "any player, keeping for the moment...",
                       TILE_XY(ptile));
@@ -321,11 +545,161 @@ void init_new_game(void)
     } while (0 < players_to_place && 0 < startpos_list_size(targeted_list));
   }
 
-  /* Now assign left start positions to every players. */
+  /* Now try to assign with regard to the 'teamplacement' setting. */
+  if (players_to_place > 0
+      && map.server.team_placement != TEAM_PLACEMENT_DISABLED
+      && player_count() > team_count()) {
+    const struct player_list *members;
+    int team_placement_players_to_place = 0;
+    int real_team_count = 0;
+
+    teams_iterate(pteam) {
+      members = team_members(pteam);
+      fc_assert(0 < player_list_size(members));
+      real_team_count++;
+      if (player_list_size(members) == 1) {
+        /* Single player teams, doesn't count for team placement. */
+        continue;
+      }
+      player_list_iterate(members, pplayer) {
+        if (player_startpos[player_index(pplayer)] == NULL) {
+          team_placement_players_to_place++;
+        }
+      } player_list_iterate_end;
+    } teams_iterate_end;
+
+    if (real_team_count > 1 && team_placement_players_to_place > 0) {
+      /* We really can do something to improve team placement. */
+      struct team_placement_config config;
+      struct team_placement_state state;
+      int i, j, t;
+
+      log_verbose("Do team placement for %d players, using %s variant.",
+                  team_placement_players_to_place,
+                  team_placement_name(map.server.team_placement));
+
+      /* Initialize configuration. */
+      config.flexible_startpos_num = startpos_list_size(flexible_list);
+      config.usable_startpos_num = config.flexible_startpos_num;
+      if (config.flexible_startpos_num < team_placement_players_to_place) {
+        config.usable_startpos_num += startpos_list_size(impossible_list);
+      }
+      config.total_startpos_num = (config.usable_startpos_num
+                                   + player_count() - players_to_place);
+      config.startpos = fc_malloc(sizeof(*config.startpos)
+                                  * config.total_startpos_num);
+      i = 0;
+      startpos_list_iterate(flexible_list, plink, psp) {
+        config.startpos[i++] = startpos_tile(psp);
+      } startpos_list_iterate_end;
+      fc_assert(i == config.flexible_startpos_num);
+      if (i < config.usable_startpos_num) {
+        startpos_list_iterate(impossible_list, plink, psp) {
+          config.startpos[i++] = startpos_tile(psp);
+        } startpos_list_iterate_end;
+      }
+      fc_assert(i == config.usable_startpos_num);
+      while (i < config.total_startpos_num) {
+        config.startpos[i++] = NULL;
+      }
+      fc_assert(i == config.total_startpos_num);
+
+      /* Initialize state. */
+      state.startpos = fc_malloc(sizeof(*state.startpos)
+                                 * config.total_startpos_num);
+      state.score = 0;
+      i = 0;
+      j = config.usable_startpos_num;
+      teams_iterate(pteam) {
+        members = team_members(pteam);
+        if (player_list_size(members) <= 1) {
+          /* Single player teams, doesn't count for team placement. */
+          continue;
+        }
+        t = team_number(pteam);
+        player_list_iterate(members, pplayer) {
+          struct tile *ptile = player_startpos[player_index(pplayer)];
+
+          if (ptile == NULL) {
+            state.startpos[i++] = t;
+          } else {
+            state.startpos[j] = t;
+            config.startpos[j] = ptile;
+            j++;
+          }
+        } player_list_iterate_end;
+      } teams_iterate_end;
+      while (i < config.usable_startpos_num) {
+        state.startpos[i++] = -1;
+      }
+      fc_assert(i == config.usable_startpos_num);
+      while (j < config.total_startpos_num) {
+        state.startpos[j++] = -1;
+      }
+      fc_assert(j == config.total_startpos_num);
+
+      /* Look for best team placement. */
+      do_team_placement(&config, &state, team_placement_players_to_place);
+
+      /* Apply result. */
+      for (i = 0; i < config.usable_startpos_num; i++) {
+        t = state.startpos[i];
+        if (t != -1) {
+          const struct team *pteam = team_by_number(t);
+          int candidate_index = -1;
+          int candidate_num = 0;
+
+          log_verbose("Start position (%d, %d) assigned to team %d (%s)",
+                      TILE_XY(config.startpos[i]),
+                      t, team_rule_name(pteam));
+
+          player_list_iterate(team_members(pteam), member) {
+            if (player_startpos[player_index(member)] == NULL
+                && fc_rand(++candidate_num) == 0) {
+              candidate_index = player_index(member);
+            }
+          } player_list_iterate_end;
+          fc_assert(candidate_index >= 0);
+          player_startpos[candidate_index] = config.startpos[i];
+          team_placement_players_to_place--;
+          players_to_place--;
+        }
+      }
+      fc_assert(team_placement_players_to_place == 0);
+
+      /* Free data. */
+      if (players_to_place > 0) {
+        /* We need to remove used startpos from the lists. */
+        i = 0;
+        startpos_list_iterate(flexible_list, plink, psp) {
+          fc_assert(config.startpos[i] == startpos_tile(psp));
+          if (state.startpos[i] != -1) {
+            startpos_list_erase(flexible_list, plink);
+          }
+          i++;
+        } startpos_list_iterate_end;
+        fc_assert(i == config.flexible_startpos_num);
+        if (i < config.usable_startpos_num) {
+          startpos_list_iterate(impossible_list, plink, psp) {
+            fc_assert(config.startpos[i] == startpos_tile(psp));
+            if (state.startpos[i] != -1) {
+              startpos_list_erase(impossible_list, plink);
+            }
+            i++;
+          } startpos_list_iterate_end;
+        }
+        fc_assert(i == config.usable_startpos_num);
+      }
+
+      free(config.startpos);
+    }
+  }
+
+  /* Now assign unrestricted start positions to any remaining players. */
   if (0 < players_to_place && 0 < startpos_list_size(flexible_list)) {
     struct tile *ptile;
 
-    log_verbose("Assigning random start positions.");
+    log_verbose("Assigning unrestricted start positions.");
 
     startpos_list_shuffle(flexible_list); /* Randomize. */
     players_iterate(pplayer) {
@@ -348,8 +722,15 @@ void init_new_game(void)
   }
 
   if (0 < players_to_place && 0 < startpos_list_size(impossible_list)) {
+    /* We still have players to place, and we have some restricted start
+     * positions whose nation requirements can't be satisfied given existing
+     * assignments. Fall back to making assignments ignoring the positions'
+     * nation requirements. */
+
     struct tile *ptile;
 
+    log_verbose("Ignoring nation restrictions on remaining start positions.");
+
     startpos_list_shuffle(impossible_list); /* Randomize. */
     players_iterate(pplayer) {
       if (NULL != player_startpos[player_index(pplayer)]) {
@@ -361,8 +742,8 @@ void init_new_game(void)
       player_startpos[player_index(pplayer)] = ptile;
       players_to_place--;
       startpos_list_pop_front(impossible_list);
-      log_verbose("Start position (%d, %d) assigned by default "
-                  "to player %s (%s).", TILE_XY(ptile), player_name(pplayer),
+      log_verbose("Start position (%d, %d) assigned to mismatched "
+                  "player %s (%s).", TILE_XY(ptile), player_name(pplayer),
                   nation_rule_name(nation_of_player(pplayer)));
       if (0 == startpos_list_size(impossible_list)) {
         break;
@@ -388,6 +769,12 @@ void init_new_game(void)
 
     fc_assert_action(NULL != ptile, continue);
 
+    /* Place first city */
+    if (game.server.start_city) {
+      create_city(pplayer, ptile, city_name_suggestion(pplayer, ptile),
+                  NULL);
+    }
+
     /* Place the first unit. */
     if (place_starting_unit(ptile, pplayer,
                             game.server.start_units[0]) != NULL) {
@@ -484,20 +871,18 @@ void send_game_info(struct conn_list *dest)
 
   /* Sometimes this function is called before the phase_timer is
    * initialized.  In that case we want to send the dummy value. */
-  if (game.info.timeout > 0 && game.server.phase_timer) {
+  if (current_turn_timeout() > 0 && game.server.phase_timer) {
     /* Whenever the client sees this packet, it starts a new timer at 0;
      * but the server's timer is only ever reset at the start of a phase
      * (and game.info.seconds_to_phasedone is relative to this).
      * Account for the difference. */
     ginfo.seconds_to_phasedone = game.info.seconds_to_phasedone
-        - read_timer_seconds(game.server.phase_timer);
+        - timer_read_seconds(game.server.phase_timer);
   } else {
     /* unused but at least initialized */
     ginfo.seconds_to_phasedone = -1.0;
   }
 
-  ginfo.trademindist_old = ginfo.trademindist_new;
-
   conn_list_iterate(dest, pconn) {
     send_packet_game_info(pconn, &ginfo);
   }
@@ -585,8 +970,8 @@ int update_timeout(void)
 **************************************************************************/
 void increase_timeout_because_unit_moved(void)
 {
-  if (game.info.timeout > 0 && game.server.timeoutaddenemymove > 0) {
-    double maxsec = (read_timer_seconds(game.server.phase_timer)
+  if (current_turn_timeout() > 0 && game.server.timeoutaddenemymove > 0) {
+    double maxsec = (timer_read_seconds(game.server.phase_timer)
 		     + (double) game.server.timeoutaddenemymove);
 
     if (maxsec > game.info.seconds_to_phasedone) {
