@@ -82,10 +82,9 @@ static void check_specials(const char *file, const char *function, int line)
     const struct terrain *pterrain = tile_terrain(ptile);
     bv_special special = tile_specials(ptile);
 
-    if (contains_special(special, S_RAILROAD))
-      SANITY_TILE(ptile, contains_special(special, S_ROAD));
-    if (contains_special(special, S_FARMLAND))
+    if (contains_special(special, S_FARMLAND)) {
       SANITY_TILE(ptile, contains_special(special, S_IRRIGATION));
+    }
 
     if (contains_special(special, S_MINE)) {
       SANITY_TILE(ptile, pterrain->mining_result == pterrain);
@@ -398,10 +397,14 @@ static void check_units(const char *file, const char *function, int line)
        * be modified. */
       if (ptrans != NULL) {
         struct unit *plevel = punit;
+        int level = 0;
 
         /* Make sure the transporter is on the tile. */
         SANITY_CHECK(same_pos(unit_tile(punit), unit_tile(ptrans)));
 
+        /* Can punit be cargo for its transporter? */
+        SANITY_CHECK(unit_transport_check(punit, ptrans));
+
         /* Check that the unit is listed as transported. */
         SANITY_CHECK(unit_list_search(unit_transport_cargo(ptrans),
                                       punit) != NULL);
@@ -411,15 +414,22 @@ static void check_units(const char *file, const char *function, int line)
           struct unit_list *pcargos = unit_transport_cargo(ptrans);
 
           SANITY_CHECK(pcargos != NULL);
+          SANITY_CHECK(level < GAME_TRANSPORT_MAX_RECURSIVE);
 
           /* Check for next level. */
           plevel = ptrans;
           ptrans = unit_transport_get(plevel);
+          level++;
         }
 
         /* Transporter capacity will be checked when transporter itself
          * is checked */
       }
+
+      /* Check that cargo is marked as transported with this unit */
+      unit_list_iterate(unit_transport_cargo(punit), pcargo) {
+        SANITY_CHECK(unit_transport_get(pcargo) == punit);
+      } unit_list_iterate_end;
     } unit_list_iterate_end;
   } players_iterate_end;
 }
@@ -455,15 +465,24 @@ static void check_players(const char *file, const char *function, int line)
     } city_list_iterate_end;
 
     players_iterate(pplayer2) {
-      struct player_diplstate *state1 = player_diplstate_get(pplayer, pplayer2);
-      struct player_diplstate *state2 = player_diplstate_get(pplayer2, pplayer);
+      struct player_diplstate *state1, *state2;
 
+      if (pplayer2 == pplayer) {
+        break; /* Do diplomatic sanity check only once per player couple. */
+      }
+
+      state1 = player_diplstate_get(pplayer, pplayer2);
+      state2 = player_diplstate_get(pplayer2, pplayer);
       SANITY_CHECK(state1->type == state2->type);
       if (state1->type == DS_CEASEFIRE) {
         SANITY_CHECK(state1->turns_left == state2->turns_left);
       }
       if (state1->type == DS_TEAM) {
         SANITY_CHECK(players_on_same_team(pplayer, pplayer2));
+        SANITY_CHECK(player_has_real_embassy(pplayer, pplayer2));
+        SANITY_CHECK(player_has_real_embassy(pplayer2, pplayer));
+        SANITY_CHECK(really_gives_vision(pplayer, pplayer2));
+        SANITY_CHECK(really_gives_vision(pplayer2, pplayer));
       }
       if (pplayer->is_alive
           && pplayer2->is_alive
