@@ -77,6 +77,7 @@
 #include "packets.h"
 
 /* server */
+#include "aiiface.h"
 #include "auth.h"
 #include "connecthand.h"
 #include "console.h"
@@ -118,13 +119,6 @@ static int socklan;
    void user_interrupt_callback();
 #endif
 
-#define SPECLIST_TAG timer
-#define SPECLIST_TYPE struct timer
-#include "speclist.h"
-#define timer_list_iterate(ARG_list, NAME_item) \
-  TYPED_LIST_ITERATE(struct timer, (ARG_list), NAME_item)
-#define timer_list_iterate_end LIST_ITERATE_END
-
 #define PROCESSING_TIME_STATISTICS 0
 
 static int server_accept_connection(int sockfd);
@@ -191,7 +185,7 @@ static void handle_readline_input_callback(char *line)
 
   con_prompt_enter();		/* just got an 'Enter' hit */
   line_internal = local_to_internal_string_malloc(line);
-  (void) handle_stdin_input(NULL, line_internal, FALSE);
+  (void) handle_stdin_input(NULL, line_internal);
   free(line_internal);
   free(line);
 
@@ -210,9 +204,6 @@ static void close_connection(struct connection *pconn)
   }
 
   if (pconn->server.ping_timers != NULL) {
-    timer_list_iterate(pconn->server.ping_timers, timer) {
-      free_timer(timer);
-    } timer_list_iterate_end;
     timer_list_destroy(pconn->server.ping_timers);
     pconn->server.ping_timers = NULL;
   }
@@ -254,6 +245,7 @@ void close_connections_and_socket(void)
   for (i = 0; i < listen_count; i++) {
     fc_closesocket(listen_socks[i]);
   }
+  FC_FREE(listen_socks);
 
   if (srvarg.announce != ANNOUNCE_NONE) {
     fc_closesocket(socklan);
@@ -263,6 +255,9 @@ void close_connections_and_socket(void)
   if (history_file) {
     write_history(history_file);
     history_truncate_file(history_file, HISTORY_LENGTH);
+    free(history_file);
+    history_file = NULL;
+    clear_history();
   }
 #endif
 
@@ -329,7 +324,7 @@ static void cut_lagging_connection(struct connection *pconn)
       && pconn->last_write
       && conn_list_size(game.all_connections) > 1
       && pconn->access_level != ALLOW_HACK
-      && read_timer_seconds(pconn->last_write) > game.server.tcptimeout) {
+      && timer_read_seconds(pconn->last_write) > game.server.tcptimeout) {
     /* Cut the connections to players who lag too much.  This
      * usually happens because client animation slows the client
      * too much and it can't keep up with the server.  We don't
@@ -419,11 +414,9 @@ struct packet_to_handle {
 static bool get_packet(struct connection *pconn, 
                        struct packet_to_handle *ppacket)
 {
-  bool got_packet;
+  ppacket->data = get_packet_from_connection(pconn, &ppacket->type);
 
-  ppacket->data = get_packet_from_connection(pconn, &ppacket->type, 
-                                             &got_packet);
-  return got_packet;
+  return NULL != ppacket->data;
 }
 
 /*****************************************************************************
@@ -444,7 +437,8 @@ static void incoming_client_packets(struct connection *pconn)
 #if PROCESSING_TIME_STATISTICS
     int request_id;
 
-    request_time = renew_timer_start(request_time, TIMER_USER, TIMER_ACTIVE);
+    request_time = timer_renew(request_time, TIMER_USER, TIMER_ACTIVE);
+    timer_start(request_time);
 #endif /* PROCESSING_TIME_STATISTICS */
 
     pconn->server.last_request_id_seen
@@ -465,7 +459,7 @@ static void incoming_client_packets(struct connection *pconn)
 
 #if PROCESSING_TIME_STATISTICS
     log_verbose("processed request %d in %gms", request_id, 
-                read_timer_seconds(request_time) * 1000.0);
+                timer_read_seconds(request_time) * 1000.0);
 #endif /* PROCESSING_TIME_STATISTICS */
 
     if (!command_ok) {
@@ -474,7 +468,7 @@ static void incoming_client_packets(struct connection *pconn)
   }
 
 #if PROCESSING_TIME_STATISTICS
-  free_timer(request_time);
+  timer_destroy(request_time);
 #endif /* PROCESSING_TIME_STATISTICS */
 }
 
@@ -603,7 +597,8 @@ enum server_events server_sniff_all_input(void)
       conn_list_iterate(game.all_connections, pconn) {
         if ((!pconn->server.is_closing
              && 0 < timer_list_size(pconn->server.ping_timers)
-	     && read_timer_seconds(timer_list_get(pconn->server.ping_timers, 0))
+	     && timer_read_seconds(timer_list_front
+                                   (pconn->server.ping_timers))
 	        > game.server.pingtimeout) 
             || pconn->ping_time > game.server.pingtimeout) {
           /* cut mute players, except for hack-level ones */
@@ -615,7 +610,10 @@ enum server_events server_sniff_all_input(void)
                         conn_description(pconn));
             connection_close_server(pconn, _("ping timeout"));
           }
-        } else {
+        } else if (pconn->established) {
+          /* We don't send ping to connection not established, because
+           * we wouldn't be able to handle asynchronous ping/pong with
+           * different packet header size. */
           connection_ping(pconn);
         }
       } conn_list_iterate_end;
@@ -633,6 +631,7 @@ enum server_events server_sniff_all_input(void)
 
     /* Don't wait if timeout == -1 (i.e. on auto games) */
     if (S_S_RUNNING == server_state() && game.info.timeout == -1) {
+      call_ai_refresh();
       (void) send_server_info_to_metaserver(META_REFRESH);
       return S_E_END_OF_TURN_TIMEOUT;
     }
@@ -685,11 +684,12 @@ enum server_events server_sniff_all_input(void)
 
     if (fc_select(max_desc + 1, &readfs, &writefs, &exceptfs, &tv) == 0) {
       /* timeout */
+      call_ai_refresh();
       (void) send_server_info_to_metaserver(META_REFRESH);
-      if (game.info.timeout > 0
+      if (current_turn_timeout() > 0
 	  && S_S_RUNNING == server_state()
 	  && game.server.phase_timer
-	  && (read_timer_seconds(game.server.phase_timer)
+	  && (timer_read_seconds(game.server.phase_timer)
 	      > game.info.seconds_to_phasedone)) {
 	con_prompt_off();
 	return S_E_END_OF_TURN_TIMEOUT;
@@ -781,7 +781,7 @@ enum server_events server_sniff_all_input(void)
       char *bufptr_internal = local_to_internal_string_malloc(bufptr);
 
       con_prompt_enter();	/* will need a new prompt, regardless */
-      handle_stdin_input(NULL, bufptr_internal, FALSE);
+      handle_stdin_input(NULL, bufptr_internal);
       free(bufptr_internal);
     }
 #else  /* !SOCKET_ZERO_ISNT_STDIN */
@@ -826,7 +826,7 @@ enum server_events server_sniff_all_input(void)
 
       if (didget >= 0) {
         buf_internal = local_to_internal_string_malloc(buffer);
-        handle_stdin_input(NULL, buf_internal, FALSE);
+        handle_stdin_input(NULL, buf_internal);
         free(buf_internal);
       }
       free(buffer);
@@ -877,10 +877,12 @@ enum server_events server_sniff_all_input(void)
   }
   con_prompt_off();
 
-  if (game.info.timeout > 0
+  call_ai_refresh();
+
+  if (current_turn_timeout() > 0
       && S_S_RUNNING == server_state()
       && game.server.phase_timer
-      && (read_timer_seconds(game.server.phase_timer)
+      && (timer_read_seconds(game.server.phase_timer)
           > game.info.seconds_to_phasedone)) {
     return S_E_END_OF_TURN_TIMEOUT;
   }
@@ -1006,6 +1008,7 @@ static int server_accept_connection(int sockfd)
 ********************************************************************/
 int server_make_connection(int new_sock, const char *client_addr, const char *client_ip)
 {
+  struct timer *timer;
   int i;
 
   fc_nonblock(new_sock);
@@ -1025,7 +1028,7 @@ int server_make_connection(int new_sock, const char *client_addr, const char *cl
       pconn->server.auth_tries = 0;
       pconn->server.auth_settime = 0;
       pconn->server.status = AS_NOT_ESTABLISHED;
-      pconn->server.ping_timers = timer_list_new();
+      pconn->server.ping_timers = timer_list_new_full(timer_destroy);
       pconn->server.granted_access_level = pconn->access_level;
       pconn->server.ignore_list =
           conn_pattern_list_new_full(conn_pattern_destroy);
@@ -1042,7 +1045,12 @@ int server_make_connection(int new_sock, const char *client_addr, const char *cl
 
       log_verbose("connection (%s) from %s (%s)", 
                   pconn->username, pconn->addr, pconn->server.ipaddr);
-      connection_ping(pconn);
+      /* Give a ping timeout to send the PACKET_SERVER_JOIN_REQ, or close
+       * the mute connection. This timer will be canceled into
+       * connecthand.c:handle_login_request(). */
+      timer = timer_new(TIMER_USER, TIMER_ACTIVE);
+      timer_start(timer);
+      timer_list_append(pconn->server.ping_timers, timer);
       return 0;
     }
   }
@@ -1076,11 +1084,8 @@ int server_open_socket(void)
               srvarg.bind_addr ? srvarg.bind_addr : "(any)",
               srvarg.port);
 
-#ifdef IPV6_SUPPORT
+  /* Any supported family will do */
   list = net_lookup_service(srvarg.bind_addr, srvarg.port, FC_ADDR_ANY);
-#else  /* IPV6_SUPPORT */
-  list = net_lookup_service(srvarg.bind_addr, srvarg.port, FC_ADDR_IPV4);
-#endif /* IPV6_SUPPORT */
 
   name_count = fc_sockaddr_list_size(list);
 
@@ -1109,12 +1114,15 @@ int server_open_socket(void)
       continue;
     }
 
+#ifndef HAVE_WINSOCK
+    /* SO_REUSEADDR considered harmful on Win, necessary otherwise */
     if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, 
                    (char *)&on, sizeof(on)) == -1) {
       log_error("setsockopt SO_REUSEADDR failed: %s",
                 fc_strerror(fc_get_errno()));
       sockaddr_debug(paddr);
     }
+#endif /* HAVE_WINSOCK */
 
     /* AF_INET6 sockets should use IPv6 only,
      * without stealing IPv4 from AF_INET sockets. */
@@ -1236,7 +1244,7 @@ int server_open_socket(void)
   if (addr.saddr.sa_family == AF_INET) {
 #ifdef HAVE_INET_ATON
     inet_aton(group, &mreq4.imr_multiaddr);
-#else  /* HEVE_INET_ATON */
+#else  /* HAVE_INET_ATON */
     mreq4.imr_multiaddr.s_addr = inet_addr(group);
 #endif /* HAVE_INET_ATON */
 #else  /* IPv6 support */
@@ -1331,10 +1339,12 @@ static void finish_processing_request(struct connection *pconn)
 ****************************************************************************/
 static void connection_ping(struct connection *pconn)
 {
+  struct timer *timer = timer_new(TIMER_USER, TIMER_ACTIVE);
+
   log_debug("sending ping to %s (open=%d)", conn_description(pconn),
             timer_list_size(pconn->server.ping_timers));
-  timer_list_append(pconn->server.ping_timers,
-                    new_timer_start(TIMER_USER, TIMER_ACTIVE));
+  timer_start(timer);
+  timer_list_append(pconn->server.ping_timers, timer);
   send_packet_conn_ping(pconn);
 }
 
@@ -1350,10 +1360,9 @@ void handle_conn_pong(struct connection *pconn)
     return;
   }
 
-  timer = timer_list_get(pconn->server.ping_timers, 0);
-  timer_list_remove(pconn->server.ping_timers, timer);
-  pconn->ping_time = read_timer_seconds(timer);
-  free_timer(timer);
+  timer = timer_list_front(pconn->server.ping_timers);
+  pconn->ping_time = timer_read_seconds(timer);
+  timer_list_pop_front(pconn->server.ping_timers);
   log_debug("got pong from %s (open=%d); ping time = %fs",
             conn_description(pconn),
             timer_list_size(pconn->server.ping_timers), pconn->ping_time);
@@ -1492,7 +1501,9 @@ static void send_lanserver_response(void)
   }
 
   /* Create a description of server state to send to clients.  */
-  if (fc_gethostname(hostname, sizeof(hostname)) != 0) {
+  if (srvarg.identity_name[0] != '\0') {
+    sz_strlcpy(hostname, srvarg.identity_name);
+  } else if (fc_gethostname(hostname, sizeof(hostname)) != 0) {
     sz_strlcpy(hostname, "none");
   }
 
