@@ -15,6 +15,7 @@
 #endif
 
 /* utility */
+#include "astring.h"
 #include "fcintl.h"
 #include "log.h"
 #include "mem.h"
@@ -52,7 +53,6 @@
 #undef TECH_UPKEEP_DEBUGGING
 
 static Tech_type_id pick_random_tech_to_lose(struct player* plr);
-static Tech_type_id pick_random_tech(struct player* plr);
 static void player_tech_lost(struct player* plr, Tech_type_id tech);
 static void forget_tech_transfered(struct player *pplayer, Tech_type_id tech);
 
@@ -64,7 +64,8 @@ void do_dipl_cost(struct player *pplayer, Tech_type_id tech)
   struct player_research * research = player_research_get(pplayer);
 
   research->bulbs_researched
-    -= (base_total_bulbs_required(pplayer, tech) * game.server.diplcost) / 100;
+    -= (base_total_bulbs_required(pplayer, tech, FALSE)
+        * game.server.diplcost) / 100;
   research->researching_saved = A_UNKNOWN;
 }
 
@@ -76,7 +77,8 @@ void do_free_cost(struct player *pplayer, Tech_type_id tech)
   struct player_research * research = player_research_get(pplayer);
 
   research->bulbs_researched
-    -= (base_total_bulbs_required(pplayer, tech) * game.server.freecost) / 100;
+    -= (base_total_bulbs_required(pplayer, tech, FALSE)
+        * game.server.freecost) / 100;
   research->researching_saved = A_UNKNOWN;
 }
 
@@ -88,10 +90,36 @@ void do_conquer_cost(struct player *pplayer, Tech_type_id tech)
   struct player_research * research = player_research_get(pplayer);  
 
   research->bulbs_researched
-    -= (base_total_bulbs_required(pplayer, tech) * game.server.conquercost) / 100;
+    -= (base_total_bulbs_required(pplayer, tech, FALSE)
+        * game.server.conquercost) / 100;
   research->researching_saved = A_UNKNOWN;
 }
 
+/****************************************************************************
+  Emit script signal(s) for player/team learning new tech.
+****************************************************************************/
+void script_tech_learned(struct player *plr, struct advance *tech,
+                         const char *reason)
+{
+  /* Emit signal for individual player whose action triggered the
+   * tech first */
+  script_server_signal_emit("tech_researched", 3,
+                            API_TYPE_TECH_TYPE, tech,
+                            API_TYPE_PLAYER, plr,
+                            API_TYPE_STRING, reason);
+
+  /* Emit signal to remaining research teammates, if any */
+  players_iterate(aplayer) {
+    if (aplayer != plr
+        && player_research_get(plr) == player_research_get(aplayer)) {
+      script_server_signal_emit("tech_researched", 3,
+                                API_TYPE_TECH_TYPE, tech,
+                                API_TYPE_PLAYER, aplayer,
+                                API_TYPE_STRING, reason);
+    }
+  } players_iterate_end;
+}
+
 /****************************************************************************
   Player has researched a new technology
 ****************************************************************************/
@@ -124,11 +152,7 @@ static void tech_researched(struct player *plr)
   /* do all the updates needed after finding new tech */
   found_new_tech(plr, research->researching, TRUE, TRUE);
 
-  script_server_signal_emit("tech_researched", 3,
-                            API_TYPE_TECH_TYPE,
-                            advance_by_number(researched_tech),
-                            API_TYPE_PLAYER, plr,
-                            API_TYPE_STRING, "researched");
+  script_tech_learned(plr, advance_by_number(researched_tech), "researched");
 }
 
 /****************************************************************************
@@ -144,15 +168,9 @@ void do_tech_parasite_effect(struct player *pplayer)
    * much worse effect. */
   if ((mod = get_player_bonus_effects(plist, pplayer,
 				      EFT_TECH_PARASITE)) > 0) {
-    char buf[512];
+    struct astring effects = ASTRING_INIT;
 
-    buf[0] = '\0';
-    effect_list_iterate(plist, peffect) {
-      if (buf[0] != '\0') {
-	sz_strlcat(buf, ", ");
-      }
-      get_effect_req_text(peffect, buf, sizeof(buf));
-    } effect_list_iterate_end;
+    get_effect_list_req_text(plist, &effects);
 
     advance_index_iterate(A_FIRST, i) {
       if (player_invention_reachable(pplayer, i, FALSE)
@@ -168,25 +186,21 @@ void do_tech_parasite_effect(struct player *pplayer)
           notify_player(pplayer, NULL, E_TECH_GAIN, ftc_server,
                         _("%s acquired from %s!"),
                         advance_name_for_player(pplayer, i),
-                        buf);
+                        astr_str(&effects));
           notify_embassies(pplayer, NULL, NULL, E_TECH_GAIN, ftc_server,
                            _("The %s have acquired %s from %s."),
                            nation_plural_for_player(pplayer),
                            advance_name_for_player(pplayer, i),
-                           buf);
+                           astr_str(&effects));
 
 	  do_free_cost(pplayer, i);
 	  found_new_tech(pplayer, i, FALSE, TRUE);
-
-          script_server_signal_emit("tech_researched", 3,
-                                    API_TYPE_TECH_TYPE,
-                                    advance_by_number(i),
-                                    API_TYPE_PLAYER, pplayer,
-                                    API_TYPE_STRING, "stolen");
+          script_tech_learned(pplayer, advance_by_number(i), "stolen");
 	  break;
 	}
       }
     } advance_index_iterate_end;
+    astr_free(&effects);
   }
   effect_list_destroy(plist);
 }
@@ -204,13 +218,13 @@ static void update_player_after_tech_researched(struct player* plr,
   player_research_update(plr);
 
   remove_obsolete_buildings(plr);
-  
-  /* Give free rails in every city */
-  if (tech_found != A_FUTURE
-   && advance_has_flag(tech_found, TF_RAILROAD)) {
-    upgrade_city_rails(plr, was_discovery);  
+
+  /* Give free roads or bases in every city */
+  if (tech_found != A_FUTURE) {
+    upgrade_all_city_roads(plr, was_discovery);  
+    upgrade_all_city_bases(plr, was_discovery);
   }
-  
+
   /* Enhance vision of units if a player-ranged effect has changed.  Note
    * that world-ranged effects will not be updated immediately. */
   unit_list_refresh_vision(plr->units);
@@ -400,8 +414,6 @@ void found_new_tech(struct player *plr, Tech_type_id tech_found,
   }
 
   if (bonus_tech_hack) {
-    int saved_bulbs;
-
     if (advance_by_number(tech_found)->bonus_message) {
       notify_research(plr, E_TECH_GAIN, ftc_server,
                       "%s", _(advance_by_number(tech_found)->bonus_message));
@@ -411,17 +423,8 @@ void found_new_tech(struct player *plr, Tech_type_id tech_found,
                         "world join your civilization: you get "
                         "an immediate advance."));
     }
-    
-    if (research->researching == A_UNSET) {
-      choose_random_tech(plr);
-    }
-    do_free_cost(plr, research->researching);
-    /* Save current number of bulbs so we can ignore changes made
-     * by tech_researched(). This is supposed to be free tech, not
-     * something having full cost. */
-    saved_bulbs = research->bulbs_researched;
-    tech_researched(plr);
-    research->bulbs_researched = saved_bulbs;
+
+    give_immediate_free_tech(plr);
   }
 
   /*
@@ -436,9 +439,9 @@ void found_new_tech(struct player *plr, Tech_type_id tech_found,
    * techs aren't researched that often.
    */
   cities_iterate(pcity) {
-    /* update squared city radius */
-    city_map_update_radius_sq(pcity, TRUE);
+    /* Refresh the city data; this als updates the squared city radius. */
     city_refresh(pcity);
+    city_refresh_vision(pcity);
     send_city_info(city_owner(pcity), pcity);
   } cities_iterate_end;
   
@@ -454,6 +457,38 @@ void found_new_tech(struct player *plr, Tech_type_id tech_found,
       } players_iterate_end;
     }
   } players_iterate_end;
+
+  conn_list_iterate(plr->connections, pconn) {
+    dsend_packet_tech_gained(pconn, tech_found);
+  } conn_list_iterate_end;
+}
+
+/****************************************************************************
+  Is player about to lose tech?
+****************************************************************************/
+static bool lose_tech(struct player *plr)
+{
+  struct player_research *research;
+
+  if (game.server.techloss_forgiveness < 0) {
+    /* Tech loss disabled */
+    return FALSE;
+  }
+
+  research = player_research_get(plr);
+
+  if (research->techs_researched == 0) {
+    /* No tech to lose */
+    fc_assert(research->future_tech == 0);
+    return FALSE;
+  }
+
+  if (research->bulbs_researched <
+      -total_bulbs_required(plr) * game.server.techloss_forgiveness / 100) {
+    return TRUE;
+  }
+
+  return FALSE;
 }
 
 /****************************************************************************
@@ -470,6 +505,11 @@ bool update_bulbs(struct player *plr, int bulbs, bool check_tech)
 {
   struct player_research *research = player_research_get(plr);
 
+  if (!plr->is_alive) {
+    /* Dead players do not produce research */
+    return FALSE;
+  }
+
   /* count our research contribution this turn */
   plr->server.bulbs_last_turn += bulbs;
   research->bulbs_researched += bulbs;
@@ -477,18 +517,22 @@ bool update_bulbs(struct player *plr, int bulbs, bool check_tech)
   /* if we have a negative number of bulbs we do
    * - try to reduce the number of future techs
    * - or lose one random tech
-   * after that the number of bulbs available is set to zero */
-  if (game.info.tech_upkeep_style > 0 && research->bulbs_researched < 0
-      && (research->techs_researched > 0 || research->future_tech > 0)) {
+   * after that the number of bulbs available is incresed based on the value
+   * of the lost tech. */
+  if (lose_tech(plr)) {
+    Tech_type_id tech;
+
     if (research->future_tech > 0) {
       notify_player(plr, NULL, E_TECH_GAIN, ftc_server,
                     _("Insufficient science output. We lost Future Tech. %d."),
                     research->future_tech);
       log_debug("%s: tech loss (future tech %d)", player_name(plr),
                 research->future_tech);
+
+      tech = A_FUTURE;
       research->future_tech--;
     } else {
-      Tech_type_id tech = pick_random_tech_to_lose(plr);
+      tech = pick_random_tech_to_lose(plr);
 
       if (tech != A_NONE) {
         notify_player(plr, NULL, E_TECH_GAIN, ftc_server,
@@ -496,12 +540,21 @@ bool update_bulbs(struct player *plr, int bulbs, bool check_tech)
                       advance_name_for_player(plr, tech));
         log_debug("%s: tech loss (%s)", player_name(plr),
                   advance_name_for_player(plr, tech));
+
         player_tech_lost(plr, tech);
       }
     }
-    player_research_update(plr);
 
-    research->bulbs_researched = 0;
+    if (tech != A_NONE) {
+      if (game.server.techloss_restore >= 0) {
+        research->bulbs_researched += base_total_bulbs_required(plr, tech, TRUE)
+          * game.server.techloss_restore / 100;
+      } else {
+        research->bulbs_researched = 0;
+      }
+    }
+
+    player_research_update(plr);
   }
 
   if (check_tech && research->researching != A_UNSET) {
@@ -526,16 +579,28 @@ bool update_bulbs(struct player *plr, int bulbs, bool check_tech)
 static Tech_type_id pick_random_tech_to_lose(struct player* plr)
 {
   bv_techs eligible_techs;
-  int chosen, eligible = 0;
+  /* A_NONE included in advance_count(). */
+  int eligible = advance_count() - 1;
+  int chosen;
 
-  BV_CLR_ALL(eligible_techs);
+  BV_SET_ALL(eligible_techs);
 
   advance_index_iterate(A_FIRST, i) {
-    /* Never lose self root_req techs */
-    if (advance_required(i, AR_ROOT) != i
-        && player_invention_state(plr, i) == TECH_KNOWN) {
-      BV_SET(eligible_techs, i);
-      eligible++;
+    if (player_invention_state(plr, i) != TECH_KNOWN) {
+      if (BV_ISSET(eligible_techs, i)) {
+        eligible--;
+        BV_CLR(eligible_techs, i);
+      }
+    } else {
+      /* Never lose techs that are root_req for a currently known tech
+       * (including self root_req) */
+      Tech_type_id root = advance_required(i, AR_ROOT);
+      if (root != A_NONE) {
+        if (BV_ISSET(eligible_techs, root)) {
+          eligible--;
+          BV_CLR(eligible_techs, root);
+        }
+      }
     }
   } advance_index_iterate_end;
 
@@ -636,19 +701,15 @@ static void player_tech_lost(struct player* plr, Tech_type_id tech)
     }
 
     /* Check all units for valid activities. */
-    if (advance_has_flag(tech, TF_BRIDGE)
-        || advance_has_flag(tech, TF_RAILROAD)
-        || advance_has_flag(tech, TF_FARMLAND)) {
-      unit_list_iterate(pplayer->units, punit) {
-        if (!can_unit_continue_current_activity(punit)) {
-          log_debug("lost technology for activity of unit %s of %s (%d, %d)",
-                    unit_name_translation(punit), player_name(pplayer),
-                    TILE_XY(unit_tile(punit)));
-          set_unit_activity(punit, ACTIVITY_IDLE);
-          send_unit_info(NULL, punit);
-        }
-      } unit_list_iterate_end;
-    }
+    unit_list_iterate(pplayer->units, punit) {
+      if (!can_unit_continue_current_activity(punit)) {
+        log_debug("lost technology for activity of unit %s of %s (%d, %d)",
+                  unit_name_translation(punit), player_name(pplayer),
+                  TILE_XY(unit_tile(punit)));
+        set_unit_activity(punit, ACTIVITY_IDLE);
+        send_unit_info(NULL, punit);
+      }
+    } unit_list_iterate_end;
 
     /* Check city production */
     city_list_iterate(pplayer->cities, pcity) {
@@ -698,7 +759,7 @@ static void player_tech_lost(struct player* plr, Tech_type_id tech)
   Returns random researchable tech or A_FUTURE.
   No side effects
 ****************************************************************************/
-static Tech_type_id pick_random_tech(struct player* plr) 
+Tech_type_id pick_random_tech(struct player* plr) 
 {
   int chosen, researchable = 0;
 
@@ -724,6 +785,54 @@ static Tech_type_id pick_random_tech(struct player* plr)
   return A_FUTURE;
 }
 
+/****************************************************************************
+  Returns cheapest researchable tech, random among equal cost ones.
+****************************************************************************/
+Tech_type_id pick_cheapest_tech(struct player* plr)
+{
+  int cheapest_cost = -1;
+  int cheapest_amount = 0;
+  Tech_type_id cheapest = A_NONE;
+  int chosen;
+
+  advance_index_iterate(A_FIRST, i) {
+    if (player_invention_state(plr, i) == TECH_PREREQS_KNOWN) {
+      int cost = base_total_bulbs_required(plr, i, FALSE);
+
+      if (cost < cheapest_cost || cheapest_cost == -1) {
+        cheapest_cost = cost;
+        cheapest_amount = 1;
+        cheapest = i;
+      } else if (cost == cheapest_cost) {
+        cheapest_amount++;
+      }
+    }
+  } advance_index_iterate_end;
+  if (cheapest_cost == -1) {
+    return A_FUTURE;
+  }
+  if (cheapest_amount == 1) {
+    /* No need to get random one among the 1 cheapest */
+    return cheapest;
+  }
+
+  chosen = fc_rand(cheapest_amount) + 1;
+
+  advance_index_iterate(A_FIRST, i) {
+    if (player_invention_state(plr, i) == TECH_PREREQS_KNOWN
+        && base_total_bulbs_required(plr, i, FALSE) == cheapest_cost) {
+      chosen--;
+      if (chosen == 0) {
+        return i;
+      }
+    }
+  } advance_index_iterate_end;
+
+  fc_assert(FALSE);
+
+  return A_NONE;
+}
+
 /****************************************************************************
   Finds and chooses (sets) a random research target from among all those
   available until plr->research->researching != A_UNSET.
@@ -830,7 +939,7 @@ void init_tech(struct player *plr, bool update)
       log_debug("[player %d] %-25s (ID: %3d) cost: %6d - reachable: %-3s "
                 "(now) / %-3s (ever)", player_number(plr),
                 advance_rule_name(advance_by_number(i)), i,
-                base_total_bulbs_required(plr, i),
+                base_total_bulbs_required(plr, i, FALSE),
                 player_invention_reachable(plr, i, FALSE) ? "yes" : "no",
                 player_invention_reachable(plr, i, TRUE) ? "yes" : "no");
     } advance_index_iterate_end;
@@ -950,10 +1059,14 @@ Tech_type_id give_random_initial_tech(struct player *pplayer)
   Returns the stolen tech or A_NONE if no tech was found.
 ****************************************************************************/
 Tech_type_id steal_a_tech(struct player *pplayer, struct player *victim,
-            	        Tech_type_id preferred)
+                          Tech_type_id preferred)
 {
   Tech_type_id stolen_tech = A_NONE;
-  
+
+  if (get_player_bonus(victim, EFT_NOT_TECH_SOURCE) > 0) {
+    return A_NONE;
+  }
+
   if (preferred == A_UNSET) {
     int j = 0;
     advance_index_iterate(A_FIRST, i) {
@@ -1020,11 +1133,7 @@ Tech_type_id steal_a_tech(struct player *pplayer, struct player *victim,
     do_conquer_cost(pplayer, stolen_tech);
     found_new_tech(pplayer, stolen_tech, FALSE, TRUE);
 
-    script_server_signal_emit("tech_researched", 3,
-                              API_TYPE_TECH_TYPE,
-                              advance_by_number(stolen_tech),
-                              API_TYPE_PLAYER, pplayer,
-                              API_TYPE_STRING, "stolen");
+    script_tech_learned(pplayer, advance_by_number(stolen_tech), "stolen");
 
     return stolen_tech;
   };
@@ -1100,7 +1209,7 @@ void handle_player_tech_goal(struct player *pplayer, int tech_goal)
 Tech_type_id give_random_free_tech(struct player* pplayer)
 {
   Tech_type_id tech;
-  
+
   tech = pick_random_tech(pplayer);
   do_free_cost(pplayer, tech);
   found_new_tech(pplayer, tech, FALSE, TRUE);
@@ -1113,10 +1222,15 @@ Tech_type_id give_random_free_tech(struct player* pplayer)
 Tech_type_id give_immediate_free_tech(struct player* pplayer)
 {
   Tech_type_id tech;
-  if (player_research_get(pplayer)->researching == A_UNSET) {
+
+  if (game.info.free_tech_method == FTM_CHEAPEST) {
+    tech = pick_cheapest_tech(pplayer);
+  } else if (player_research_get(pplayer)->researching == A_UNSET
+      || game.info.free_tech_method == FTM_RANDOM) {
     return give_random_free_tech(pplayer);
+  } else {
+    tech = player_research_get(pplayer)->researching;
   }
-  tech = player_research_get(pplayer)->researching;
   do_free_cost(pplayer, tech);
   found_new_tech(pplayer, tech, FALSE, TRUE);
   return tech;
@@ -1141,8 +1255,20 @@ static void forget_tech_transfered(struct player *pplayer, Tech_type_id tech)
 bool tech_transfer(struct player *plr_recv, struct player *plr_donor,
                    Tech_type_id tech)
 {
-  if (fc_rand(100) < game.server.techlost_donor) {
-    forget_tech_transfered(plr_donor, tech);
+  if (game.server.techlost_donor > 0) {
+    /* Don't let donor lose tech if it's root_req for some other known
+     * tech */
+    bool donor_can_lose = TRUE;
+    advance_index_iterate(A_FIRST, i) {
+      if (player_invention_state(plr_donor, i) == TECH_KNOWN
+          && advance_required(i, AR_ROOT) == tech) {
+        donor_can_lose = FALSE;
+        break;
+      }
+    } advance_index_iterate_end;
+    if (donor_can_lose && fc_rand(100) < game.server.techlost_donor) {
+      forget_tech_transfered(plr_donor, tech);
+    }
   }
 
   if (fc_rand(100) < game.server.techlost_recv) {
