@@ -18,6 +18,7 @@
 #include <stdlib.h>
 
 /* utility */
+#include "fcthread.h"
 #include "log.h"
 #include "mem.h"
 #include "shared.h"  /* array_shuffle */
@@ -36,35 +37,12 @@ struct genlist_link {
  * of the list. */
 struct genlist {
   int nelements;
+  fc_mutex mutex;
   struct genlist_link *head_link;
   struct genlist_link *tail_link;
   genlist_free_fn_t free_data_func;
 };
 
-/****************************************************************************
-  Default function of type genlist_free_fn_t.
-****************************************************************************/
-static void genlist_default_free_data_func(void *data)
-{
-  /* Do nothing. */
-}
-
-/****************************************************************************
-  Default function of type genlist_copy_fn_t.
-****************************************************************************/
-static void *genlist_default_copy_data_func(const void *data)
-{
-  return (void *) data; /* Returns the data. */
-}
-
-/****************************************************************************
-  Default function of type genlist_comp_fn_t.
-****************************************************************************/
-static bool genlist_default_comp_data_func(const void *data1,
-                                           const void *data2)
-{
-  return data1 == data2;
-}
 
 /****************************************************************************
   Create a new empty genlist.
@@ -86,8 +64,8 @@ struct genlist *genlist_new_full(genlist_free_fn_t free_data_func)
   pgenlist->head_link = NULL;
   pgenlist->tail_link = NULL;
 #endif /* ZERO_VARIABLES_FOR_SEARCHING */
-  pgenlist->free_data_func = (free_data_func ? free_data_func
-                              : genlist_default_free_data_func);
+  fc_init_mutex(&pgenlist->mutex);
+  pgenlist->free_data_func = free_data_func;
 
   return pgenlist;
 }
@@ -97,8 +75,12 @@ struct genlist *genlist_new_full(genlist_free_fn_t free_data_func)
 ****************************************************************************/
 void genlist_destroy(struct genlist *pgenlist)
 {
-  fc_assert_ret(NULL != pgenlist);
+  if (pgenlist == NULL) {
+    return;
+  }
+
   genlist_clear(pgenlist);
+  fc_destroy_mutex(&pgenlist->mutex);
   free(pgenlist);
 }
 
@@ -149,7 +131,9 @@ static void genlist_link_destroy(struct genlist *pgenlist,
 
   /* NB: detach the link before calling the free function for avoiding
    * re-entrant code. */
-  pgenlist->free_data_func(plink->dataptr);
+  if (NULL != pgenlist->free_data_func) {
+    pgenlist->free_data_func(plink->dataptr);
+  }
   free(plink);
 }
 
@@ -206,12 +190,15 @@ struct genlist *genlist_copy_full(const struct genlist *pgenlist,
   if (pgenlist) {
     struct genlist_link *plink;
 
-    if (NULL == copy_data_func) {
-      copy_data_func = genlist_default_copy_data_func;
-    }
-    for (plink = pgenlist->head_link; plink; plink = plink->next) {
-      genlist_link_new(pcopy, copy_data_func(plink->dataptr),
-                       pcopy->tail_link, NULL);
+    if (NULL != copy_data_func) {
+      for (plink = pgenlist->head_link; plink; plink = plink->next) {
+        genlist_link_new(pcopy, copy_data_func(plink->dataptr),
+                         pcopy->tail_link, NULL);
+      }
+    } else {
+      for (plink = pgenlist->head_link; plink; plink = plink->next) {
+        genlist_link_new(pcopy, plink->dataptr, pcopy->tail_link, NULL);
+      }
     }
   }
 
@@ -299,11 +286,18 @@ void genlist_clear(struct genlist *pgenlist)
 
     pgenlist->nelements = 0;
 
-    do {
-      plink2 = plink->next;
-      free_data_func(plink->dataptr);
-      free(plink);
-    } while ((plink = plink2) != NULL);
+    if (NULL != free_data_func) {
+      do {
+        plink2 = plink->next;
+        free_data_func(plink->dataptr);
+        free(plink);
+      } while (NULL != (plink = plink2));
+    } else {
+      do {
+        plink2 = plink->next;
+        free(plink);
+      } while (NULL != (plink = plink2));
+    }
   }
 }
 
@@ -329,17 +323,24 @@ void genlist_unique_full(struct genlist *pgenlist,
   if (2 <= pgenlist->nelements) {
     struct genlist_link *plink = pgenlist->head_link, *plink2;
 
-    if (NULL == comp_data_func) {
-      comp_data_func = genlist_default_comp_data_func;
+    if (NULL != comp_data_func) {
+      do {
+        plink2 = plink->next;
+        if (NULL != plink2 && comp_data_func(plink->dataptr,
+                                             plink2->dataptr)) {
+          /* Remove this element. */
+          genlist_link_destroy(pgenlist, plink);
+        }
+      } while ((plink = plink2) != NULL);
+    } else {
+      do {
+        plink2 = plink->next;
+        if (NULL != plink2 && plink->dataptr == plink2->dataptr) {
+          /* Remove this element. */
+          genlist_link_destroy(pgenlist, plink);
+        }
+      } while ((plink = plink2) != NULL);
     }
-    do {
-      plink2 = plink->next;
-      if (NULL != plink2 && comp_data_func(plink->dataptr,
-                                           plink2->dataptr)) {
-        /* Remove this element. */
-        genlist_link_destroy(pgenlist, plink);
-      }
-    } while ((plink = plink2) != NULL);
   }
 }
 
@@ -702,6 +703,22 @@ void genlist_reverse(struct genlist *pgenlist)
   }
 }
 
+/****************************************************************************
+  Allocates list mutex
+****************************************************************************/
+void genlist_allocate_mutex(struct genlist *pgenlist)
+{
+  fc_allocate_mutex(&pgenlist->mutex);
+}
+
+/****************************************************************************
+  Releases list mutex
+****************************************************************************/
+void genlist_release_mutex(struct genlist *pgenlist)
+{
+  fc_release_mutex(&pgenlist->mutex);
+}
+
 /****************************************************************************
   Returns the pointer of this link.
 ****************************************************************************/
