@@ -112,6 +112,15 @@ static char *grouping_sep = NULL;
 static const char base64url[] =
   "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_";
 
+static struct strvec *data_dir_names = NULL;
+static struct strvec *save_dir_names = NULL;
+static struct strvec *scenario_dir_names = NULL;
+
+static char *mc_group = NULL;
+static char *home_dir = NULL;
+
+static struct astring realfile = ASTRING_INIT;
+
 static int compare_file_mtime_ptrs(const struct fileinfo *const *ppa,
                                    const struct fileinfo *const *ppb);
 
@@ -587,7 +596,7 @@ char *skip_leading_spaces(char *s)
   Removes leading spaces in string pointed to by 's'.
   Note 's' must point to writeable memory!
 ***************************************************************************/
-static void remove_leading_spaces(char *s)
+void remove_leading_spaces(char *s)
 {
   char *t;
 
@@ -605,7 +614,7 @@ static void remove_leading_spaces(char *s)
   Terminates string pointed to by 's' to remove traling spaces;
   Note 's' must point to writeable memory!
 ***************************************************************************/
-static void remove_trailing_spaces(char *s)
+void remove_trailing_spaces(char *s)
 {
   char *t;
   size_t len;
@@ -737,12 +746,11 @@ char *user_home_dir(void)
   return "PROGDIR:";
 #else  /* AMIGA */
   static bool init = FALSE;
-  static char *home_dir = NULL;
 
   if (!init) {
     char *env = getenv("HOME");
     if (env) {
-      home_dir = fc_strdup(env);        /* never free()d */
+      home_dir = fc_strdup(env);
       log_verbose("HOME is %s", home_dir);
     } else {
 
@@ -800,6 +808,17 @@ char *user_home_dir(void)
 #endif /* AMIGA */
 }
 
+/***************************************************************************
+  Free user home directory information
+***************************************************************************/
+void free_user_home_dir(void)
+{
+  if (home_dir != NULL) {
+    free(home_dir);
+    home_dir = NULL;
+  }
+}
+
 /***************************************************************************
   Returns string which gives user's username, as specified by $USER or
   as given in password file for this user's uid, or a made up name if
@@ -931,6 +950,25 @@ static struct strvec *base_get_dirs(const char *dir_list)
   return dirs;
 }
 
+/***************************************************************************
+  Free data dir name vectors.
+***************************************************************************/
+void free_data_dir_names(void)
+{
+  if (data_dir_names != NULL) {
+    strvec_destroy(data_dir_names);
+    data_dir_names = NULL;
+  }
+  if (save_dir_names != NULL) {
+    strvec_destroy(save_dir_names);
+    save_dir_names = NULL;
+  }
+  if (scenario_dir_names != NULL) {
+    strvec_destroy(scenario_dir_names);
+    scenario_dir_names = NULL;
+  }
+}
+
 /***************************************************************************
   Returns a list of data directory paths, in the order in which they should
   be searched.  These paths are specified internally or may be set as the
@@ -945,12 +983,10 @@ static struct strvec *base_get_dirs(const char *dir_list)
 ***************************************************************************/
 const struct strvec *get_data_dirs(void)
 {
-  static struct strvec *dirs = NULL;
-
   /* The first time this function is called it will search and
    * allocate the directory listing.  Subsequently we will already
    * know the list and can just return it. */
-  if (NULL == dirs) {
+  if (NULL == data_dir_names) {
     const char *path;
 
     if ((path = getenv(FREECIV_DATA_PATH)) && '\0' == path[0]) {
@@ -966,14 +1002,14 @@ const struct strvec *get_data_dirs(void)
                 FREECIV_PATH, DEFAULT_DATA_PATH);
       path = NULL;
     }
-    dirs = base_get_dirs(NULL != path ? path : DEFAULT_DATA_PATH);
-    strvec_remove_duplicate(dirs, strcmp);      /* Don't set a path both. */
-    strvec_iterate(dirs, dirname) {
+    data_dir_names = base_get_dirs(NULL != path ? path : DEFAULT_DATA_PATH);
+    strvec_remove_duplicate(data_dir_names, strcmp); /* Don't set a path both. */
+    strvec_iterate(data_dir_names, dirname) {
       log_verbose("Data path component: %s", dirname);
     } strvec_iterate_end;
   }
 
-  return dirs;
+  return data_dir_names;
 }
 
 /***************************************************************************
@@ -990,12 +1026,10 @@ const struct strvec *get_data_dirs(void)
 ***************************************************************************/
 const struct strvec *get_save_dirs(void)
 {
-  static struct strvec *dirs = NULL;
-
   /* The first time this function is called it will search and
    * allocate the directory listing.  Subsequently we will already
    * know the list and can just return it. */
-  if (NULL == dirs) {
+  if (NULL == save_dir_names) {
     const char *path;
     bool from_freeciv_path = FALSE;
 
@@ -1016,25 +1050,25 @@ const struct strvec *get_save_dirs(void)
         from_freeciv_path = TRUE;
       }
     }
-    dirs = base_get_dirs(NULL != path ? path : DEFAULT_SAVE_PATH);
+    save_dir_names = base_get_dirs(NULL != path ? path : DEFAULT_SAVE_PATH);
     if (from_freeciv_path) {
       /* Then also append a "/saves" suffix to every directory. */
       char buf[512];
       size_t i;
 
-      for (i = 0; i < strvec_size(dirs); i++) {
-        path = strvec_get(dirs, i);
+      for (i = 0; i < strvec_size(save_dir_names); i++) {
+        path = strvec_get(save_dir_names, i);
         fc_snprintf(buf, sizeof(buf), "%s/saves", path);
-        strvec_insert(dirs, ++i, buf);
+        strvec_insert(save_dir_names, ++i, buf);
       }
     }
-    strvec_remove_duplicate(dirs, strcmp);      /* Don't set a path both. */
-    strvec_iterate(dirs, dirname) {
+    strvec_remove_duplicate(save_dir_names, strcmp); /* Don't set a path both. */
+    strvec_iterate(save_dir_names, dirname) {
       log_verbose("Save path component: %s", dirname);
     } strvec_iterate_end;
   }
 
-  return dirs;
+  return save_dir_names;
 }
 
 /***************************************************************************
@@ -1051,12 +1085,10 @@ const struct strvec *get_save_dirs(void)
 ***************************************************************************/
 const struct strvec *get_scenario_dirs(void)
 {
-  static struct strvec *dirs = NULL;
-
   /* The first time this function is called it will search and
    * allocate the directory listing.  Subsequently we will already
    * know the list and can just return it. */
-  if (NULL == dirs) {
+  if (NULL == scenario_dir_names) {
     const char *path;
     bool from_freeciv_path = FALSE;
 
@@ -1077,7 +1109,7 @@ const struct strvec *get_scenario_dirs(void)
         from_freeciv_path = TRUE;
       }
     }
-    dirs = base_get_dirs(NULL != path ? path : DEFAULT_SCENARIO_PATH);
+    scenario_dir_names = base_get_dirs(NULL != path ? path : DEFAULT_SCENARIO_PATH);
     if (from_freeciv_path) {
       /* Then also append subdirs every directory. */
       const char *subdirs[] = {
@@ -1087,21 +1119,21 @@ const struct strvec *get_scenario_dirs(void)
       const char **subdir;
       size_t i;
 
-      for (i = 0; i < strvec_size(dirs); i++) {
-        path = strvec_get(dirs, i);
+      for (i = 0; i < strvec_size(scenario_dir_names); i++) {
+        path = strvec_get(scenario_dir_names, i);
         for (subdir = subdirs; NULL != *subdir; subdir++) {
           fc_snprintf(buf, sizeof(buf), "%s/%s", path, *subdir);
-          strvec_insert(dirs, ++i, buf);
+          strvec_insert(scenario_dir_names, ++i, buf);
         }
       }
     }
-    strvec_remove_duplicate(dirs, strcmp);      /* Don't set a path both. */
-    strvec_iterate(dirs, dirname) {
+    strvec_remove_duplicate(scenario_dir_names, strcmp);      /* Don't set a path both. */
+    strvec_iterate(scenario_dir_names, dirname) {
       log_verbose("Scenario path component: %s", dirname);
     } strvec_iterate_end;
   }
 
-  return dirs;
+  return scenario_dir_names;
 }
 
 /***************************************************************************
@@ -1185,11 +1217,11 @@ struct strvec *fileinfolist(const struct strvec *dirs, const char *suffix)
   read-opened.)  The returned pointer points to static memory, so this
   function can only supply one filename at a time.  Don't free that
   pointer.
+
+  TODO: Make this re-entrant
 ***************************************************************************/
 const char *fileinfoname(const struct strvec *dirs, const char *filename)
 {
-  static struct astring realfile = ASTRING_INIT;
-
   if (NULL == dirs) {
     return NULL;
   }
@@ -1206,6 +1238,7 @@ const char *fileinfoname(const struct strvec *dirs, const char *filename)
         astr_add(&realfile, "%s", dirname);
       }
     } strvec_iterate_end;
+
     return astr_str(&realfile);
   }
 
@@ -1219,9 +1252,18 @@ const char *fileinfoname(const struct strvec *dirs, const char *filename)
   } strvec_iterate_end;
 
   log_verbose("Could not find readable file \"%s\" in data path.", filename);
+
   return NULL;
 }
 
+/**************************************************************************
+  Free resources allocated for fileinfoname service
+**************************************************************************/
+void free_fileinfo_data(void)
+{
+  astr_free(&realfile);
+}
+
 /**************************************************************************
   Destroys the file info structure.
 **************************************************************************/
@@ -1664,31 +1706,41 @@ enum m_pre_result match_prefix_full(m_pre_accessor_fn_t accessor_fn,
 ***************************************************************************/
 char *get_multicast_group(bool ipv6_prefered)
 {
-  static bool init = FALSE;
-  static char *group = NULL;
   static char *default_multicast_group_ipv4 = "225.1.1.1";
 #ifdef IPV6_SUPPORT
   /* TODO: Get useful group (this is node local) */
   static char *default_multicast_group_ipv6 = "FF31::8000:15B4";
-#endif
+#endif /* IPv6 support */
 
-  if (!init) {
+  if (mc_group == NULL) {
     char *env = getenv("FREECIV_MULTICAST_GROUP");
+
     if (env) {
-      group = fc_strdup(env);
+      mc_group = fc_strdup(env);
     } else {
 #ifdef IPV6_SUPPORT
       if (ipv6_prefered) {
-        group = fc_strdup(default_multicast_group_ipv6);
+        mc_group = fc_strdup(default_multicast_group_ipv6);
       } else
 #endif /* IPv6 support */
       {
-        group = fc_strdup(default_multicast_group_ipv4);
+        mc_group = fc_strdup(default_multicast_group_ipv4);
       }
     }
-    init = TRUE;
   }
-  return group;
+
+  return mc_group;
+}
+
+/***************************************************************************
+  Free multicast group resources
+***************************************************************************/
+void free_multicast_group(void)
+{
+  if (mc_group != NULL) {
+    free(mc_group);
+    mc_group = NULL;
+  }
 }
 
 /***************************************************************************
